@@ -47,12 +47,15 @@
 #endif
 #include <tclap/CmdLine.h>
 
+#include "DebugOut/HRConsoleOut.h"
 #include "../Tuvok/Controller/Controller.h"
 #include "../Tuvok/Basics/SysTools.h"
-#include "DebugOut/HRConsoleOut.h"
-#include "../Tuvok/IO/TuvokIOError.h"
-#include "../Tuvok/IO/IOManager.h"
+#include "../Tuvok/Basics/SystemInfo.h"
+#include "../Tuvok/IO/AbstrConverter.h"
+#include "../Tuvok/IO/AbstrGeoConverter.h"
 #include "../Tuvok/IO/DirectoryParser.h"
+#include "../Tuvok/IO/IOManager.h"
+#include "../Tuvok/IO/TuvokIOError.h"
 
 using namespace std;
 using namespace tuvok;
@@ -71,6 +74,8 @@ enum {
   EXIT_FAILURE_UNKNOWN_OUT,   // unknown file type for output file
   EXIT_FAILURE_RO_VOL_OUT,    // file known as volume but converter is read only
   EXIT_FAILURE_RO_GEO_OUT,    // file known as mesh but converter is read only
+  EXIT_FAILURE_RO_VOL_IN,    // file known as volume but converter is write only
+  EXIT_FAILURE_RO_GEO_IN,    // file known as mesh but converter is write only
   EXIT_FAILURE_UNKNOWN_1,     // unknown file type for first input file
   EXIT_FAILURE_UNKNOWN_2,     // unknown file type for second file in merge
   EXIT_FAILURE_CROSS_1,       // trying to convert a volume into a mesh
@@ -135,6 +140,8 @@ int main(int argc, const char* argv[])
   double fScale = 0.0;
   double fBias = 0.0;
   bool debug;
+  uint32_t bricksize = 64;
+  const uint32_t brickoverlap = 2;
 
   try {
     TCLAP::CmdLine cmd("uvf converter");
@@ -154,14 +161,21 @@ int main(int argc, const char* argv[])
     TCLAP::ValueArg<double> scale("s", "scale",
                                   "(merging) scaling value for second file",
                                   false, 0.0, "floating point number");
+    TCLAP::ValueArg<uint32_t> opt_bricksize("c", "bricksize",
+                                        "set maximum brick size (64)", false,
+                                        64, "positive integer");
     TCLAP::SwitchArg dbg("g", "debug", "Enable debugging mode", false);
+    TCLAP::SwitchArg experim("", "experimental",
+                             "Enable experimental features", false);
     
     cmd.xorAdd(inputs, directory);
     cmd.add(output);
     cmd.add(bias);
     cmd.add(scale);
+    cmd.add(opt_bricksize);
     cmd.add(expr);
     cmd.add(dbg);
+    cmd.add(experim);
     cmd.parse(argc, argv);
 
     // which of "-i" or "-d" did they give?
@@ -178,6 +192,7 @@ int main(int argc, const char* argv[])
     strOutFile = output.getValue();
     fBias = bias.getValue();
     fScale = scale.getValue();
+    bricksize = opt_bricksize.getValue();
 
     if(expr.isSet()) {
       expression = expr.getValue();
@@ -186,6 +201,7 @@ int main(int argc, const char* argv[])
       }
     }
     debug = dbg.getValue();
+    Controller::Instance().ExperimentalFeatures(experim.getValue());
   } catch(const TCLAP::ArgException& e) {
     std::cerr << "error: " << e.error() << " for arg " << e.argId() << "\n";
     return EXIT_FAILURE_ARG;
@@ -198,6 +214,11 @@ int main(int argc, const char* argv[])
   }
 
   Controller::Instance().AddDebugOut(debugOut);
+  Controller::Instance().SetMaxCPUMem(0.8f);
+  uint32_t mem = uint32_t(Controller::Instance().SysInfo()->GetMaxUsableCPUMem()/1024/1024);
+  MESSAGE("Using up to %u MB RAM", mem);
+  cout << endl;
+
   IOManager ioMan;
 
   // If they gave us an expression, evaluate that.  Otherwise we're doing a
@@ -228,8 +249,8 @@ int main(int argc, const char* argv[])
     for(std::vector<std::string>::const_iterator f = input.begin();
         f != input.end(); ++f) {
       std::string ext = SysTools::ToLowerCase(SysTools::GetExt(*f));
-      bool conv_vol = ioMan.GetConverterForExt(ext, false) != NULL;
-      bool conv_geo = ioMan.GetGeoConverterForExt(ext, false) != NULL;
+      bool conv_vol = ioMan.GetConverterForExt(ext, false, true) != NULL;
+      bool conv_geo = ioMan.GetGeoConverterForExt(ext, false, true) != NULL;
       if(conv_vol || conv_geo) { continue; }
 
       if(!conv_vol && !conv_geo) {
@@ -243,10 +264,10 @@ int main(int argc, const char* argv[])
   if (!strInFile.empty()) {
     string sourceType = SysTools::ToLowerCase(SysTools::GetExt(strInFile));
 
-    bool bIsVolExtOut = ioMan.GetConverterForExt(targetType, true) != NULL;
-    bool bIsGeoExtOut = ioMan.GetGeoConverterForExt(targetType, true) != NULL;
-    bool bIsVolExt1 = ioMan.GetConverterForExt(sourceType, false) != NULL;
-    bool bIsGeoExt1 = ioMan.GetGeoConverterForExt(sourceType, false) != NULL;
+    bool bIsVolExtOut = ioMan.GetConverterForExt(targetType, false, false) != NULL;
+    bool bIsGeoExtOut = ioMan.GetGeoConverterForExt(targetType, false, false) != NULL;
+    bool bIsVolExt1 = ioMan.GetConverterForExt(sourceType, false, false) != NULL;
+    bool bIsGeoExt1 = ioMan.GetGeoConverterForExt(sourceType, false, false) != NULL;
 
     if (!bIsVolExt1 && !bIsGeoExt1)  {
       std::cerr << "error: Unknown file type for '" << strInFile << "'\n";
@@ -263,6 +284,36 @@ int main(int argc, const char* argv[])
       return EXIT_FAILURE_CROSS_2;
     }
 
+    if (bIsGeoExt1 && bIsGeoExtOut)  {
+      AbstrGeoConverter* importer = ioMan.GetGeoConverterForExt(sourceType, false, false);
+      AbstrGeoConverter* exporter =  ioMan.GetGeoConverterForExt(targetType, false, false);
+
+      if (!importer->CanImportData()) {
+        std::cerr << "error: cannot read that type of geometry (only write)\n";
+        return EXIT_FAILURE_RO_GEO_IN;
+      }
+
+      if (!exporter->CanExportData()) {
+        std::cerr << "error: cannot write that type of geometry (only read)\n";
+        return EXIT_FAILURE_RO_GEO_OUT;
+      }
+    }
+
+    if (bIsVolExt1 && bIsVolExtOut)  {
+      AbstrConverter* importer = ioMan.GetConverterForExt(sourceType, false, false);
+      AbstrConverter* exporter =  ioMan.GetConverterForExt(targetType, false, false);
+
+      if (!importer->CanImportData()) {
+        std::cerr << "error: cannot read that type of volume (only write)\n";
+        return EXIT_FAILURE_RO_VOL_IN;
+      }
+
+      if (!exporter->CanExportData()) {
+        std::cerr << "error: cannot write that type of volume (only read)\n";
+        return EXIT_FAILURE_RO_VOL_OUT;
+      }
+    }
+
     if (strInFile2.empty()) {
       if (bIsVolExt1) {
         if (targetType == "uvf" && sourceType == "uvf") {
@@ -276,7 +327,8 @@ int main(int argc, const char* argv[])
 
           // HACK: use the output file's dir as temp dir
           if (ioMan.ConvertDataset(strInFile, tmpFile,
-                                   SysTools::GetPath(tmpFile))) {
+                                   SysTools::GetPath(tmpFile), true,
+                                   bricksize, brickoverlap)) {
             cout << endl << "Success." << endl << endl;
           } else {
             cout << endl << "Extraction failed!" << endl << endl;
@@ -286,7 +338,8 @@ int main(int argc, const char* argv[])
           cout << "Step 2. Writing new UVF file" << endl;
           // HACK: use the output file's dir as temp dir
           if (ioMan.ConvertDataset(tmpFile, strOutFile,
-                                   SysTools::GetPath(strOutFile))) {
+                                   SysTools::GetPath(strOutFile), true,
+                                   bricksize, brickoverlap)) {
             if(std::remove(tmpFile.c_str()) == -1) {
              cout << endl << "Conversion succeeded but "
                   << " could not delete tmp file " << tmpFile << "\n\n";
@@ -308,7 +361,8 @@ int main(int argc, const char* argv[])
                << strInFile << " to " << strOutFile << "\n\n";
           // HACK: use the output file's dir as temp dir
           if (ioMan.ConvertDataset(strInFile, strOutFile,
-                                   SysTools::GetPath(strOutFile))) {
+                                   SysTools::GetPath(strOutFile), true,
+                                   bricksize, brickoverlap)) {
             cout << "\nSuccess.\n\n";
             return EXIT_SUCCESS;
           } else {
@@ -317,14 +371,14 @@ int main(int argc, const char* argv[])
           }
         }
       } else {
-          AbstrGeoConverter* sourceConv = ioMan.GetGeoConverterForExt(sourceType, false);
-          AbstrGeoConverter* targetConv = ioMan.GetGeoConverterForExt(targetType, false);
+          AbstrGeoConverter* sourceConv = ioMan.GetGeoConverterForExt(sourceType, false, true);
+          AbstrGeoConverter* targetConv = ioMan.GetGeoConverterForExt(targetType, true, false);
 
           cout << "\nRunning in geometry file mode.\n"
                << "Converting " << strInFile
                << " (" << sourceConv->GetDesc() << ") to "
                << strOutFile << " (" << targetConv->GetDesc() << ")\n";
-          Mesh* m = NULL;
+          std::shared_ptr<Mesh> m;
           try {
             m = sourceConv->ConvertToMesh(strInFile);
           } catch (const tuvok::io::DSOpenFailed& err) {
@@ -336,14 +390,13 @@ int main(int argc, const char* argv[])
             cerr << "Error writing target mesh\n";
             return EXIT_FAILURE_OUT_MESH_WRITE;
           }
-          delete m;
       }
     } else {
 
       string sourceType2 = SysTools::ToLowerCase(SysTools::GetExt(strInFile2));
 
-      bool bIsVolExt2 = ioMan.GetConverterForExt(sourceType2, false) != NULL;
-      bool bIsGeoExt2 = ioMan.GetGeoConverterForExt(sourceType2, false) != NULL;
+      bool bIsVolExt2 = ioMan.GetConverterForExt(sourceType2, false, true) != NULL;
+      bool bIsGeoExt2 = ioMan.GetGeoConverterForExt(sourceType2, false, true) != NULL;
 
       if (!bIsVolExt2 && !bIsGeoExt2)  {
         std::cerr << "error: Unknown file type for '" << strInFile2 << "'\n";
@@ -400,7 +453,7 @@ int main(int argc, const char* argv[])
     cout << "\nRunning in directory mode.\nConverting "
          << strInDir << " to " << strOutFile << "\n\n";
 
-    vector<std::tr1::shared_ptr<FileStackInfo> > dirinfo =
+    vector<std::shared_ptr<FileStackInfo>> dirinfo =
       ioMan.ScanDirectory(strInDir);
 
     vector<string> vStrFilenames(dirinfo.size());
@@ -419,7 +472,8 @@ int main(int argc, const char* argv[])
     int iFailCount = 0;
     for (size_t i = 0;i<dirinfo.size();i++) {
       if (ioMan.ConvertDataset(&*dirinfo[i], vStrFilenames[i],
-                               SysTools::GetPath(vStrFilenames[i]))) {
+                               SysTools::GetPath(vStrFilenames[i]), true,
+                               bricksize, brickoverlap)) {
         cout << "\nSuccess.\n\n";
       } else {
         cout << "\nConversion failed!\n\n";
