@@ -60,13 +60,47 @@
 #include <string>
 #include "../Tuvok/Basics/SystemInfo.h"
 #include "../Tuvok/Basics/SysTools.h"
-#include "../Tuvok/IO/IOManager.h"
 #include "../Tuvok/Basics/MathTools.h"
 #include "../IO/DialogConverter.h"
 
+#include "LuaScripting/TuvokSpecific/LuaTuvokTypes.h"
+
 using namespace std;
 
-static const int IV3D_TIMER_INTERVAL = 20; // in milliseconds
+static FLOATMATRIX4 RotateX(float angle) {
+  FLOATMATRIX4 matRot;
+  matRot.RotationX(3.141592653589793238462643383*angle/180.0);
+  return matRot;
+}
+static FLOATMATRIX4 RotateY(float angle) {
+  FLOATMATRIX4 matRot;
+  matRot.RotationY(3.141592653589793238462643383*angle/180.0);
+  return matRot;
+}
+static FLOATMATRIX4 RotateZ(float angle) {
+  FLOATMATRIX4 matRot;
+  matRot.RotationZ(3.141592653589793238462643383*angle/180.0);
+  return matRot;
+}
+static FLOATMATRIX4 Translate(float x, float y, float z) {
+  FLOATMATRIX4 matTrans;
+  matTrans.Translation(x, y, z);
+  return matTrans;
+}
+static FLOATMATRIX4 Identity() {
+  FLOATMATRIX4 matIdent;
+  return matIdent;
+}
+std::string VecToString(const FLOATVECTOR3& v) {
+  std::ostringstream s;
+  s << "{ " << v.x << ", " << v.y << ", " << v.z << "}";
+  return s.str();
+}
+
+static FLOATMATRIX4 MulMatrices(const FLOATMATRIX4& a,
+                                const FLOATMATRIX4& b) {
+  return a * b;
+}
 
 MainWindow::MainWindow(MasterController& masterController,
            bool bScriptMode, /* = false */
@@ -81,7 +115,7 @@ MainWindow::MainWindow(MasterController& masterController,
   m_bQuickopen(true),
   m_iMinFramerate(10),
   m_iLODDelay(1000),
-  m_bUseAllMeans(false),
+  m_bRenderLowResIntermediateResults(false),
   m_iActiveTS(500),
   m_iInactiveTS(100),
   m_bWriteLogFile(false),
@@ -89,12 +123,13 @@ MainWindow::MainWindow(MasterController& masterController,
   m_iLogLevel(2),
   m_pWelcomeDialog(new WelcomeDialog(this, Qt::Tool)),
   m_pMetadataDialog(new MetadataDlg(this, Qt::Tool)),
+  //m_pDebugScriptWindow(new DebugScriptWindow(masterController, this)),
   m_iBlendPrecisionMode(0),
   m_bPowerOfTwo(true),
   m_bDownSampleTo8Bits(false),
   m_bDisableBorder(false),
-  m_bNoRCClipplanes(false),
   m_bI3MFeatures(false),
+  m_bAdvancedSettings(false),
   m_bAutoSaveGEO(true),
   m_bAutoSaveWSP(true),
   m_eVolumeRendererType(MasterController::OPENGL_SBVR),
@@ -110,6 +145,7 @@ MainWindow::MainWindow(MasterController& masterController,
   m_bStayOpenAfterScriptEnd(false),
   m_pTextout(NULL),
   m_pActiveRenderWin(NULL),
+  m_pLastLoadedRenderWin(NULL),
   m_pHttp(NULL),
   m_pUpdateFile(NULL),
   m_iHttpGetId(-1),
@@ -120,9 +156,11 @@ MainWindow::MainWindow(MasterController& masterController,
   m_bFTPDeleteSource(true),
   m_bFTPFinished(true),
   m_bClipDisplay(true),
-  m_bClipLocked(false)
+  m_bClipLocked(false),
+  m_bIgnoreLoadDatasetFailure(false),
+  m_MemReg(m_MasterController.LuaScript())
 {
-  RegisterCalls(m_MasterController.ScriptEngine());
+  RegisterLuaClasses();
 
   QCoreApplication::setOrganizationName("Scientific Computing and Imaging Institute, University of Utah");
   QCoreApplication::setOrganizationDomain(SCI_ORGANIZATION_DOMAIN);
@@ -137,6 +175,7 @@ MainWindow::MainWindow(MasterController& masterController,
 
   SysTools::GetTempDirectory(m_strTempDir);
 
+  ApplySettings();
   SetupWorkspaceMenu();
 
   if (!LoadDefaultGeometry()) SaveDefaultGeometry();
@@ -149,7 +188,10 @@ MainWindow::MainWindow(MasterController& masterController,
   this->show();
   SetAndCheckRunningFlag();
 
-  masterController.IOMan()->RegisterFinalConverter(new DialogConverter(this));
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript()); 
+  ss->cexec("tuvok.io.registerFinalConverter",
+      dynamic_pointer_cast<AbstrConverter>(shared_ptr<DialogConverter>(
+          new DialogConverter(this))));
 
   UpdateMRUActions();
   UpdateWSMRUActions();
@@ -174,6 +216,7 @@ MainWindow::MainWindow(MasterController& masterController,
 
   checkBox_ClipShow->setEnabled(false);
   checkBox_ClipLockObject->setEnabled(false);
+  toolButton_CropData->setEnabled(false);
   DisableStereoWidgets();
 
   m_pRedrawTimer = new QTimer(this);
@@ -192,7 +235,7 @@ MainWindow::~MainWindow()
     m_MasterController.RemoveDebugOut(m_pDebugOut);
   }
 
-  // cleanup updatefile, this codepath is taken for instance when the
+  // cleanup updatefile, this code path is taken for instance when the
   // windows firewall blocked an http request
   if (m_pUpdateFile && m_pUpdateFile->isOpen()) {
     m_pUpdateFile->close();
@@ -202,7 +245,6 @@ MainWindow::~MainWindow()
   }
 
   delete m_pHttp;
-  delete m_pFTPDialog;
   m_pRedrawTimer->stop();
   delete m_pRedrawTimer;
 
@@ -233,8 +275,8 @@ void MainWindow::StartTimer() {
 
 void MainWindow::SetAndCheckRunningFlag() {
   QSettings settings;
-  UINT32 iInstanceCounter = settings.value("InstanceCounter", 0).toUInt();
-  UINT32 iSaneCounter = settings.value("SaneCounter", 0).toUInt();
+  uint32_t iInstanceCounter = settings.value("InstanceCounter", 0).toUInt();
+  uint32_t iSaneCounter = settings.value("SaneCounter", 0).toUInt();
 
   settings.beginGroup("Performance");
   bool bWriteLogFile = settings.value("WriteLogFile", m_bWriteLogFile).toBool();
@@ -259,8 +301,8 @@ void MainWindow::SetAndCheckRunningFlag() {
             settings.setValue("Performance/WriteLogFile", true);
             settings.setValue("Performance/LogLevel", 2);
           } else {
-            // if debuging was not enabled assume that has been opended multiple times
-            settings.setValue("SaneCounter", iSaneCounter);
+            // if debugging was not enabled assume that has been opened multiple times
+            settings.setValue("SaneCounter", (unsigned int)iSaneCounter);
           }
         }
       }
@@ -281,16 +323,16 @@ void MainWindow::SetAndCheckRunningFlag() {
     }
   }
 
-  settings.setValue("InstanceCounter", iInstanceCounter+1);
+  settings.setValue("InstanceCounter", (unsigned int)iInstanceCounter+1);
 }
 
 void MainWindow::RemoveRunningFlag() {
   QSettings settings;
-//  UINT32 iInstanceCounter = settings.value("InstanceCounter", 1).toUInt();
+//  uint32_t iInstanceCounter = settings.value("InstanceCounter", 1).toUInt();
   settings.setValue("InstanceCounter", 0);
 
-  UINT32 iSaneCounter = settings.value("SaneCounter", 0).toUInt();
-  settings.setValue("SaneCounter", iSaneCounter+1);
+  uint32_t iSaneCounter = settings.value("SaneCounter", 0).toUInt();
+  settings.setValue("SaneCounter", (unsigned int)iSaneCounter+1);
 }
 
 // ******************************************
@@ -448,7 +490,7 @@ void MainWindow::SetSampleRate(int iValue) {
 
 void MainWindow::UpdateSampleRateLabel(int iValue) {
   QString desc;
-  desc = tr("Sampling Rate (%1%)").arg(iValue);
+  desc = tr("Sampling Rate (%1%):").arg(iValue);
   label_SamplingRate->setText(desc);
 }
 
@@ -457,10 +499,26 @@ void MainWindow::SetSampleRateSlider(int iValue) {
   UpdateSampleRateLabel(iValue);
 }
 
+void MainWindow::SetFoV(int iValue) {
+  if (m_pActiveRenderWin != NULL) m_pActiveRenderWin->SetFoV(iValue);
+  UpdateFoVLabel(iValue);
+}
+
+void MainWindow::SetFoVSlider(int iValue) {
+  horizontalSlider_FoV->setValue(iValue);
+  UpdateFoVLabel(iValue);
+}
+
+void MainWindow::UpdateFoVLabel(int iValue) {
+  QString desc;
+  desc = tr("Field of View (%1):").arg(iValue);
+  label_FoV->setText(desc);
+}
+
 // Script command to update isovalue.
 void MainWindow::SetIsoValue(float fValue) {
   if (m_pActiveRenderWin != NULL) {
-    int iMaxSize = int(m_pActiveRenderWin->GetDynamicRange().second)-1;
+    int iMaxSize = int(m_pActiveRenderWin->GetDynamicRange().second);
     fValue = MathTools::Clamp(fValue, 0.0f, 1.0f);
     m_pActiveRenderWin->SetIsoValue(fValue * iMaxSize);
     UpdateIsoValLabel(int(fValue * iMaxSize), iMaxSize);
@@ -471,7 +529,7 @@ void MainWindow::SetIsoValue(float fValue) {
 void MainWindow::SetIsoValue(int iValue) {
   if (m_pActiveRenderWin != NULL) {
     m_pActiveRenderWin->SetIsoValue(float(iValue));
-    int iMaxSize = int(m_pActiveRenderWin->GetDynamicRange().second-1);
+    int iMaxSize = int(m_pActiveRenderWin->GetDynamicRange().second);
     UpdateIsoValLabel(iValue, iMaxSize);
   }
 }
@@ -479,7 +537,7 @@ void MainWindow::SetIsoValue(int iValue) {
 // Script command to update focus isovalue
 void MainWindow::SetClearViewIsoValue(float fValue) {
   if (m_pActiveRenderWin != NULL) {
-    int iMaxSize = int(m_pActiveRenderWin->GetDynamicRange().second)-1;
+    int iMaxSize = int(m_pActiveRenderWin->GetDynamicRange().second);
     fValue = MathTools::Clamp(fValue, 0.0f, 1.0f);
     m_pActiveRenderWin->SetCVIsoValue(fValue * iMaxSize);
     UpdateIsoValLabel(int(fValue*iMaxSize), iMaxSize);
@@ -499,7 +557,7 @@ void MainWindow::UpdateIsoValLabel(int iValue, int iMaxValue) {
 
 
 void MainWindow::SetFocusIsoValue(int iValue) {
-  int iMaxSize = int(m_pActiveRenderWin->GetDynamicRange().second-1);
+  int iMaxSize = int(m_pActiveRenderWin->GetDynamicRange().second);
   if (m_pActiveRenderWin != NULL) {
     m_pActiveRenderWin->SetCVIsoValue(float(iValue));
   }
@@ -577,13 +635,22 @@ void MainWindow::ToggleClipPlane(bool bClip)
   if(rw == NULL) { return; }
 
   rw->SetClipPlaneEnabled(bClip);
-  if(bClip) {
-    checkBox_ClipShow->setEnabled(true);
-    checkBox_ClipLockObject->setEnabled(true);
-  } else {
-    checkBox_ClipShow->setEnabled(false);
-    checkBox_ClipLockObject->setEnabled(false);
-  }
+
+  // We can live without calling LuaCallbackToggleClipPlane explicitly,
+  // since SetClipPlaneEnabled will end up calling it for us.
+  //LuaCallbackToggleClipPlane(bClip);
+}
+
+void MainWindow::LuaCallbackToggleClipPlane(bool bClip)
+{
+  // Check the 'enable' box, but make sure we don't send any sort of event.
+  checkBox_ClipPlane->blockSignals(true);
+  checkBox_ClipPlane->setChecked(bClip);
+  checkBox_ClipPlane->blockSignals(false);
+
+  checkBox_ClipShow->setEnabled(bClip);
+  checkBox_ClipLockObject->setEnabled(bClip);
+  toolButton_CropData->setEnabled(bClip);
 }
 
 void MainWindow::ClipToggleShow(bool bShow)
@@ -610,9 +677,10 @@ void MainWindow::SetTimestep(int t)
   if(rw == NULL) { return; }
 
   rw->SetTimestep(static_cast<size_t>(t));
+  LuaClassInstance ds = m_pActiveRenderWin->GetRendererDataset();
   int iMaxVal = static_cast<int>(
-    m_pActiveRenderWin->GetRenderer()->GetDataset().GetNumberOfTimesteps()
-  );
+      m_MasterController.LuaScript()->cexecRet<uint64_t>(ds.fqName() + 
+                                                     ".getNumberOfTimesteps"));
   UpdateTimestepLabel(int(t), iMaxVal);
 }
 
@@ -815,7 +883,7 @@ void MainWindow::PickLightColor() {
       return;
 
     m_pQLightPreview->SetData(cAmbient,cDiffuse,cSpecular,vLightDir);
-    m_pActiveRenderWin->GetRenderer()->SetColors(cAmbient, cDiffuse,
+    m_pActiveRenderWin->SetLightColors(cAmbient, cDiffuse,
                                                  cSpecular, vLightDir);
   }
 }
@@ -829,8 +897,8 @@ void MainWindow::LightMoved() {
   FLOATVECTOR4 cSpecular = m_pQLightPreview->GetSpecular();
   FLOATVECTOR3 vLightDir = m_pQLightPreview->GetLightDir();
 
-  m_pActiveRenderWin->GetRenderer()->SetColors(cAmbient, cDiffuse,
-                                               cSpecular, vLightDir);
+  m_pActiveRenderWin->SetLightColors(cAmbient, cDiffuse,
+                                     cSpecular, vLightDir);
 }
 
 void MainWindow::ChangeLightColors() {
@@ -846,8 +914,21 @@ void MainWindow::ChangeLightColors() {
   cSpecular[3] = horizontalSlider_specularIntensity->value()/100.0f;
 
   m_pQLightPreview->SetData(cAmbient,cDiffuse,cSpecular,vLightDir);
-  m_pActiveRenderWin->GetRenderer()->SetColors(cAmbient, cDiffuse,
-                                               cSpecular, vLightDir);
+  m_pActiveRenderWin->SetLightColors(cAmbient, cDiffuse,
+                                     cSpecular, vLightDir);
+}
+
+void MainWindow::SetTagVolume() {
+  if (!m_pActiveRenderWin) return;
+  if (checkBox_TagVolume->isChecked())
+    m_pActiveRenderWin->SetInterpolant(tuvok::NearestNeighbor);
+  else
+    m_pActiveRenderWin->SetInterpolant(tuvok::Linear);
+}
+
+void MainWindow::UpdateInterpolant() {
+  if (!m_pActiveRenderWin) return;
+  checkBox_TagVolume->setChecked( m_pActiveRenderWin->GetInterpolant() == tuvok::NearestNeighbor);
 }
 
 void MainWindow::UpdateTFScaleSliders() {
@@ -863,11 +944,11 @@ void MainWindow::UpdateColorWidget() {
   FLOATVECTOR4 cSpecular(1,1,1,1);
   FLOATVECTOR3 vLightDir(0,0,-1);
 
-  if (m_pActiveRenderWin && m_pActiveRenderWin->GetRenderer()) {
-    cAmbient  = m_pActiveRenderWin->GetRenderer()->GetAmbient();
-    cDiffuse  = m_pActiveRenderWin->GetRenderer()->GetDiffuse();
-    cSpecular = m_pActiveRenderWin->GetRenderer()->GetSpecular();
-    vLightDir = m_pActiveRenderWin->GetRenderer()->GetLightDir();
+  if (m_pActiveRenderWin && m_pActiveRenderWin->IsRendererValid()) {
+    cAmbient  = m_pActiveRenderWin->GetAmbient();
+    cDiffuse  = m_pActiveRenderWin->GetDiffuse();
+    cSpecular = m_pActiveRenderWin->GetSpecular();
+    vLightDir = m_pActiveRenderWin->GetLightDir();
   }
 
   m_pQLightPreview->SetData(cAmbient,cDiffuse,cSpecular,vLightDir);
@@ -888,3 +969,153 @@ void MainWindow::ResetRenderingParameters() {
   if (!m_pActiveRenderWin) return;
   m_pActiveRenderWin->ResetRenderingParameters();
 }
+
+
+static void CallQTProcessEvents()
+{
+  QApplication::processEvents();
+}
+
+void MainWindow::LuaSetIsoValueFloat(float fValue)
+{
+  SetIsoValue(fValue);
+}
+
+void MainWindow::LuaSetIsoValueInteger(int iValue)
+{
+  SetIsoValue(iValue);
+}
+
+void MainWindow::RegisterLuaClasses() {
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  string prefix = "iv3d.";
+
+  ss->registerClass<RenderWindow>(
+      this, &MainWindow::LuaCreateNewWindow,
+      prefix + "renderer", "Constructs a new render window",
+      LuaClassRegCallback<RenderWindow>::Type(
+          &RenderWindow::RegisterLuaFunctions));
+
+  ss->registerClass<RenderWindow>(
+      this, &MainWindow::LuaLoadDatasetInternal, 
+      prefix + "rendererWithParams",
+      "Open a render window the additional parameters and a list of files.",
+      LuaClassRegCallback<RenderWindow>::Type(
+                &RenderWindow::RegisterLuaFunctions));
+
+  // And some lua functions...
+
+  // We need to expose QApplication::processEvents() because we are running
+  // in the GUI thread... This is useful for scripts that take a long time to
+  // run and need the UI to update.
+  ss->registerFunction(
+      &CallQTProcessEvents,
+      prefix + "processUI",
+      "Calls QT's QApplication::processEvents(). If you have a long running "
+      "script and want the UI to update while the script is running, you will "
+      "need to call this periodically. This is necessary because Tuvok runs on "
+      "the GUI's thread instead of on its own thread.",
+      false);
+
+  m_MemReg.registerFunction(this, &MainWindow::LuaResizeActiveWindow, 
+                            prefix + "resizeActiveWindow",
+                            "Resizes the active window.", true);
+
+  m_MemReg.registerFunction(this, &MainWindow::ListSupportedImages, 
+                            prefix + "listSupportedImages",
+                            "Lists supported images.", false);
+
+  m_MemReg.registerFunction(this, &MainWindow::ListSupportedVolumes, 
+                            prefix + "listSupportedVolumes",
+                            "Lists supported volumes.", false);
+
+  m_MemReg.registerFunction(this, &MainWindow::ListSupportedGeometry, 
+                            prefix + "listSupportedGeometry",
+                            "Lists supported geometry.", false);
+
+  m_MemReg.registerFunction(this, &MainWindow::SetStayOpen, 
+                            prefix + "setStayOpen",
+                            "Stay open after script ended "
+                            "passed in via command line.", false);
+
+  m_MemReg.registerFunction(this, &MainWindow::Use1DTrans, 
+                            prefix + "use1DTFMode",
+                            "Enable 1D transfer function mode.", false);
+
+  m_MemReg.registerFunction(this, &MainWindow::Use2DTrans, 
+                            prefix + "use2DTFMode",
+                            "Enable 2D transfer function mode.", false);
+
+  m_MemReg.registerFunction(this, &MainWindow::UseIso, 
+                            prefix + "useIsoMode",
+                            "Enable iso-surface rendering mode.", false);
+
+  m_MemReg.registerFunction(this, &MainWindow::LuaSetIsoValueFloat, 
+                            prefix + "setIsoValueFloat",
+                            "Set threshold value for iso-surface rendering in the range [0.0, 1.0].", false);
+
+  m_MemReg.registerFunction(this, &MainWindow::LuaSetIsoValueInteger, 
+                            prefix + "setIsoValueInteger",
+                            "Set threshold value for iso-surface rendering as integer.", false);
+
+  std::shared_ptr<LuaScripting> reg = m_MasterController.LuaScript();
+
+  reg->registerFunction(&RotateX, "matrix.rotateX",
+                        "Sets rotation to be around X, n degrees.", false);
+  reg->registerFunction(&RotateY, "matrix.rotateY",
+                        "Sets rotation to be around Y, n degrees.", false);
+  reg->registerFunction(&RotateZ, "matrix.rotateZ",
+                        "Sets rotation to be around Z, n degrees.", false);
+  reg->registerFunction(&Translate, "matrix.translate",
+                        "Sets translation vector.", false);
+  reg->registerFunction(&Identity, "matrix.identity",
+                        "Sets identity.", false);
+  reg->registerFunction(&MulMatrices, "matrix.multiply", "", false);
+  reg->registerFunction(&VecToString, "strvec", "converts vec to string",
+                        false);
+}
+
+void MainWindow::closeMDISubWindowWithWidget(QWidget* widget) {
+  QMdiSubWindow* foundWindow = NULL;
+  QList<QMdiSubWindow*> list = mdiArea->subWindowList();
+
+  for (QList<QMdiSubWindow*>::iterator it = list.begin();
+       it != list.end(); ++it) {
+    if ((*it)->widget() == widget) {
+      foundWindow = *it;
+      break;
+    }
+  }
+
+  if (foundWindow) {
+    mdiArea->removeSubWindow(foundWindow);
+  }
+}
+
+static std::string readfile(const std::string& filename) {
+  std::ifstream ifs(filename.c_str(), std::ios::in);
+  ifs >> noskipws;
+  return std::string(
+    (std::istream_iterator<char>(ifs)),
+    (std::istream_iterator<char>())
+  );
+}
+
+bool MainWindow::RunLuaScript(const std::string& strFilename) {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  ss->setExpectedExceptionFlag(true);
+  try {
+    ss->exec(readfile(strFilename));
+  } catch(const tuvok::LuaError& e) {
+    T_ERROR("Lua error executing script: %s", e.what());
+    return false;
+  } catch(const std::exception& e) {
+    T_ERROR("Error executing script: %s", e.what());
+    return false;
+  } catch(...) {
+    T_ERROR("Unknown error executing script!");
+    return false;
+  }
+  ss->setExpectedExceptionFlag(false);
+  return true;
+}
