@@ -60,7 +60,11 @@ class TextfileOut;
 #include <UI/SettingsDlg.h>
 #include <UI/Welcome.h>
 #include <UI/MetadataDlg.h>
-#include "../Tuvok/IO/uvfDataset.h"
+#include "DebugScriptWindow.h"
+
+#include "../Tuvok/LuaScripting/LuaScripting.h"
+#include "../Tuvok/LuaScripting/LuaClassRegistration.h"
+#include "../Tuvok/LuaScripting/LuaMemberReg.h"
 
 class QDragEnterEvent;
 class QDropEvent;
@@ -72,8 +76,9 @@ class FTPDialog;
 class PleaseWaitDialog;
 class ScaleAndBiasDlg;
 
-class MainWindow : public QMainWindow, protected Ui_MainWindow,
-                   public Scriptable
+static const int IV3D_TIMER_INTERVAL = 20; // in milliseconds
+
+class MainWindow : public QMainWindow, protected Ui_MainWindow
 {
   Q_OBJECT
 
@@ -87,17 +92,30 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
 
     QTOut* GetDebugOut() {return m_pDebugOut;}
 
-    // Scriptable implementation
-    virtual void RegisterCalls(Scripting* pScriptEngine);
-    virtual bool Execute(const std::string& strCommand, const std::vector< std::string >& strParams, std::string& strMessage);
-
-    bool RunScript(const std::string& strFilename);
     bool StayOpen() const {return m_bStayOpenAfterScriptEnd;}
     /// Starts the internal timer, used for checking if we should continue
     /// rendering.
     void StartTimer();
     const std::string& GetTempDir() {return m_strTempDir;}
 
+    // Lua function binding
+
+    /// Member function constructor for RenderWindowGL.
+    RenderWindow* LuaCreateNewWindow(std::string dataset);
+
+    void LuaResizeActiveWindow(const UINTVECTOR2& newSize);
+
+    void closeMDISubWindowWithWidget(QWidget* widget);
+
+    // Lua callback functions (called from Lua when actions are undone/redone).
+    void LuaCallbackToggleClipPlane(bool bClip);
+
+    // directly loads the TFqn; used for Lua scripting.
+    void LoadTransferFunction1D(const std::string& tf);
+    void LoadTransferFunction2D(const std::string& tf);
+
+    bool RunLuaScript(const std::string& strFilename);
+
   public slots:
     void SetRenderProgressAnUpdateInfo(unsigned int iLODCount,
                                        unsigned int iCurrentCount,
@@ -144,6 +162,7 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void UseIso();
     void DisableAllTrans();
 
+    void Invert1DTransComp();
     void Transfer1DSetColors();
     void Transfer1DSetGroups();
     void Transfer1DLoad();
@@ -170,6 +189,7 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void Transfer2DToggleTFMode();
 
     void SetUpdateMode();
+    void SetTagVolume();
     void ApplyUpdate();
 
     void Transfer2DSwatchesChanged();
@@ -197,9 +217,11 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void UpdateMenus();
     void ExportDataset();
     void MergeDatasets();
-    bool ExportDataset(UINT32 iLODLevel, std::string targetFileName);
+    bool ExportDataset(uint32_t iLODLevel, std::string targetFileName);
     void ExportIsosurface();
-    bool ExportIsosurface(UINT32 iLODLevel, std::string targetFileName);
+    bool ExportIsosurface(uint32_t iLODLevel, std::string targetFileName);
+    void ExportImageStack();
+    bool ExportImageStack(uint32_t iLODLevel, std::string targetFileName, bool bAllDirs);
 
     void RenderWindowActive(RenderWindow* sender);
     void RenderWindowClosing(RenderWindow* sender);
@@ -215,18 +237,20 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void ListSupportedVolumes();
     void ListSupportedGeometry();
     void ClearDebugWin();
-    void ParseAndExecuteDebugCommand();
     void SetDebugViewMask();
 
     void CheckForRedraw();
-    bool ShowSettings() {return ShowSettings(false);}
-    bool ShowSettings(bool bInitializeOnly);
+    bool ShowAdvancedSettings(bool bInitializeOnly);
+    bool ShowBasicSettings(bool initOnly);
+    bool ShowSettings(bool initOnly=false);
     void SetLighting(bool bLighting);
     void ToggleLighting();
 
     void Collapse2DWidgets();
     void Expand2DWidgets();
     void SetSampleRate(int iValue);
+    void SetFoV(int iValue);
+    void SetFoVSlider(int iValue);
     void SetIsoValue(float fValue);
     void SetIsoValue(int iValue);
     void SetClearViewIsoValue(float fValue);
@@ -235,11 +259,13 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void ToggleClipPlane(bool);
     void ClipToggleLock(bool);
     void ClipToggleShow(bool);
+    void CropData();
     void SetTimestep(int);
     void SetTimestepSlider(int iValue, int iMaxValue);
     void UpdateTimestepLabel(int iValue, int iMaxValue);
     void ResetTimestepUI();
 
+    void SetStayOpen(bool bStayOpenAfterScriptEnd);
     void SetRescaleFactors();
     virtual void closeEvent(QCloseEvent *event);
 
@@ -269,8 +295,6 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void ToggleStereoEyeSwap();
     void SetStereoEyeDistance();
     void SetStereoFocalLength();
-    void InitAFStereo();
-    void ToggleAFStereo();
 
     void Show1DTrans();
     void Show2DTrans();
@@ -318,7 +342,7 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     bool                                      m_bQuickopen;
     unsigned int                              m_iMinFramerate;
     unsigned int                              m_iLODDelay;
-    bool                                      m_bUseAllMeans;
+    bool                                      m_bRenderLowResIntermediateResults;
     unsigned int                              m_iActiveTS;
     unsigned int                              m_iInactiveTS;
     bool                                      m_bWriteLogFile;
@@ -327,13 +351,14 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
 
     WelcomeDialog*                            m_pWelcomeDialog;
     MetadataDlg*                              m_pMetadataDialog;
+    //DebugScriptWindow*                        m_pDebugScriptWindow;
 
     unsigned int                              m_iBlendPrecisionMode;
     bool                                      m_bPowerOfTwo;
     bool                                      m_bDownSampleTo8Bits;
     bool                                      m_bDisableBorder;
-    bool                                      m_bNoRCClipplanes;
     bool                                      m_bI3MFeatures;
+    bool                                      m_bAdvancedSettings;
     bool                                      m_bAutoSaveGEO;
     bool                                      m_bAutoSaveWSP;
     MasterController::EVolumeRendererType     m_eVolumeRendererType;
@@ -387,10 +412,15 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     bool LoadDataset(const std::vector< std::string >& strParams);
     bool LoadDataset(QStringList fileName, QString targetFileName="",
                      bool bNoUserInteraction=false);
+    bool LoadDatasetInternal(QStringList files, QString targetFilename,
+                             bool bNoUserInteraction);
+    RenderWindow* LuaLoadDatasetInternal(std::vector<std::string> filename,
+                                         std::string targetFileName,
+                                         bool bNoUserInteraction);
     bool RebrickDataset(QString filename, QString targetFilename,
                         bool bNoUserInteraction);
 
-    QString GetConvFilename();
+    QString GetConvFilename(const QString& sourceName = "");
 
     void InitDockWidget(QDockWidget * v) const;
     void InitAllWorkspaces();
@@ -398,8 +428,26 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void CheckSettings();
     void ApplySettings();
     void ApplySettings(RenderWindow* renderWin);
+    void SaveSettings(
+      uint64_t CPUMem, uint64_t maxCPU, uint64_t GPUMem,
+      uint64_t maxGPU, bool ignoreMax,
+      const std::string& tempDir,
+      bool checksum, unsigned framerate, bool lowResSubframes, unsigned LODDelay,
+      unsigned activeTS, unsigned inactiveTS, bool writeLog, bool showCrashDialog,
+      const std::string& logFile, uint32_t logLevel, bool showVersion,
+      bool autoSaveGeo, bool autoSaveWSP, bool autoLockCloned, bool absoluteLocks,
+      bool checkForUpdates,
+      bool checkForDevBuilds, bool showWelcome, bool invertWheel, bool i3mFeatures,
+      unsigned volRenType, unsigned blendPrecision, bool powerTwo,
+      bool downsampleTo8, bool disableBorder, const FLOATVECTOR3& backColor1,
+      const FLOATVECTOR3& backColor2, const FLOATVECTOR4& textColor,
+      const QString& logo, int logoPosition, unsigned maxBrickSize,
+      unsigned builderBrickSize, bool medianFilter, bool clampToEdge,
+      uint32_t compression, bool experimentalFeatures, bool advancedSettings
+    );
     void SetSampleRateSlider(int iValue);
     void UpdateSampleRateLabel(int iValue);
+    void UpdateFoVLabel(int iValue);
     void SetIsoValueSlider(int iValue, int iMaxValue);
     void UpdateIsoValLabel(int iValue, int iMaxValue);
     void SetToggleGlobalBBoxLabel(bool bRenderBBox);
@@ -447,14 +495,20 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void UpdateMinMaxLODLimitLabel();
     void UpdateExplorerView(bool bRepopulateListBox);
     void UpdateTFScaleSliders();
+    void UpdateInterpolant();
     void UpdateColorWidget();
-    
+
+    void LuaSetIsoValueFloat(float fValue);
+    void LuaSetIsoValueInteger(int iValue);
+
+    void RegisterLuaClasses();
+
     std::string ConvertTF(const std::string& strSource1DTFilename,
                           const std::string& strTargetDir,
-                          const UVFDataset* currentDataset,
+                          const std::string& strTargetFullFilename,
                           PleaseWaitDialog& pleaseWait);
 
-    std::string ConvertDataToI3M(const UVFDataset* currentDataset,
+    std::string ConvertDataToI3M(LuaClassInstance currentDataset,
                                  const std::string& strTargetDir,
                                  PleaseWaitDialog& pleaseWait,
                                  bool bOverrideExisting);
@@ -465,6 +519,9 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     RenderWindow* ActiveRenderWin();
     QMdiSubWindow* ActiveSubWindow();
     RenderWindow* m_pActiveRenderWin;
+    RenderWindow* m_pLastLoadedRenderWin; ///< Use with caution -- could have
+                                          ///< been destroyed. Used to return
+                                          ///< render window.
 
     // update
     struct VersionNumber {
@@ -505,12 +562,14 @@ class MainWindow : public QMainWindow, protected Ui_MainWindow,
     void SetStereoMode(unsigned int iMode);
     void SetStereoFocalLength(float fLength);
     void SetStereoEyeDistance(float fEyeDist);
-    void IncStereoFocalLength(float fLengthDelta);
-    void IncStereoEyeDistance(float fEyeDistDelta);
 
     bool m_bClipDisplay;
     bool m_bClipLocked;
 
+    bool m_bIgnoreLoadDatasetFailure;
+
+    tuvok::LuaMemberReg       m_MemReg;
+
     friend class MergeDlg;
 };
 
