@@ -37,29 +37,20 @@
 
 #include "../Tuvok/Basics/StdDefines.h"
 #include <algorithm>
+#include <array>
 #include <cstdio>
-#include <string>
-#ifdef _MSC_VER
-# include <array>
-#else
-# include <tr1/array>
-#endif
-
-#ifdef DETECTED_OS_LINUX
-# include <libgen.h>
-# include <unistd.h>
-#endif
 #include <fstream>
+#include <string>
 
 #include <QtCore/QUrl>
 #include <QtCore/QTime>
 #include <QtCore/QDate>
 #include <QtCore/QDir>
-#include <QtCore/QTemporaryFile>
+#include <QtCore/QFile>
 #include <QtCore/QFileInfo>
+#include <QtCore/QSettings>
 #include <QtCore/QTemporaryFile>
 #include <QtCore/QTextStream>
-#include <QtCore/QSettings>
 #include <QtGui/QMessageBox>
 #include <QtGui/QDesktopServices>
 #include <QtNetwork/QHttp>
@@ -132,6 +123,7 @@ void MainWindow::CheckForUpdates() {
 }
 
 void MainWindow::CheckForUpdatesInternal() {
+#ifndef PACKAGE_MANAGER
   // cleanup updatefile, this codepath is taken for instance when the windows firewall blocked an http request
   if (m_pUpdateFile && m_pUpdateFile->isOpen()) {
     m_pUpdateFile->close();
@@ -161,6 +153,7 @@ void MainWindow::CheckForUpdatesInternal() {
   QByteArray remotePath = QUrl::toPercentEncoding(url.path(), "!$&'()*+,;=:@/");
   if (remotePath.isEmpty()) remotePath = "/";
   m_iHttpGetId = m_pHttp->get(remotePath, m_pUpdateFile);
+#endif
 }
 
 void MainWindow::httpRequestFinished(int requestId, bool error) {
@@ -359,14 +352,14 @@ std::string MainWindow::GenUniqueName(const std::string& strPrefix, const std::s
   return string(tr("%1_%2_%3.%4").arg(strPrefix.c_str()).arg(QTime::currentTime().toString()).arg(QDate::currentDate().toString()).arg(strExt.c_str()).toAscii());
 }
 
-bool MainWindow::FtpTransfer(string strSource, string strDest, bool bDeleteSource) {
+bool MainWindow::FtpTransfer(string strSource, string strDest,
+                             bool bDeleteSource) {
   if (!m_bFTPFinished) return false;
   m_bFTPFinished = true;
   m_strFTPTempFile = strSource;
 
   if (m_pFTPDialog) {
-    disconnect(m_pFTPDialog, SIGNAL(TransferFailure()), this, SLOT(FtpFail()));
-    disconnect(m_pFTPDialog, SIGNAL(TransferSuccess()), this, SLOT(FtpSuccess()));
+    disconnect(m_pFTPDialog, 0,0,0);
     delete m_pFTPDialog;
   }
 
@@ -399,11 +392,9 @@ void MainWindow::OnlineHelp() {
   QDesktopServices::openUrl(QString(HELP_URL));
 }
 
-#ifdef DETECTED_OS_LINUX
-static bool readable(const std::string &f) {
-  return (access(f.c_str(), R_OK) == 0);
+bool readable(const std::string& f) {
+  return QFile(QString(f.c_str())).permissions() & QFile::ReadUser;
 }
-#endif
 
 void MainWindow::OpenManual() {
 #ifdef DETECTED_OS_WINDOWS
@@ -417,19 +408,20 @@ void MainWindow::OpenManual() {
 
   /// Find out where our binary lives.  The manual is placed in the same
   /// directory in the case of the binary tarballs we provide.
-  std::vector<std::string> paths(4);
+  std::vector<std::string> paths; paths.reserve(4);
   char linkbuf[1024];
   memset(linkbuf, 0, 1024);
   if(readlink("/proc/self/exe", linkbuf, 1024) == -1) {
     T_ERROR("Error reading /proc/self/exe; ignoring binary directory while "
             "searching for manual.");
   } else {
-    paths.push_back(std::string(dirname(linkbuf)) + MANUAL_NAME);
+    paths.push_back(QFileInfo(linkbuf).absolutePath().toStdString() +
+                    MANUAL_NAME);
   }
   paths.push_back(std::string("/usr/share/doc/imagevis3d/") + MANUAL_NAME);
   paths.push_back(std::string("/usr/local/share/doc/imagevis3d/") +
                   MANUAL_NAME);
-  paths.push_back(std::string("./") + MANUAL_NAME);
+  paths.push_back(MANUAL_NAME);
   std::vector<std::string>::const_iterator found = find_if(paths.begin(),
                                                            paths.end(),
                                                            readable);
@@ -447,24 +439,28 @@ void MainWindow::OpenManual() {
   }
   MESSAGE("Manual found at: %s", found->c_str());
 
-  // Now, figure out how to load the PDF.  Ideally, we'd use FD.o's
-  // shared-mime-info to lookup which program the user wants to use for
-  // this.  Qt is probably linking to this in some way, so it's likely
-  // we can use it directly, but I'm having trouble verifying that.
-  /// @todo FIXME use shared-mime-info to figure out PDF-reading program
-  // So, for now, just iterate through a hardcoded list and use the first that
-  // works.
-
-  typedef std::tr1::array<std::string, 3> progvec;
-  progvec viewers = {{ "evince", "xpdf", "acroread" }};
-  // Now iterate through each of those programs until one of them
-  // successfully launches.
-  for(progvec::const_iterator prog = viewers.begin(); prog != viewers.end();
-      ++prog) {
-    char manual[1024];
-    snprintf(manual, 1024, "%s %s", prog->c_str(), found->c_str());
-    if(system(manual) != -1) {
-      break;
+  // Now, figure out how to load the PDF.
+  QUrl uri;
+  {
+    std::ostringstream u;
+    u << *found;
+    QString pth = QFileInfo(u.str().c_str()).absoluteFilePath();
+    uri.setUrl(pth);
+    MESSAGE("uri: %s", pth.toStdString().c_str());
+  }
+  if(!QDesktopServices::openUrl(uri)) {
+    MESSAGE("Manual open failed, relying on manual method.");
+    typedef std::array<std::string, 3> progvec;
+    progvec viewers = {{ "evince", "xpdf", "acroread" }};
+    // Now iterate through each of those programs until one of them
+    // successfully launches.
+    for(progvec::const_iterator prog = viewers.begin(); prog != viewers.end();
+        ++prog) {
+      char manual[1024];
+      snprintf(manual, 1024, "%s %s", prog->c_str(), found->c_str());
+      if(system(manual) != -1) {
+        break;
+      }
     }
   }
 #endif
@@ -545,7 +541,7 @@ void MainWindow::ReportABug(const string& strFile) {
     if (b.SubmitSysinfo() ) {
       reportFile << endl << endl << "Memory info:" << endl;
 
-      const UINT64 mb = 1024*1024;
+      const uint64_t mb = 1024*1024;
       reportFile << "CPU Memory: Total "
                  << m_MasterController.MemMan()->GetCPUMem()/mb
                  << " MB, Usable "
