@@ -53,8 +53,11 @@
 
 #include "../Tuvok/Controller/Controller.h"
 #include "../Tuvok/Basics/SysTools.h"
+#include "../Tuvok/LuaScripting/LuaScripting.h"
+#include "../Tuvok/LuaScripting/TuvokSpecific/LuaTuvokTypes.h"
 
 #include "ImageVis3D.h"
+#include "MDIRenderWin.h"
 #include "BrowseData.h"
 #include "RenderWindowGL.h"
 #include "RenderWindowDX.h"
@@ -236,8 +239,12 @@ void MainWindow::SetHistogramScale2D(int v) {
 }
 
 void MainWindow::setupUi(QMainWindow *MainWindow) {
+
   Ui_MainWindow::setupUi(MainWindow);
 
+  //this->addDockWidget(Qt::BottomDockWidgetArea, m_pDebugScriptWindow);
+  //m_pDebugScriptWindow->setObjectName(QString::fromUtf8("DebugScriptWindow"));
+
   SetTitle();
 
   m_1DTransferFunction =
@@ -397,6 +404,7 @@ void MainWindow::SetupWorkspaceMenu() {
 
   if(m_MasterController.ExperimentalFeatures()) {
     menu_Workspace->addAction(dockWidget_Time->toggleViewAction());
+    dockWidget_Time->toggleViewAction()->setShortcut(tr("Ctrl-Alt-t"));
   }
   /// @todo FIXME need a shortcut for timestep dockWidget
   //dockWidget_IsoSurface->toggleViewAction()->setShortcut(tr("Ctrl+Alt+5"));
@@ -416,6 +424,8 @@ void MainWindow::SetupWorkspaceMenu() {
 
   menu_Help->addAction(dockWidget_Debug->toggleViewAction());
   dockWidget_Debug->toggleViewAction()->setShortcut(tr("Ctrl+Alt+D"));
+
+  //menu_Help->addAction(m_pDebugScriptWindow->toggleViewAction());
 }
 
 void MainWindow::ClearWSMRUList()
@@ -510,6 +520,7 @@ void MainWindow::InitAllWorkspaces() {
   InitDockWidget(dockWidget_Time);
   InitDockWidget(dockWidget_Debug);
   InitDockWidget(dockWidget_Stereo);
+  //InitDockWidget(m_pDebugScriptWindow);
 }
 
 
@@ -633,8 +644,9 @@ bool MainWindow::ApplyWorkspace() {
 
 
 void MainWindow::ResizeCurrentView(int iSizeX, int iSizeY) {
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
   if (ActiveSubWindow()) {
-    UINTVECTOR2 renderSize = m_pActiveRenderWin->GetRenderer()->GetSize();
+    UINTVECTOR2 renderSize = m_pActiveRenderWin->GetRendererSize();
     UINTVECTOR2 windowSize(ActiveSubWindow()->size().width(), 
                            ActiveSubWindow()->size().height());
 
@@ -642,7 +654,8 @@ void MainWindow::ResizeCurrentView(int iSizeX, int iSizeY) {
     ActiveSubWindow()->resize(iSizeX+winDecoSize.x, iSizeY+winDecoSize.y);
   } else {
     if (mdiArea->activeSubWindow()) {
-      UINTVECTOR2 renderSize = WidgetToRenderWin(mdiArea->activeSubWindow()->widget())->GetRenderer()->GetSize();
+      UINTVECTOR2 renderSize = 
+          WidgetToRenderWin(mdiArea->activeSubWindow()->widget())->GetRendererSize();
       UINTVECTOR2 windowSize(mdiArea->activeSubWindow()->size().width(), 
                              mdiArea->activeSubWindow()->size().height());
 
@@ -674,10 +687,12 @@ void MainWindow::CloneCurrentView() {
   if (m_bAutoLockClonedWindow)
     for (size_t i = 0;i<RenderWindow::ms_iLockCount;i++) SetLock(i, renderWin, m_pActiveRenderWin);
 
-  QMdiSubWindow * pActiveWin = mdiArea->activeSubWindow(); // as "show" toggles the active renderwin we need to remeber it
+  QMdiSubWindow* pActiveWin = mdiArea->activeSubWindow(); // as "show" toggles the active renderwin we need to remeber it
   renderWin->GetQtWidget()->show();
   RenderWindowActive(renderWin);
-  mdiArea->activeSubWindow()->resize(pActiveWin->size().width(), pActiveWin->size().height());
+  mdiArea->activeSubWindow()->resize(pActiveWin->size().width(),
+                                     pActiveWin->size().height());
+  SetTagVolume();
 
   CheckForMeshCapabilities(true);
 }
@@ -708,31 +723,43 @@ bool MainWindow::CheckRenderwindowFitness(RenderWindow *renderWin, bool bIfNotOk
   return false;
 }
 
-RenderWindow* MainWindow::CreateNewRenderWindow(QString dataset)
-{
+
+void MainWindow::LuaResizeActiveWindow(const UINTVECTOR2& newSize) {
+  ResizeCurrentView(newSize.x, newSize.y);
+}
+
+RenderWindow* MainWindow::LuaCreateNewWindow(std::string dataset) {
+  std::vector<std::string> fileList;
+  fileList.push_back(dataset);
+  return LuaLoadDatasetInternal(fileList, std::string(""), false);
+}
+
+RenderWindow* MainWindow::CreateNewRenderWindow(QString dataset) {
   static unsigned int iCounter = 0;
   RenderWindow *renderWin;
 
   #if defined(_WIN32) && defined(USE_DIRECTX)
     if (m_eVolumeRendererType == MasterController::DIRECTX_SBVR ||
         m_eVolumeRendererType == MasterController::DIRECTX_RAYCASTER ||
-        m_eVolumeRendererType == MasterController::DIRECTX_2DSBVR) {
+        m_eVolumeRendererType == MasterController::DIRECTX_2DSBVR ||
+        m_eVolumeRendererType == MasterController::DIRECTX_GRIDLEAPER) {
       renderWin = new RenderWindowDX(m_MasterController, m_eVolumeRendererType, dataset,
-                                       iCounter++, m_bPowerOfTwo, m_bDownSampleTo8Bits,
-                                       m_bDisableBorder, this, 0);
+                                     iCounter++, m_bPowerOfTwo, m_bDownSampleTo8Bits,
+                                     m_bDisableBorder, this, 0);
     } else {
       QGLFormat fmt;
       fmt.setRgba(true);
       fmt.setAlpha(true);
       renderWin = new RenderWindowGL(m_MasterController, m_eVolumeRendererType, dataset,
                                      iCounter++, m_bPowerOfTwo, m_bDownSampleTo8Bits,
-                                     m_bDisableBorder, m_bNoRCClipplanes,
+                                     m_bDisableBorder,
                                      m_glShareWidget, fmt, this, 0);
     }
   #else
     if (m_eVolumeRendererType == MasterController::DIRECTX_SBVR ||
         m_eVolumeRendererType == MasterController::DIRECTX_RAYCASTER ||
-        m_eVolumeRendererType == MasterController::DIRECTX_2DSBVR) {
+        m_eVolumeRendererType == MasterController::DIRECTX_2DSBVR ||
+        m_eVolumeRendererType == MasterController::DIRECTX_GRIDLEAPER) {
       ShowInformationDialog( "No DirectX Support", "The system was unable to open a DirectX 10 render window, falling back to OpenGL. Please check your settings.");
       m_MasterController.DebugOut()->Message("MainWindow::CreateNewRenderWindow","The system was unable to open a DirectX 10 render window, falling back to OpenGL. Please check your settings.");
 
@@ -740,18 +767,32 @@ RenderWindow* MainWindow::CreateNewRenderWindow(QString dataset)
         m_eVolumeRendererType = MasterController::OPENGL_SBVR;
       else if (m_eVolumeRendererType == MasterController::DIRECTX_RAYCASTER)
         m_eVolumeRendererType = MasterController::OPENGL_RAYCASTER;
-      else
+      else if (m_eVolumeRendererType == MasterController::DIRECTX_2DSBVR)
         m_eVolumeRendererType = MasterController::OPENGL_2DSBVR;
+      else
+        m_eVolumeRendererType = MasterController::OPENGL_GRIDLEAPER;
+    }
+    // if they're using the basic configuration, choose a renderer for them.
+    // We can't just choose this here because we need to query some OGL
+    // parameters to figure this out.
+    if(m_bAdvancedSettings == false) {
+      m_eVolumeRendererType = MasterController::OPENGL_CHOOSE;
     }
     QGLFormat fmt;
     fmt.setAlpha(true);
     fmt.setRgba(true);
     renderWin = new RenderWindowGL(m_MasterController, m_eVolumeRendererType, dataset,
                                    iCounter++, m_bPowerOfTwo, m_bDownSampleTo8Bits,
-                                   m_bDisableBorder, m_bNoRCClipplanes,
+                                   m_bDisableBorder,
                                    m_glShareWidget, fmt, this, 0);
   #endif
 
+  if (m_pActiveRenderWin != renderWin && !renderWin->IsRenderSubsysOK()) {
+    delete renderWin;
+    T_ERROR("Could not initialize render window!");
+    return NULL;
+  }
+
   connect(renderWin->GetQtWidget(), SIGNAL(WindowActive(RenderWindow*)),
           this, SLOT(RenderWindowActive(RenderWindow*)));
   connect(renderWin->GetQtWidget(), SIGNAL(WindowClosing(RenderWindow*)),
@@ -760,16 +801,20 @@ RenderWindow* MainWindow::CreateNewRenderWindow(QString dataset)
           this, SLOT(RenderWindowViewChanged(int)));
   connect(renderWin->GetQtWidget(), SIGNAL(StereoDisabled()),
           this, SLOT(StereoDisabled()));
-  mdiArea->addSubWindow(renderWin->GetQtWidget());
+
+  mdiArea->addSubWindow(new MDIRenderWin(m_MasterController, renderWin));
   renderWin->InitializeContext();
 
-  if(m_pActiveRenderWin != renderWin && !renderWin->IsRenderSubsysOK()) {
-    T_ERROR("Could not initialize render window!");
+  if (m_pActiveRenderWin != renderWin && !renderWin->IsRenderSubsysOK()) {
+    delete renderWin;
+    T_ERROR("Could not initialize render window context!");
     return NULL;
-  } else {
-    ApplySettings(renderWin);
   }
 
+  m_pLastLoadedRenderWin = renderWin;
+
+  ApplySettings(renderWin);
+
   QCoreApplication::processEvents();
 
   return renderWin;
@@ -795,19 +840,27 @@ void MainWindow::RenderWindowActive(RenderWindow* sender) {
     if (w) w->close();
     return;
   }
-  AbstrRenderer *const ren = sender->GetRenderer();
 
   MESSAGE("Getting 1D Transfer Function.");
-
-
-  std::pair<double,double> range = ren->GetDataset().GetRange();
-  m_1DTransferFunction->SetData(&ren->GetDataset().Get1DHistogram(),
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  LuaClassInstance ds = sender->GetRendererDataset();
+  std::pair<double,double> range = 
+      ss->cexecRet<std::pair<double,double>>(ds.fqName() + ".getRange");
+  shared_ptr<const Histogram1D> hist1D = 
+      ss->cexecRet<shared_ptr<const Histogram1D>>(
+          ds.fqName() + ".get1DHistogram");
+  LuaClassInstance tf1d = sender->GetRendererTransferFunction1D();
+  m_1DTransferFunction->SetData(hist1D,
                                 static_cast<unsigned int>(range.second-range.first),
-                                ren->Get1DTrans());
+                                tf1d);
   m_1DTransferFunction->update();
+
   MESSAGE("Getting 2D Transfer Function.");
-  m_2DTransferFunction->SetData(&ren->GetDataset().Get2DHistogram(),
-                                ren->Get2DTrans());
+  shared_ptr<const Histogram2D> hist2D = 
+      ss->cexecRet<shared_ptr<const Histogram2D>>(
+          ds.fqName() + ".get2DHistogram");
+  LuaClassInstance tf2d = sender->GetRendererTransferFunction2D();
+  m_2DTransferFunction->SetData(hist2D, tf2d);
   m_2DTransferFunction->update();
 
   MESSAGE("Getting other Renderwindow parameters.");
@@ -825,32 +878,33 @@ void MainWindow::RenderWindowActive(RenderWindow* sender) {
   }
 
   EnableStereoWidgets();
-  checkBox_Stereo->setChecked(ren->GetStereo());
-  horizontalSlider_EyeDistance->setValue(int(ren->GetStereoEyeDist()*100));
-  horizontalSlider_FocalLength->setValue(int(ren->GetStereoFocalLength()*10));
-  checkBox_EyeSwap->setChecked(ren->GetStereoEyeSwap());
+  checkBox_Stereo->setChecked(sender->GetRendererStereoEnabled());
+  horizontalSlider_EyeDistance->setValue(int(sender->GetRendererStereoEyeDist()*100));
+  horizontalSlider_FocalLength->setValue(int(sender->GetRendererStereoFocalLength()*10));
+  checkBox_EyeSwap->setChecked(sender->GetRendererStereoEyeSwap());
 
-  switch (ren->GetStereoMode()) {
+  switch (sender->GetRendererStereoMode()) {
     case AbstrRenderer::SM_RB : radioButton_RBStereo->setChecked(true); break;
     default                   : radioButton_ScanlineStereo->setChecked(true); break;
   }
 
-  checkBox_Lighting->setChecked(ren->GetUseLighting());
-  SetSampleRateSlider(int(ren->GetSampleRateModifier()*100));
-  int iRange = int(m_pActiveRenderWin->GetDynamicRange().second)-1;
-  SetIsoValueSlider(int(ren->GetIsoValue()), iRange);
+  checkBox_Lighting->setChecked(sender->GetUseLighting());
+  SetSampleRateSlider(int(sender->GetRendererSampleRateModifier()*100));
+  SetFoVSlider(int(sender->GetRendererFoV()));
+  int iRange = int(m_pActiveRenderWin->GetDynamicRange().second);
+  SetIsoValueSlider(int(sender->GetRendererIsoValue()), iRange);
 
-  DOUBLEVECTOR3 vfRescaleFactors = ren->GetRescaleFactors();
+  DOUBLEVECTOR3 vfRescaleFactors = sender->GetRendererRescaleFactors();
   doubleSpinBox_RescaleX->setValue(vfRescaleFactors.x);
   doubleSpinBox_RescaleY->setValue(vfRescaleFactors.y);
   doubleSpinBox_RescaleZ->setValue(vfRescaleFactors.z);
 
-  SetToggleGlobalBBoxLabel(ren->GetGlobalBBox());
-  SetToggleLocalBBoxLabel(ren->GetLocalBBox());
+  SetToggleGlobalBBoxLabel(sender->GetRendererGlobalBBox());
+  SetToggleLocalBBoxLabel(sender->GetRendererLocalBBox());
 
-  SetToggleClipEnabledLabel(ren->ClipPlaneEnabled());
-  SetToggleClipShownLabel(ren->ClipPlaneShown());
-  SetToggleClipLockedLabel(ren->ClipPlaneLocked());
+  SetToggleClipEnabledLabel(sender->GetRendererClipPlaneEnabled());
+  SetToggleClipShownLabel(sender->GetRendererClipPlaneShown());
+  SetToggleClipLockedLabel(sender->GetRendererClipPlaneLocked());
 
   ClearProgressViewAndInfo();
 
@@ -859,38 +913,44 @@ void MainWindow::RenderWindowActive(RenderWindow* sender) {
 
 
   UpdateExplorerView(true);
-  groupBox_ClipPlane->setVisible(ren->CanDoClipPlane());
+//  groupBox_ClipPlane->setVisible(sender->GetRendererCanDoClipPlane());
 
   lineEdit_DatasetName->setText(QFileInfo(m_pActiveRenderWin->
                                           GetDatasetName()).fileName());
-  UINT64VECTOR3 vSize = ren->GetDataset().GetDomainSize();
-  UINT64 iBitWidth = ren->GetDataset().GetBitWidth();
 
-  pair<double, double> pRange = ren->GetDataset().GetRange();
+  UINT64VECTOR3 vSize = ss->cexecRet<UINT64VECTOR3>(
+          ds.fqName() + ".getDomainSize", size_t(0), size_t(0));
+  uint64_t iBitWidth = ss->cexecRet<uint64_t>(ds.fqName() + ".getBitWidth");
+  pair<double, double> pRange = ss->cexecRet<pair<double, double>>(
+      ds.fqName() + ".getRange");
+  uint64_t numDSTimesteps = ss->cexecRet<uint64_t>(ds.fqName() 
+                                                   + ".getNumberOfTimesteps");
+  uint64_t lodLevelCount = ss->cexecRet<uint64_t>(ds.fqName()
+                                                  + ".getLODLevelCount");
 
   QString strSize = tr("%1 x %2 x %3 (%4bit)").arg(vSize.x).
                                                arg(vSize.y).
                                                arg(vSize.z).
                                                arg(iBitWidth);
   if (pRange.first<=pRange.second) {
-    strSize = strSize + tr(" Min=%1 Max=%2").arg(UINT64(pRange.first)).
-                                             arg(UINT64(pRange.second));
+    strSize = strSize + tr(" Min=%1 Max=%2").arg(uint64_t(pRange.first)).
+                                             arg(uint64_t(pRange.second));
   }
   // -1: slider is 0 based, but label is not.
-  SetTimestepSlider(static_cast<int>(ren->Timestep()),
-                    ren->GetDataset().GetNumberOfTimesteps()-1);
-  UpdateTimestepLabel(static_cast<int>(ren->Timestep()),
-                      ren->GetDataset().GetNumberOfTimesteps());
+  SetTimestepSlider(static_cast<int>(sender->GetRendererTimestep()),
+                    numDSTimesteps-1);
+  UpdateTimestepLabel(static_cast<int>(sender->GetRendererTimestep()),
+                      numDSTimesteps);
 
   lineEdit_MaxSize->setText(strSize);
-  UINT64 iLevelCount = ren->GetDataset().GetLODLevelCount();
+  uint64_t iLevelCount = lodLevelCount;
   QString strLevelCount = tr("%1").arg(iLevelCount);
   lineEdit_MaxLODLevels->setText(strLevelCount);
 
   horizontalSlider_maxLODLimit->setMaximum(iLevelCount-1);
   horizontalSlider_minLODLimit->setMaximum(iLevelCount-1);
 
-  UINTVECTOR2 iLODLimits = ren->GetLODLimits();
+  UINTVECTOR2 iLODLimits = sender->GetRendererLODLimits();
 
   horizontalSlider_minLODLimit->setValue(iLODLimits.x);
   horizontalSlider_maxLODLimit->setValue(iLODLimits.y);
@@ -900,6 +960,8 @@ void MainWindow::RenderWindowActive(RenderWindow* sender) {
   UpdateColorWidget();
 
   UpdateTFScaleSliders();
+
+  UpdateInterpolant();
 }
 
 void MainWindow::ToggleMesh() {
@@ -908,9 +970,9 @@ void MainWindow::ToggleMesh() {
   int iCurrent = listWidget_DatasetComponents->currentRow();
   if (iCurrent < 1 || iCurrent >= listWidget_DatasetComponents->count()) return;
 
-  RenderMesh* mesh = (RenderMesh*)m_pActiveRenderWin->GetRenderer()->GetMeshes()[iCurrent-1];
+  shared_ptr<RenderMesh> mesh = m_pActiveRenderWin->GetRendererMeshes()[iCurrent-1];
   mesh->SetActive(checkBox_ComponenEnable->isChecked());
-  m_pActiveRenderWin->GetRenderer()->Schedule3DWindowRedraws();
+  m_pActiveRenderWin->RendererSchedule3DWindowRedraws();
   ToggleClearViewControls();
 }
 
@@ -921,8 +983,7 @@ void MainWindow::SetMeshDefOpacity() {
   int iCurrent = listWidget_DatasetComponents->currentRow();
   if (iCurrent < 1 || iCurrent >= listWidget_DatasetComponents->count()) return;
 
-  AbstrRenderer* renderer = m_pActiveRenderWin->GetRenderer();
-  RenderMesh* mesh = (RenderMesh*)renderer->GetMeshes()[iCurrent-1];
+  shared_ptr<RenderMesh> mesh = m_pActiveRenderWin->GetRendererMeshes()[iCurrent-1];
   
   FLOATVECTOR4 meshcolor = mesh->GetDefaultColor();
 
@@ -931,7 +992,7 @@ void MainWindow::SetMeshDefOpacity() {
   if (fSlideVal != meshcolor.w) {
     meshcolor.w = fSlideVal;
     mesh->SetDefaultColor(meshcolor);
-    m_pActiveRenderWin->GetRenderer()->Schedule3DWindowRedraws();
+    m_pActiveRenderWin->RendererSchedule3DWindowRedraws();
   }
 }
 
@@ -941,11 +1002,11 @@ void MainWindow::SetMeshScaleAndBias() {
   int iCurrent = listWidget_DatasetComponents->currentRow();
   if (iCurrent < 1 || iCurrent >= listWidget_DatasetComponents->count()) return;
 
-  AbstrRenderer* renderer = m_pActiveRenderWin->GetRenderer();
-  RenderMesh* mesh = (RenderMesh*)renderer->GetMeshes()[iCurrent-1];
+  shared_ptr<RenderMesh> mesh = m_pActiveRenderWin->GetRendererMeshes()[iCurrent-1];
 
   FLOATVECTOR3 vCenter, vExtend;
-  renderer->GetVolumeAABB(vCenter, vExtend);
+  vCenter = m_pActiveRenderWin->GetRendererVolumeAABBCenter();
+  vExtend = m_pActiveRenderWin->GetRendererVolumeAABBExtents();
 
   ScaleAndBiasDlg sbd(mesh,iCurrent-1,
                       vCenter-0.5f*vExtend,
@@ -966,52 +1027,57 @@ void MainWindow::SetMeshScaleAndBias() {
 
 void MainWindow::ApplMeshTransform(ScaleAndBiasDlg* sender) {
   if (!m_pActiveRenderWin || !sender) return;
-  AbstrRenderer* renderer = m_pActiveRenderWin->GetRenderer();
 
   sender->m_pMesh->ScaleAndBias(sender->scaleVec, sender->biasVec);
-  renderer->Schedule3DWindowRedraws();
+  m_pActiveRenderWin->RendererSchedule3DWindowRedraws();
 }
 
 void MainWindow::ApplyMatrixMeshTransform(ScaleAndBiasDlg* sender) {
   if (!m_pActiveRenderWin || !sender) return;
-  AbstrRenderer* renderer = m_pActiveRenderWin->GetRenderer();
 
   sender->m_pMesh->Transform(sender->GetExpertTransform());
-  renderer->Schedule3DWindowRedraws();
+  m_pActiveRenderWin->RendererSchedule3DWindowRedraws();
 }
 
 void MainWindow::RestoreMeshTransform(ScaleAndBiasDlg* sender) {
   if (!m_pActiveRenderWin || !sender) return;
-  const Mesh* m = m_pActiveRenderWin->GetRenderer()->GetDataset().GetMeshes()[sender->m_index];
-  m_pActiveRenderWin->GetRenderer()->ReloadMesh(sender->m_index, m);
-  m_pActiveRenderWin->GetRenderer()->Schedule3DWindowRedraws();
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  LuaClassInstance ds = m_pActiveRenderWin->GetRendererDataset();
+  vector<shared_ptr<Mesh>> meshes = ss->cexecRet<vector<shared_ptr<Mesh>>>(
+      ds.fqName() + ".getMeshes");
+  const shared_ptr<Mesh> m = meshes[sender->m_index];
+  m_pActiveRenderWin->RendererReloadMesh(sender->m_index, m);
+  m_pActiveRenderWin->RendererSchedule3DWindowRedraws();
 }
 
 void MainWindow::SaveMeshTransform(ScaleAndBiasDlg* sender) {
   if (!m_pActiveRenderWin || !sender) return;
-  UVFDataset* currentDataset = dynamic_cast<UVFDataset*>(&(m_pActiveRenderWin->GetRenderer()->GetDataset()));
-  if (!currentDataset) return;
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  LuaClassInstance ds = m_pActiveRenderWin->GetRendererDataset();
+  if (ss->cexecRet<LuaDatasetProxy::DatasetType>(
+          ds.fqName() + ".getDSType") != LuaDatasetProxy::UVF) return;
 
   PleaseWaitDialog pleaseWait(this);
   pleaseWait.SetText("Saving transformation to UVF file...");
   pleaseWait.AttachLabel(&m_MasterController);
 
-  const FLOATMATRIX4& m = m_pActiveRenderWin->GetRenderer()->GetMeshes()[sender->m_index]->GetTransformFromOriginal();
+  const FLOATMATRIX4& m = m_pActiveRenderWin->GetRendererMeshes()[sender->m_index]->GetTransformFromOriginal();
 
-  m_pActiveRenderWin->GetRenderer()->SetDatasetIsInvalid(true);
+  m_pActiveRenderWin->SetDatasetIsInvalid(true);
 
-  if (!currentDataset->GeometryTransformToFile(size_t(sender->m_index),m)) {
+  if (!ss->cexecRet<bool>(ds.fqName() + ".geomTransformToFile", 
+                          static_cast<size_t>(sender->m_index), m)) {
     pleaseWait.close();
     ShowCriticalDialog("Transform Save Failed.",
              "Could not save geometry transform to the UVF file, "
              "maybe the file is write protected? For details please "
              "check the debug log ('Help | Debug Window').");
   } else {
-    m_pActiveRenderWin->GetRenderer()->GetMeshes()[sender->m_index]->DeleteTransformFromOriginal();
+    m_pActiveRenderWin->GetRendererMeshes()[sender->m_index]->DeleteTransformFromOriginal();
     pleaseWait.close();
   }
 
-    m_pActiveRenderWin->GetRenderer()->SetDatasetIsInvalid(false);
+  m_pActiveRenderWin->SetDatasetIsInvalid(false);
 }
 
 void MainWindow::SetMeshDefColor() {
@@ -1020,8 +1086,7 @@ void MainWindow::SetMeshDefColor() {
   int iCurrent = listWidget_DatasetComponents->currentRow();
   if (iCurrent < 1 || iCurrent >= listWidget_DatasetComponents->count()) return;
 
-  AbstrRenderer* renderer = m_pActiveRenderWin->GetRenderer();
-  RenderMesh* mesh = (RenderMesh*)renderer->GetMeshes()[iCurrent-1];
+  shared_ptr<RenderMesh> mesh = m_pActiveRenderWin->GetRendererMeshes()[iCurrent-1];
   
   FLOATVECTOR4 meshcolor = mesh->GetDefaultColor();
 
@@ -1038,24 +1103,29 @@ void MainWindow::SetMeshDefColor() {
     meshcolor.y = color.green()/255.0f;
     meshcolor.z = color.blue()/255.0f;
     mesh->SetDefaultColor(meshcolor);
-    renderer->Schedule3DWindowRedraws();
+    m_pActiveRenderWin->RendererSchedule3DWindowRedraws();
   }
 }
 
 void MainWindow::UpdateExplorerView(bool bRepopulateListBox) {
   if (!m_pActiveRenderWin) return;
 
-  AbstrRenderer* renderer = m_pActiveRenderWin->GetRenderer();
   if (bRepopulateListBox) {
     listWidget_DatasetComponents->clear();
     
-    QString voldesc = tr("Volume (%1)").arg(renderer->GetDataset().Name());
+    shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+    LuaClassInstance ds = m_pActiveRenderWin->GetRendererDataset();
+    
+    QString voldesc = tr("Volume (%1)").arg(
+        ss->cexecRet<const char*>(ds.fqName() + ".name"));
     listWidget_DatasetComponents->addItem(voldesc);
 
+    size_t meshListSize = m_pActiveRenderWin->GetRendererMeshes().size();
     for (size_t i = 0;
-         i<renderer->GetMeshes().size();
+         i<meshListSize;
          i++) {
-      const RenderMesh* mesh = (const RenderMesh*)renderer->GetMeshes()[i];
+      shared_ptr<const RenderMesh> mesh = 
+          m_pActiveRenderWin->GetRendererMeshes()[i];
       QString meshdesc = tr("%1 (%2)").arg(mesh->GetMeshType() == Mesh::MT_TRIANGLES ? "Triangle Mesh" : "Lines").arg(mesh->Name().c_str());
       listWidget_DatasetComponents->addItem(meshdesc);
     }
@@ -1083,7 +1153,8 @@ void MainWindow::UpdateExplorerView(bool bRepopulateListBox) {
     page_Geometry->setVisible(true);   
     stackedWidget_componentInfo->setCurrentIndex(1);
 
-    const RenderMesh* mesh = (const RenderMesh*)renderer->GetMeshes()[iCurrent-1];
+    shared_ptr<const RenderMesh> mesh = 
+        m_pActiveRenderWin->GetRendererMeshes()[iCurrent-1];
 
     size_t iVerticesPerPoly = mesh->GetVerticesPerPoly();
 
@@ -1129,32 +1200,40 @@ void MainWindow::UpdateMinMaxLODLimitLabel() {
 void MainWindow::ToggleClearViewControls() {
   if (!m_pActiveRenderWin) return;
 
-  if (m_pActiveRenderWin->GetRenderer()->SupportsClearView()) {
+  if (m_pActiveRenderWin->SupportsClearView()) {
     checkBox_ClearView->setVisible(true);
     frame_ClearView->setVisible(true);
-    checkBox_ClearView->setChecked(m_pActiveRenderWin->GetRenderer()->GetCV());
+    checkBox_ClearView->setChecked(m_pActiveRenderWin->GetClearViewEnabled());
     label_CVDisableReason->setVisible(false);
   } else {
     checkBox_ClearView->setChecked(false);
     checkBox_ClearView->setVisible(false);
     frame_ClearView->setVisible(false);
 
-    QString reason = tr("ClearView is disabled because %1").arg(m_pActiveRenderWin->GetRenderer()->ClearViewDisableReason().c_str());
+    QString reason = tr("ClearView is disabled because %1").arg(
+        m_pActiveRenderWin->GetRendererClearViewDisabledReason().c_str());
     label_CVDisableReason->setText(reason);
     label_CVDisableReason->setVisible(true);
     label_CVDisableReason->setWordWrap(true);
-    m_pActiveRenderWin->GetRenderer()->Schedule3DWindowRedraws();
+    m_pActiveRenderWin->RendererSchedule3DWindowRedraws();
   }
 }
 
 void MainWindow::ToggleClearViewControls(int iRange) {
   ToggleClearViewControls();
-  AbstrRenderer * const ren = m_pActiveRenderWin->GetRenderer();
 
-  SetFocusIsoValueSlider(int(ren->GetCVIsoValue()), iRange);
-  SetFocusSizeValueSlider(99-int(ren->GetCVSize()*9.9f));
-  SetContextScaleValueSlider(int(ren->GetCVContextScale()*10.0f));
-  SetBorderSizeValueSlider(int(99-ren->GetCVBorderScale()));
+  SetFocusIsoValueSlider(
+      int(m_pActiveRenderWin->GetRendererClearViewIsoValue()), iRange);
+  SetFocusSizeValueSlider(
+      99-int(m_pActiveRenderWin->GetRendererClearViewSize()*9.9f));
+  SetContextScaleValueSlider(
+      int(m_pActiveRenderWin->GetRendererClearViewContextScale()*10.0f));
+  SetBorderSizeValueSlider(
+      int(99-m_pActiveRenderWin->GetRendererClearViewBorderScale()));
+}
+
+void MainWindow::SetStayOpen(bool bStayOpenAfterScriptEnd) {
+  m_bStayOpenAfterScriptEnd = bStayOpenAfterScriptEnd;
 }
 
 void MainWindow::SetRescaleFactors() {
@@ -1163,7 +1242,7 @@ void MainWindow::SetRescaleFactors() {
   vfRescaleFactors.x = std::max<float>(0.001f,doubleSpinBox_RescaleX->value());
   vfRescaleFactors.y = std::max<float>(0.001f,doubleSpinBox_RescaleY->value());
   vfRescaleFactors.z = std::max<float>(0.001f,doubleSpinBox_RescaleZ->value());
-  m_pActiveRenderWin->GetRenderer()->SetRescaleFactors(vfRescaleFactors);
+  m_pActiveRenderWin->SetRendererRescaleFactors(vfRescaleFactors);
 }
 
 
@@ -1198,14 +1277,11 @@ void MainWindow::RenderWindowClosing(RenderWindow* sender) {
 
   RemoveAllLocks(sender);
 
-  disconnect(sender->GetQtWidget(), SIGNAL(WindowActive(RenderWindow*)),  this, SLOT(RenderWindowActive(RenderWindow*)));
-  disconnect(sender->GetQtWidget(), SIGNAL(WindowClosing(RenderWindow*)), this, SLOT(RenderWindowClosing(RenderWindow*)));
-  disconnect(sender->GetQtWidget(), SIGNAL(RenderWindowViewChanged(int)), this, SLOT(RenderWindowViewChanged(int)));
-  disconnect(sender->GetQtWidget(), SIGNAL(StereoDisabled()), this, SLOT(StereoDisabled()));
+  disconnect(sender->GetQtWidget());
 
-  m_1DTransferFunction->SetData(NULL, 10, NULL);
+  m_1DTransferFunction->SetData(NULL, 10, LuaClassInstance());
   m_1DTransferFunction->update();
-  m_2DTransferFunction->SetData(NULL, NULL);
+  m_2DTransferFunction->SetData(NULL, LuaClassInstance());
   m_2DTransferFunction->update();
 
   DisableAllTrans();
@@ -1246,7 +1322,7 @@ void MainWindow::CheckForRedraw() {
       r->CheckForRedraw();
     }
   }
-  m_pRedrawTimer->start(20);
+  m_pRedrawTimer->start(IV3D_TIMER_INTERVAL);
 }
 
 // ******************************************
@@ -1259,12 +1335,15 @@ void MainWindow::OpenRecentFile(){
   if (SysTools::FileExists(string(action->data().toString().toAscii()))) {
     if (action) {
       if (!LoadDataset(QStringList(action->data().toString()))) {
-        ShowCriticalDialog("Render window initialization failed.",
-                     "Could not open a render window!  This normally "
-                     "means ImageVis3D does not support your GPU.  Please"
-                     " check the debug log ('Help | Debug Window') for "
-                     "errors, and/or use 'Help | Report an Issue' to "
-                     "notify the ImageVis3D developers.");
+        if (m_bIgnoreLoadDatasetFailure == false) {
+          ShowCriticalDialog("Render window initialization failed.",
+                       "Could not open a render window!  This normally "
+                       "means ImageVis3D does not support your GPU.  Please"
+                       " check the debug log ('Help | Debug Window') for "
+                       "errors, and/or use 'Help | Report an Issue' to "
+                       "notify the ImageVis3D developers.");
+        }
+        m_bIgnoreLoadDatasetFailure = false;
       }
     }
   } else {
@@ -1304,6 +1383,7 @@ void MainWindow::OpenRecentWSFile(){
 void MainWindow::UpdateMenus() {
   bool bHasMdiChild = mdiArea->subWindowList().size() > 0;
   actionExport_Dataset->setEnabled(bHasMdiChild);
+  actionExport_Image_Stack->setEnabled(bHasMdiChild);
   actionTransfer_to_ImageVis3D_Mobile_Device->setEnabled(bHasMdiChild);
   actionAdd_Geometry_to_Data_Set->setEnabled(bHasMdiChild);
 
@@ -1468,7 +1548,10 @@ void MainWindow::ShowWelcomeScreen() {
 void MainWindow::DisplayMetadata() {
 
   if (m_pActiveRenderWin)  {
-    const vector< pair <string, string > >& metadata = m_pActiveRenderWin->GetRenderer()->GetDataset().GetMetadata();
+    shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+    LuaClassInstance ds = m_pActiveRenderWin->GetRendererDataset();
+    const vector<pair<string, string>>& metadata = 
+        ss->cexecRet<vector<pair<string, string>>>(ds.fqName() + ".getMetadata");
 
     if (metadata.size() > 0) {
       m_pMetadataDialog->setWindowIcon(windowIcon());
