@@ -45,13 +45,13 @@
 #include "../Tuvok/Controller/Controller.h"
 #include "../Tuvok/Basics/MathTools.h"
 #include "../Tuvok/Renderer/GPUMemMan/GPUMemMan.h"
+#include "../Tuvok/LuaScripting/TuvokSpecific/LuaTransferFun1DProxy.h"
 
 using namespace std;
 
 
 Q1DTransferFunction::Q1DTransferFunction(MasterController& masterController, QWidget *parent) :
   QTransferFunction(masterController, parent),
-  m_pTrans(NULL),
   m_iPaintMode(PAINT_RED | PAINT_GREEN | PAINT_BLUE | PAINT_ALPHA),
   m_iCachedHeight(0),
   m_iCachedWidth(0),
@@ -96,13 +96,17 @@ void Q1DTransferFunction::PreparePreviewData() {
   m_bBackdropCacheUptodate = false;
 }
 
-void Q1DTransferFunction::SetData(const Histogram1D* vHistogram,
+void Q1DTransferFunction::SetData(shared_ptr<const Histogram1D> vHistogram,
                                   unsigned int iMaxValue,
-                                  TransferFunction1D* pTrans) {
-  m_pTrans = pTrans;
-  if (m_pTrans == NULL || vHistogram == NULL) return;
+                                  LuaClassInstance trans) {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  // There has got to be a better way of checking if a shared_ptr is valid...
+  // We can't use nullptr yet because of our octagon ci machine.
+  if (trans.isValid(ss) == false || vHistogram == 0) return;
 
-  m_iMarkersX = iMaxValue;
+  m_trans = trans;
+
+  m_iMarkersX = std::max<unsigned int>(1,iMaxValue);
   while (m_iMarkersX > 100) m_iMarkersX /= 10;
 
   // resize internal histogram, we only consider histogram values to the maximum entry that is non-zero
@@ -186,7 +190,8 @@ void Q1DTransferFunction::DrawCoordinateSystem(QPainter& painter) {
 
 void Q1DTransferFunction::DrawHistogram(QPainter& painter) {
 
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false 
+      || m_vHistogram.GetSize() < 2) return;
 
   // compute some grid dimensions
   unsigned int iGridWidth  = width()-(m_iLeftBorder+m_iRightBorder)-3;
@@ -230,7 +235,7 @@ void Q1DTransferFunction::DrawHistogram(QPainter& painter) {
 }
 
 void Q1DTransferFunction::DrawFunctionPlots(QPainter& painter) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
 
   // compute some grid dimensions
   unsigned int iGridWidth  = width()-(m_iLeftBorder+m_iRightBorder)-3;
@@ -238,23 +243,46 @@ void Q1DTransferFunction::DrawFunctionPlots(QPainter& painter) {
 
   // draw the tranfer function as one larger polyline
   std::vector<QPointF> pointList(m_vHistogram.GetSize());
-  QPen penCurve(m_colorBorder, 1, Qt::SolidLine);
+  QPen penCurve(m_colorBorder, 2, Qt::CustomDashLine);
+
+  QVector<qreal> dashes;
+  qreal space = 4;
+  dashes << 1 << space;
+  penCurve.setDashPattern(dashes);
+
+  // Retrieve color data from Lua script class.
+  std::shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  shared_ptr<vector<FLOATVECTOR4>> cdata = 
+      ss->cexecRet<shared_ptr<vector<FLOATVECTOR4> > >(
+          m_trans.fqName() + ".getColorData");
 
   // for every component
   for (unsigned int j=0; j < 4; j++) {
     // select the color
     switch (j) {
-    case 0  : penCurve.setColor(m_colorRedLine);   break;
-    case 1  : penCurve.setColor(m_colorGreenLine); break;
-    case 2  : penCurve.setColor(m_colorBlueLine);  break;
-    default : penCurve.setColor(m_colorAlphaLine); break;
+      case 0  : penCurve.setColor(m_colorRedLine);  
+                penCurve.setDashOffset(0.0);
+                break;
+      case 1  : penCurve.setColor(m_colorGreenLine); 
+                penCurve.setDashOffset(1.0);
+                break;
+      case 2  : penCurve.setColor(m_colorBlueLine);  
+                penCurve.setDashOffset(2.0);
+                break;
+      default : penCurve.setColor(m_colorAlphaLine); 
+                penCurve.setDashOffset(3.0);
+                break;
     }
 
+    // Make sure our tfqn is as wide as our histogram, since we'll be accessing
+    // every element in the histogram.
+    assert(cdata->size() >= m_vHistogram.GetSize());
+
     // define the polyline
     for (size_t i=0; i < pointList.size(); i++) {
       pointList[i] = QPointF(
           m_iLeftBorder + 1 + float(iGridWidth) * i / (pointList.size() - 1),
-          m_iTopBorder + iGridHeight - std::max(0.0f,std::min(m_pTrans->vColorData[i][j],1.0f)) * iGridHeight
+          m_iTopBorder + iGridHeight - std::max(0.0f,std::min((*cdata)[i][j],1.0f)) * iGridHeight
       );
     }
 
@@ -267,10 +295,10 @@ void Q1DTransferFunction::DrawFunctionPlots(QPainter& painter) {
   // draw preview bar
   for (unsigned int x = 0;x<m_vHistogram.GetSize();x++) {
 
-    float r = std::max(0.0f,std::min(m_pTrans->vColorData[x][0],1.0f));
-    float g = std::max(0.0f,std::min(m_pTrans->vColorData[x][1],1.0f));
-    float b = std::max(0.0f,std::min(m_pTrans->vColorData[x][2],1.0f));
-    float a = std::max(0.0f,std::min(m_pTrans->vColorData[x][3],1.0f));
+    float r = std::max(0.0f,std::min((*cdata)[x][0],1.0f));
+    float g = std::max(0.0f,std::min((*cdata)[x][1],1.0f));
+    float b = std::max(0.0f,std::min((*cdata)[x][2],1.0f));
+    float a = std::max(0.0f,std::min((*cdata)[x][3],1.0f));
 
     m_pPreviewColor->setPixel(x,0,qRgba(int(r*255),int(g*255),int(b*255),int(a*255)));
   }
@@ -284,7 +312,7 @@ void Q1DTransferFunction::DrawFunctionPlots(QPainter& painter) {
 }
 
 void Q1DTransferFunction::mousePressEvent(QMouseEvent *event) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
 
   // call superclass method
   QWidget::mousePressEvent(event);
@@ -296,7 +324,7 @@ void Q1DTransferFunction::mousePressEvent(QMouseEvent *event) {
 }
 
 void Q1DTransferFunction::mouseReleaseEvent(QMouseEvent *event) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
 
   // call superclass method
   QWidget::mouseReleaseEvent(event);
@@ -311,12 +339,10 @@ void Q1DTransferFunction::mouseReleaseEvent(QMouseEvent *event) {
   if(m_eExecutionMode == ONRELEASE) {
     ApplyFunction();
   }
-
-  Controller::Instance().Provenance("1dtf", "set_tf_1d");
 }
 
 void Q1DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
 
   // call superclass method
   QWidget::mouseMoveEvent(event);
@@ -327,7 +353,7 @@ void Q1DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
   // compute some grid dimensions
   unsigned int iGridWidth  = width()-(m_iLeftBorder+m_iRightBorder)-3;
   unsigned int iGridHeight = height()-(m_iBottomBorder+m_iTopBorder)-2;
-  unsigned int iVectorSize = UINT32(m_vHistogram.GetSize());
+  unsigned int iVectorSize = uint32_t(m_vHistogram.GetSize());
 
   // compute position in color array
   int iCurrentIndex = int((float(event->x())-float(m_iLeftBorder)-1.0f)*float(iVectorSize-1)/float(iGridWidth));
@@ -337,6 +363,12 @@ void Q1DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
   float fValue = (float(m_iTopBorder)+float(iGridHeight)-float(event->y()))/float(iGridHeight);
   fValue    = std::min<float>(1.0f, std::max<float>(0.0f,fValue));
 
+  // Retrieve color data from Lua script class.
+  std::shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  shared_ptr<vector<FLOATVECTOR4> > cdata = 
+      ss->cexecRet<shared_ptr<vector<FLOATVECTOR4> > >(
+          m_trans.fqName() + ".getColorData");
+
   if (m_bMouseLeft) {
     // find out the range to change
     if (m_iLastIndex == -1) {
@@ -368,28 +400,28 @@ void Q1DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
     if (m_iPaintMode & PAINT_RED) {
       float _fValueMin = fValueMin;
       for (int iIndex = iIndexMin;iIndex<=iIndexMax;++iIndex) {
-        m_pTrans->vColorData[iIndex][0] = _fValueMin;
+        (*cdata)[iIndex][0] = _fValueMin;
         _fValueMin += fValueInc;
       }
     }
     if (m_iPaintMode & PAINT_GREEN) {
       float _fValueMin = fValueMin;
       for (int iIndex = iIndexMin;iIndex<=iIndexMax;++iIndex) {
-        m_pTrans->vColorData[iIndex][1] = _fValueMin;
+        (*cdata)[iIndex][1] = _fValueMin;
         _fValueMin += fValueInc;
       }
     }
     if (m_iPaintMode & PAINT_BLUE) {
       float _fValueMin = fValueMin;
       for (int iIndex = iIndexMin;iIndex<=iIndexMax;++iIndex) {
-        m_pTrans->vColorData[iIndex][2] = _fValueMin;
+        (*cdata)[iIndex][2] = _fValueMin;
         _fValueMin += fValueInc;
       }
     }
     if (m_iPaintMode & PAINT_ALPHA) {
       float _fValueMin = fValueMin;
       for (int iIndex = iIndexMin;iIndex<=iIndexMax;++iIndex) {
-        m_pTrans->vColorData[iIndex][3] = _fValueMin;
+        (*cdata)[iIndex][3] = _fValueMin;
         _fValueMin += fValueInc;
       }
     }
@@ -404,10 +436,23 @@ void Q1DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
       bool bShiftPressed = ( event->modifiers() & Qt::ShiftModifier);
 
       // set "step" function
-      if (m_iPaintMode & PAINT_RED)   m_pTrans->SetStdFunction(float(iCurrentIndex)/float(m_pTrans->GetSize()), fValue,0, bShiftPressed);
-      if (m_iPaintMode & PAINT_GREEN) m_pTrans->SetStdFunction(float(iCurrentIndex)/float(m_pTrans->GetSize()), fValue,1, bShiftPressed);
-      if (m_iPaintMode & PAINT_BLUE)  m_pTrans->SetStdFunction(float(iCurrentIndex)/float(m_pTrans->GetSize()), fValue,2, bShiftPressed);
-      if (m_iPaintMode & PAINT_ALPHA) m_pTrans->SetStdFunction(float(iCurrentIndex)/float(m_pTrans->GetSize()), fValue,3, bShiftPressed);
+      size_t cdataSize = cdata->size();
+      if (m_iPaintMode & PAINT_RED)   
+        ss->cexec(m_trans.fqName() + ".setStdFunction", 
+                  float(iCurrentIndex)/float(cdataSize), fValue, 
+                  0, bShiftPressed);
+      if (m_iPaintMode & PAINT_GREEN)
+        ss->cexec(m_trans.fqName() + ".setStdFunction", 
+                  float(iCurrentIndex)/float(cdataSize), fValue, 
+                  1, bShiftPressed);
+      if (m_iPaintMode & PAINT_BLUE)
+        ss->cexec(m_trans.fqName() + ".setStdFunction", 
+                  float(iCurrentIndex)/float(cdataSize), fValue, 
+                  2, bShiftPressed);
+      if (m_iPaintMode & PAINT_ALPHA)
+        ss->cexec(m_trans.fqName() + ".setStdFunction", 
+                  float(iCurrentIndex)/float(cdataSize), fValue, 
+                  3, bShiftPressed);
 
       // redraw this widget
       update();
@@ -419,7 +464,8 @@ void Q1DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
 
 void Q1DTransferFunction::ApplyFunction() {
   // send message to update the GLtexture
-  m_MasterController.MemMan()->Changed1DTrans(NULL, m_pTrans);
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  ss->cexec("tuvok.gpu.changed1DTrans", LuaClassInstance(), m_trans);
 }
 
 void Q1DTransferFunction::SetColor(bool bIsEnabled) {
@@ -480,7 +526,7 @@ void Q1DTransferFunction::paintEvent(QPaintEvent *event) {
   // call superclass method
   QWidget::paintEvent(event);
 
-  if (m_pTrans == NULL) {
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) {
     QPainter painter(this);
     DrawCoordinateSystem(painter);
     return;
@@ -520,14 +566,20 @@ void Q1DTransferFunction::paintEvent(QPaintEvent *event) {
   DrawFunctionPlots(painter);
 }
 
-bool Q1DTransferFunction::LoadFromFile(const QString& strFilename) {
+bool Q1DTransferFunction::QLoadFromFile(const QString& strFilename) {
+  return this->LoadFromFile(strFilename.toStdString());
+}
+
+bool Q1DTransferFunction::LoadFromFile(const std::string& strFilename) {
   // hand the load call over to the TransferFunction1D class
   size_t iSize = 0;
-  if(m_pTrans) {
-    iSize = m_pTrans->GetSize();
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if (m_trans.isValid(m_MasterController.LuaScript())) {
+    iSize = ss->cexecRet<size_t>(m_trans.fqName() + ".getSize");
   }
 
-  if( m_pTrans->Load(strFilename.toStdString(), iSize) ) {
+  if (ss->cexecRet<bool>(m_trans.fqName() + ".loadFromFileWithSize", 
+                         strFilename, iSize)) {
     PreparePreviewData();
     update();
     ApplyFunction();
@@ -537,15 +589,21 @@ bool Q1DTransferFunction::LoadFromFile(const QString& strFilename) {
 
 bool Q1DTransferFunction::AddFromFile(const QString& strFilename) {
   size_t iSize = 0;
-  if(m_pTrans) {
-    iSize = m_pTrans->GetSize();
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if (m_trans.isValid(ss)) {
+    iSize = ss->cexecRet<size_t>(m_trans.fqName() + ".getSize");
   } else return false;
 
+  // Retrieve color data from Lua script class.
+  shared_ptr<vector<FLOATVECTOR4> > cdata = 
+      ss->cexecRet<shared_ptr<vector<FLOATVECTOR4> > >(
+          m_trans.fqName() + ".getColorData");
+
   TransferFunction1D other(iSize);
   if( other.Load(strFilename.toStdString(), iSize) ) {
 
     for (size_t i = 0;i<iSize;i++)
-      m_pTrans->vColorData[i] += other.vColorData[i];
+      (*cdata)[i] += other.GetColor(i);
 
     PreparePreviewData();
     update();
@@ -558,15 +616,21 @@ bool Q1DTransferFunction::AddFromFile(const QString& strFilename) {
 
 bool Q1DTransferFunction::SubtractFromFile(const QString& strFilename) {
   size_t iSize = 0;
-  if(m_pTrans) {
-    iSize = m_pTrans->GetSize();
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if (m_trans.isValid(ss)) {
+    iSize = ss->cexecRet<size_t>(m_trans.fqName() + ".getSize");
   } else return false;
 
+  // Retrieve color data from Lua script class.
+  shared_ptr<vector<FLOATVECTOR4> > cdata = 
+      ss->cexecRet<shared_ptr<vector<FLOATVECTOR4> > >(
+          m_trans.fqName() + ".getColorData");
+
   TransferFunction1D other(iSize);
   if( other.Load(strFilename.toStdString(), iSize) ) {
 
     for (size_t i = 0;i<iSize;i++)
-      m_pTrans->vColorData[i] -= other.vColorData[i];
+      (*cdata)[i] -= other.GetColor(i);
 
     PreparePreviewData();
     update();
@@ -579,5 +643,7 @@ bool Q1DTransferFunction::SubtractFromFile(const QString& strFilename) {
 
 bool Q1DTransferFunction::SaveToFile(const QString& strFilename) {
   // hand the save call over to the TransferFunction1D class
-  return m_pTrans->Save(strFilename.toStdString());
+  std::shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  return ss->cexecRet<bool>(m_trans.fqName() + ".save", 
+                            strFilename.toStdString());
 }
