@@ -42,6 +42,9 @@
 
 #include "../Tuvok/Controller/Controller.h"
 #include "../Tuvok/Renderer/GPUMemMan/GPUMemMan.h"
+#include "../Tuvok/LuaScripting/TuvokSpecific/LuaTuvokTypes.h"
+#include "../Tuvok/LuaScripting/TuvokSpecific/LuaTransferFun1DProxy.h"
+#include "../Tuvok/LuaScripting/TuvokSpecific/LuaTransferFun2DProxy.h"
 
 #ifdef max
   #undef max
@@ -55,7 +58,6 @@ using namespace std;
 
 Q2DTransferFunction::Q2DTransferFunction(MasterController& masterController, QWidget *parent) :
   QTransferFunction(masterController, parent),
-  m_pTrans(NULL),
   m_iPaintmode(Q2DT_PAINT_NONE),
   m_iActiveSwatchIndex(-1),
   m_eTransferFunctionMode(TFM_EXPERT),
@@ -100,9 +102,13 @@ QSize Q2DTransferFunction::sizeHint() const
   return QSize(400, 400);
 }
 
-void Q2DTransferFunction::SetData(const Histogram2D* vHistogram, TransferFunction2D* pTrans) {
-  m_pTrans = pTrans;
-  if (m_pTrans == NULL) return;
+void Q2DTransferFunction::SetData(shared_ptr<const Histogram2D> vHistogram,
+                                  LuaClassInstance tf2d) {
+  m_trans = tf2d;
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  if (m_trans.isValid(ss) == false) {
+    return;
+  }
 
   // resize the histogram vector
   m_vHistogram.Resize(vHistogram->GetSize());
@@ -126,7 +132,7 @@ void Q2DTransferFunction::SetData(const Histogram2D* vHistogram, TransferFunctio
     m_vHistogram.SetLinear(i, (float(vHistogram->GetLinear(i)) - float(iMin)) / fDiff);
 
   // Upload the new TF to the GPU.
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
 
   // force the draw routine to recompute the backdrop cache
   m_bHistogramChanged = true;
@@ -136,7 +142,7 @@ void Q2DTransferFunction::SetData(const Histogram2D* vHistogram, TransferFunctio
 
 
 void Q2DTransferFunction::GenerateHistogramImage() {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
 
   // convert the histogram into an image
   // define the bitmap ...
@@ -165,7 +171,7 @@ void Q2DTransferFunction::GenerateHistogramImage() {
 }
 
 void Q2DTransferFunction::DrawHistogram(QPainter& painter) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
 
   if (m_bHistogramChanged) GenerateHistogramImage();
 
@@ -278,10 +284,10 @@ void Q2DTransferFunction::DrawPolyVertex(QPainter& painter, const INTVECTOR2& p,
 }
 
 void Q2DTransferFunction::DrawSwatcheDecoration(QPainter& painter) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
 
   // if we start in simple mode we need to build the swatch type list in the first draw call
-  if (m_vSimpleSwatchInfo.size() != m_pTrans->m_Swatches.size()) UpdateSwatchTypes();
+  if (m_vSimpleSwatchInfo.size() != GetSwatchCount()) UpdateSwatchTypes();
 
   painter.setRenderHint(painter.Antialiasing, true);
 //  painter.translate(+0.5, +0.5);  /// \todo check if we need this
@@ -300,10 +306,12 @@ void Q2DTransferFunction::DrawSwatcheDecoration(QPainter& painter) {
 
   QBrush solidBrush = QBrush(m_colorSwatchBorderCircle, Qt::SolidPattern);
 
+  // Obtain swatch vector from Lua.
+  shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
 
   // render swatches
-  for (size_t i = 0;i<m_pTrans->m_Swatches.size();i++) {
-    TFPolygon& currentSwatch = m_pTrans->m_Swatches[i];
+  for (size_t i = 0;i<GetSwatchCount();i++) {
+    const TFPolygon& currentSwatch = (*swatches)[i];
 
     std::vector<QPoint> pointList(currentSwatch.pPoints.size());
     for (size_t j = 0;j<currentSwatch.pPoints.size();j++) {
@@ -416,7 +424,7 @@ void Q2DTransferFunction::DrawSwatcheDecoration(QPainter& painter) {
 }
 
 void Q2DTransferFunction::DrawSwatches(QPainter& painter) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
 
   QPen noBorderPen(Qt::NoPen);
   QPen circlePen(m_colorSwatchBorderCircle, m_iSwatchBorderSize, Qt::SolidLine);
@@ -428,9 +436,11 @@ void Q2DTransferFunction::DrawSwatches(QPainter& painter) {
 
   QBrush solidBrush = QBrush(m_colorSwatchBorderCircle, Qt::SolidPattern);
 
+  shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+
   // render swatches
-  for (size_t i = 0;i<m_pTrans->m_Swatches.size();i++) {
-    TFPolygon& currentSwatch = m_pTrans->m_Swatches[i];
+  for (size_t i = 0;i<GetSwatchCount();i++) {
+    const TFPolygon& currentSwatch = (*swatches)[i];
 
     std::vector<QPoint> pointList(currentSwatch.pPoints.size());
     for (size_t j = 0;j<currentSwatch.pPoints.size();j++) {
@@ -496,10 +506,13 @@ void Q2DTransferFunction::DragInit(INTVECTOR2 vMousePressPos, Qt::MouseButton mo
   m_vMousePressPos = vMousePressPos;
   m_mouseButton = mouseButton;
 
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+
   if (m_eTransferFunctionMode == TFM_EXPERT) {
 
-    if (m_iActiveSwatchIndex >= 0 && m_iActiveSwatchIndex<int(m_pTrans->m_Swatches.size())) {
-      TFPolygon& currentSwatch = m_pTrans->m_Swatches[m_iActiveSwatchIndex];
+    shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+    if (m_iActiveSwatchIndex >= 0 && m_iActiveSwatchIndex<int(GetSwatchCount())) {
+      const TFPolygon& currentSwatch = (*swatches)[m_iActiveSwatchIndex];
 
       // left mouse drags points around
       if (mouseButton == Qt::LeftButton) {
@@ -557,7 +570,9 @@ void Q2DTransferFunction::DragInit(INTVECTOR2 vMousePressPos, Qt::MouseButton mo
           if (currentSwatch.pPoints.size() > 3) {
             INTVECTOR2 vPixelDist = Normalized2Offscreen(vfP)-Normalized2Offscreen(A);
             if ( sqrt( float(vPixelDist.x*vPixelDist.x+vPixelDist.y*vPixelDist.y)) <= m_iSwatchBorderSize*3) {
-              currentSwatch.pPoints.erase(currentSwatch.pPoints.begin()+j);
+              ss->cexec(m_trans.fqName() + ".swatchErasePoint", 
+                        static_cast<size_t>(m_iActiveSwatchIndex),
+                        static_cast<size_t>(j));
               iInsertIndex = -1;
               emit SwatchChange();
               break;
@@ -586,7 +601,10 @@ void Q2DTransferFunction::DragInit(INTVECTOR2 vMousePressPos, Qt::MouseButton mo
         }
 
         if (iInsertIndex >= 0) {
-          currentSwatch.pPoints.insert(currentSwatch.pPoints.begin()+iInsertIndex, vfInserCoord);
+          ss->cexec(m_trans.fqName() + ".swatchInsertPoint",
+                    static_cast<size_t>(m_iActiveSwatchIndex),
+                    static_cast<size_t>(iInsertIndex),
+                    vfInserCoord);
           emit SwatchChange();
         }
       }
@@ -652,8 +670,11 @@ bool Q2DTransferFunction::PointInPolygon(const FLOATVECTOR2& point, const TFPoly
 
 int Q2DTransferFunction::PickEdge(const FLOATVECTOR2& pickPos, int& iEdgeIndex) const {
   FLOATVECTOR2 pixelPickPos = FLOATVECTOR2(Normalized2Screen(pickPos));
-  for (size_t i = 0;i<m_pTrans->m_Swatches.size();i++) {
-    TFPolygon& currentSwatch = m_pTrans->m_Swatches[i];
+
+  shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+  for (vector<TFPolygon>::const_iterator it = swatches->begin(); 
+       it != swatches->end(); ++it) {
+    const TFPolygon& currentSwatch = *it;
 
     for (size_t j = 0;j<currentSwatch.pPoints.size();j++) {
       FLOATVECTOR2 A = FLOATVECTOR2(Normalized2Screen(currentSwatch.pPoints[j]));
@@ -672,7 +693,7 @@ int Q2DTransferFunction::PickEdge(const FLOATVECTOR2& pickPos, int& iEdgeIndex)
 
       if (fDist <= max<float>(m_iSwatchBorderSize,4.0f)) {  // give the user at least four pixel to pick
         iEdgeIndex = int(j);
-        return int(i);
+        return static_cast<int>(std::distance(swatches->begin(), it));
       }
     }
 
@@ -681,8 +702,10 @@ int Q2DTransferFunction::PickEdge(const FLOATVECTOR2& pickPos, int& iEdgeIndex)
 }
 
 int Q2DTransferFunction::PickGradient(const FLOATVECTOR2& pickPos) const {
-  for (size_t i = 0;i<m_pTrans->m_Swatches.size();i++) {
-    TFPolygon& currentSwatch = m_pTrans->m_Swatches[i];
+  shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+  for (vector<TFPolygon>::const_iterator it = swatches->begin(); 
+       it != swatches->end(); ++it) {
+    const TFPolygon& currentSwatch = *it;
 
     // only consider 3 stop gradients
     if (currentSwatch.pGradientStops.size() != 3) continue;
@@ -691,22 +714,26 @@ int Q2DTransferFunction::PickGradient(const FLOATVECTOR2& pickPos) const {
                      currentSwatch.pGradientCoords[1]*currentSwatch.pGradientStops[1].first;
     INTVECTOR2 vPixelDist = Normalized2Screen(pickPos)-Normalized2Screen(A);
     if ( sqrt( float(vPixelDist.x*vPixelDist.x+vPixelDist.y*vPixelDist.y)) <= m_iSwatchBorderSize*3) {
-      return int(i);
+      return static_cast<int>(std::distance(swatches->begin(), it));
     }
   }
   return -1;
 }
 
 int Q2DTransferFunction::PickVertex(const FLOATVECTOR2& pickPos, int& iVertexIndex) const {
-  for (size_t i = 0;i<m_pTrans->m_Swatches.size();i++) {
-    TFPolygon& currentSwatch = m_pTrans->m_Swatches[i];
+  shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+  for (vector<TFPolygon>::const_iterator it = swatches->begin(); 
+       it != swatches->end(); ++it) {
+    const TFPolygon& currentSwatch = *it;
 
-    for (size_t j = 0;j<currentSwatch.pPoints.size();j++) {
-      FLOATVECTOR2 A = currentSwatch.pPoints[j];
+    for (vector<FLOATVECTOR2>::const_iterator itPnt = currentSwatch.pPoints.begin();
+         itPnt != currentSwatch.pPoints.end(); ++itPnt) {
+      FLOATVECTOR2 A = *itPnt;
       INTVECTOR2 vPixelDist = Normalized2Screen(pickPos)-Normalized2Screen(A);
       if ( sqrt( float(vPixelDist.x*vPixelDist.x+vPixelDist.y*vPixelDist.y)) <= m_iSwatchBorderSize*3) {
-        iVertexIndex = int(j);
-        return int(i);
+        iVertexIndex = static_cast<int>(
+            std::distance(currentSwatch.pPoints.begin(), itPnt));
+        return static_cast<int>(std::distance(swatches->begin(), it));
       }
     }
   }
@@ -714,17 +741,19 @@ int Q2DTransferFunction::PickVertex(const FLOATVECTOR2& pickPos, int& iVertexInd
 }
 
 int Q2DTransferFunction::PickSwatch(const FLOATVECTOR2& pickPos) const {
-  for (size_t i = 0;i<m_pTrans->m_Swatches.size();i++) {
-    TFPolygon& currentSwatch = m_pTrans->m_Swatches[i];
+  shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+  for (vector<TFPolygon>::const_iterator it = swatches->begin(); 
+       it != swatches->end(); ++it) {
+    const TFPolygon& currentSwatch = *it;
     if (PointInPolygon(pickPos, currentSwatch)) {
-      return int(i);
+      return static_cast<int>(std::distance(swatches->begin(), it));
     }
   }
   return -1;
 }
 
 void Q2DTransferFunction::mousePressEvent(QMouseEvent *event) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
   // call superclass method
   QWidget::mousePressEvent(event);
 
@@ -766,7 +795,7 @@ void Q2DTransferFunction::wheelEvent(QWheelEvent *event) {
 
 
 void Q2DTransferFunction::mouseReleaseEvent(QMouseEvent *event) {
-  if (m_pTrans == NULL) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
   // call superclass method
   QWidget::mouseReleaseEvent(event);
 
@@ -783,13 +812,12 @@ void Q2DTransferFunction::mouseReleaseEvent(QMouseEvent *event) {
   if(m_eExecutionMode == ONRELEASE) {
     ApplyFunction();
   }
-
-  Controller::Instance().Provenance("2dtf", "set_tf_2d");
 }
 
 void Q2DTransferFunction::ApplyFunction() {
   // send message to update the GLtexture
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
 }
 
 
@@ -810,7 +838,8 @@ void Q2DTransferFunction::RecomputeLowerPseudoTrisPoints(TFPolygon& currentSwatc
 }
 
 void Q2DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
-  if (m_pTrans == NULL) return;
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if (m_trans.isValid(ss) == false) return;
   // call superclass method
   QWidget::mouseMoveEvent(event);
 
@@ -854,7 +883,12 @@ void Q2DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
 
     if (m_eTransferFunctionMode == TFM_EXPERT) {
 
-      TFPolygon& currentSwatch = m_pTrans->m_Swatches[m_iActiveSwatchIndex];
+      // current swatch will be used as a staging area for the Lua scripting
+      // system. When we are done modifying the current swatch, we will call
+      // swatchUpdate. This will allow us to record the update for provenance 
+      // purposes.
+      shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+      TFPolygon currentSwatch = (*swatches)[m_iActiveSwatchIndex];
 
       if (m_bDraggingAll)  {
         switch (m_eDragMode) {
@@ -902,9 +936,13 @@ void Q2DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
           currentSwatch.pGradientCoords[m_iGradSelIndex] += vfDelta;
         }
       }
+
+      ss->cexec(m_trans.fqName() + ".swatchUpdate", 
+                static_cast<size_t>(m_iActiveSwatchIndex), currentSwatch);
     } else {
       if (m_iActiveSwatchIndex < 0) return;
-      TFPolygon& currentSwatch = m_pTrans->m_Swatches[m_iActiveSwatchIndex];
+      shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+      TFPolygon currentSwatch = (*swatches)[m_iActiveSwatchIndex];
 
       switch (m_eSimpleDragMode) {
         case SDM_NONE:
@@ -1028,6 +1066,9 @@ void Q2DTransferFunction::mouseMoveEvent(QMouseEvent *event) {
           }
         } break;
       }
+
+      ss->cexec(m_trans.fqName() + ".swatchUpdate", 
+                static_cast<size_t>(m_iActiveSwatchIndex), currentSwatch);
     }
 
     m_vMousePressPos = vMouseCurrentPos;
@@ -1098,25 +1139,36 @@ void Q2DTransferFunction::Draw1DTrans(QPainter& painter) {
   QRectF imageRect(0, 0,
                   painter.viewport().width(), painter.viewport().height());
 
-  QRectF source(m_vZoomWindow.x * m_pTrans->Get1DTransImage().width(),
-                m_vZoomWindow.y * m_pTrans->Get1DTransImage().height(),
-                m_vZoomWindow.z * m_pTrans->Get1DTransImage().width(),
-                m_vZoomWindow.w * m_pTrans->Get1DTransImage().height());
-  painter.drawImage(imageRect,m_pTrans->Get1DTransImage(), source);
+  // This hack exists because IO has a dependency on QT.
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  LuaTransferFun2DProxy* prox = m_trans.getRawPointer<LuaTransferFun2DProxy>(ss);
+  TransferFunction2D* hackTF2D = prox->get2DTransferFunction();
+  if (hackTF2D == NULL) return;
+
+  QRectF source(m_vZoomWindow.x * hackTF2D->Get1DTransImage().width(),
+                m_vZoomWindow.y * hackTF2D->Get1DTransImage().height(),
+                m_vZoomWindow.z * hackTF2D->Get1DTransImage().width(),
+                m_vZoomWindow.w * hackTF2D->Get1DTransImage().height());
+  painter.drawImage(imageRect,hackTF2D->Get1DTransImage(), source);
 }
 
 
-void Q2DTransferFunction::ComputeCachedImageSize(UINT32 &w , UINT32 &h) const {
+void
+Q2DTransferFunction::ComputeCachedImageSize(uint32_t &w , uint32_t &h) const {
   // find an image size that has the same aspect ratio as the histogram
   // but is no smaller than the widget
 
-  w = UINT32(width());
-  float fRatio = float(m_pTrans->GetRenderSize().x) / float(m_pTrans->GetRenderSize().y);
-  h = static_cast<UINT32>(w / fRatio);
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  VECTOR2<size_t> rendererSize = ss->cexecRet<VECTOR2<size_t>>(
+      m_trans.fqName() + ".getRenderSize");
 
-  if (h > UINT32(height())) {
-    h = UINT32(height());
-    w = static_cast<UINT32>(h * fRatio);
+  w = uint32_t(width());
+  float fRatio = float(rendererSize.x) / float(rendererSize.y);
+  h = static_cast<uint32_t>(w / fRatio);
+
+  if (h > uint32_t(height())) {
+    h = uint32_t(height());
+    w = static_cast<uint32_t>(h * fRatio);
   }
 }
 
@@ -1124,13 +1176,13 @@ void Q2DTransferFunction::paintEvent(QPaintEvent *event) {
   // call superclass method
   QWidget::paintEvent(event);
 
-  if (m_pTrans == NULL) {
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) {
     QPainter painter(this);
     ClearToBlack(painter);
     return;
   }
 
-  UINT32 w,h;
+  uint32_t w,h;
   ComputeCachedImageSize(w,h);
 
   // as drawing the histogram can become quite expensive we'll cache it in an image and only redraw if needed
@@ -1181,11 +1233,15 @@ void Q2DTransferFunction::paintEvent(QPaintEvent *event) {
 
 bool Q2DTransferFunction::LoadFromFile(const QString& strFilename) {
   // hand the load call over to the TransferFunction1D class
-  if( m_pTrans->Load(strFilename.toStdString(), m_pTrans->GetSize() ) ) {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  VECTOR2<size_t> transSize = ss->cexecRet<VECTOR2<size_t>>(m_trans.fqName() +
+                                                            ".getSize");
+  if (ss->cexecRet<bool>(m_trans.fqName() + ".loadWithSize", 
+                         strFilename.toStdString(), transSize)) {
     m_iActiveSwatchIndex = 0;
     m_bBackdropCacheUptodate = false;
     update();
-    m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+    ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
     emit SwatchChange();
     return true;
   } else return false;
@@ -1193,19 +1249,24 @@ bool Q2DTransferFunction::LoadFromFile(const QString& strFilename) {
 
 bool Q2DTransferFunction::SaveToFile(const QString& strFilename) {
   // hand the save call over to the TransferFunction1D class
-  return m_pTrans->Save(strFilename.toStdString());
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  return ss->cexecRet<bool>(m_trans.fqName() + ".save", 
+                            strFilename.toStdString());
 }
 
 
-void Q2DTransferFunction::Set1DTrans(const TransferFunction1D* p1DTrans) {
-  m_pTrans->Update1DTrans(p1DTrans);
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+void Q2DTransferFunction::Set1DTrans(LuaClassInstance inst) {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  ss->cexec(m_trans.fqName() + ".update1DTrans", inst);
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
   m_bBackdropCacheUptodate = false;
   update();
 }
 
 void Q2DTransferFunction::Transfer2DSetActiveSwatch(const int iActiveSwatch) {
-  if (iActiveSwatch == -1 && m_pTrans && m_pTrans->m_Swatches.size() > 0) return;
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  if (iActiveSwatch == -1 && m_trans.isValid(ss) && 
+      ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount") > 0) return;
   m_iActiveSwatchIndex = iActiveSwatch;
   update();
 }
@@ -1213,6 +1274,8 @@ void Q2DTransferFunction::Transfer2DSetActiveSwatch(const int iActiveSwatch) {
 void Q2DTransferFunction::Transfer2DAddCircleSwatch() {
   TFPolygon newSwatch;
 
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+
   FLOATVECTOR2 vPoint(m_vZoomWindow.x + 0.8f*m_vZoomWindow.z, m_vZoomWindow.y + 0.8f*m_vZoomWindow.w);
   FLOATVECTOR2 vCenter(m_vZoomWindow.x + 0.5f*m_vZoomWindow.z, m_vZoomWindow.y + 0.5f*m_vZoomWindow.w);
   unsigned int iNumberOfSegments = 20;
@@ -1224,15 +1287,18 @@ void Q2DTransferFunction::Transfer2DAddCircleSwatch() {
   newSwatch.pGradientCoords[0] = FLOATVECTOR2(m_vZoomWindow.x + 0.1f*m_vZoomWindow.z, m_vZoomWindow.y + 0.5f*m_vZoomWindow.w);
   newSwatch.pGradientCoords[1] = FLOATVECTOR2(m_vZoomWindow.x + 0.9f*m_vZoomWindow.z, m_vZoomWindow.y + 0.5f*m_vZoomWindow.w);
 
-  GradientStop g1(0,FLOATVECTOR4(0,0,0,0)),g2(0.5f,FLOATVECTOR4(1,1,1,1)),g3(1,FLOATVECTOR4(0,0,0,0));
+  GradientStop g1(0.0f, FLOATVECTOR4(0.0f,0.0f,0.0f,0.0f)),
+               g2(0.5f, FLOATVECTOR4(1.0f,1.0f,1.0f,1.0f)),
+               g3(1.0f, FLOATVECTOR4(0.0f,0.0f,0.0f,0.0f));
   newSwatch.pGradientStops.push_back(g1);
   newSwatch.pGradientStops.push_back(g2);
   newSwatch.pGradientStops.push_back(g3);
 
-  m_pTrans->m_Swatches.push_back(newSwatch);
+  ss->cexec(m_trans.fqName() + ".swatchPushBack", newSwatch);
 
-  m_iActiveSwatchIndex = int(m_pTrans->m_Swatches.size()-1);
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  m_iActiveSwatchIndex = static_cast<int>(
+      ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount")-1);
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
   emit SwatchChange();
 }
 
@@ -1247,15 +1313,19 @@ void Q2DTransferFunction::Transfer2DAddSwatch() {
   newSwatch.pGradientCoords[0] = FLOATVECTOR2(m_vZoomWindow.x + 0.3f*m_vZoomWindow.z, m_vZoomWindow.y + 0.5f*m_vZoomWindow.w);
   newSwatch.pGradientCoords[1] = FLOATVECTOR2(m_vZoomWindow.x + 0.7f*m_vZoomWindow.z, m_vZoomWindow.y + 0.5f*m_vZoomWindow.w);
 
-  GradientStop g1(0,FLOATVECTOR4(0,0,0,0)),g2(0.5f,FLOATVECTOR4(1,1,1,1)),g3(1,FLOATVECTOR4(0,0,0,0));
+  GradientStop g1(0.0f,FLOATVECTOR4(0.0f,0.0f,0.0f,0.0f)),
+               g2(0.5f,FLOATVECTOR4(1.0f,1.0f,1.0f,1.0f)),
+               g3(1.0f,FLOATVECTOR4(0.0f,0.0f,0.0f,0.0f));
   newSwatch.pGradientStops.push_back(g1);
   newSwatch.pGradientStops.push_back(g2);
   newSwatch.pGradientStops.push_back(g3);
 
-  m_pTrans->m_Swatches.push_back(newSwatch);
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  ss->cexec(m_trans.fqName() + ".swatchPushBack", newSwatch);
 
-  m_iActiveSwatchIndex = int(m_pTrans->m_Swatches.size()-1);
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  m_iActiveSwatchIndex = static_cast<int>(
+      ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount")-1);
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
   emit SwatchChange();
 }
 
@@ -1270,25 +1340,30 @@ void Q2DTransferFunction::Transfer2DAddRectangleSwatch() {
   newSwatch.pGradientCoords[0] = (newSwatch.pPoints[0]+newSwatch.pPoints[1])/2.0f;
   newSwatch.pGradientCoords[1] = (newSwatch.pPoints[2]+newSwatch.pPoints[3])/2.0f;
 
-  GradientStop g1(0,FLOATVECTOR4(0,0,0,0)),g2(0.5,FLOATVECTOR4(rand()/float(RAND_MAX),rand()/float(RAND_MAX),rand()/float(RAND_MAX),1)),g3(1,FLOATVECTOR4(0,0,0,0));
+  GradientStop g1(0.0f, FLOATVECTOR4(0.0f,0.0f,0.0f,0.0f)),
+               g2(0.5f,
+                  FLOATVECTOR4(rand()/float(RAND_MAX), rand()/float(RAND_MAX),
+                               rand()/float(RAND_MAX), 1.0f)),
+               g3(1.0f, FLOATVECTOR4(0.0f,0.0f,0.0f,0.0f));
   newSwatch.pGradientStops.push_back(g1);
   newSwatch.pGradientStops.push_back(g2);
   newSwatch.pGradientStops.push_back(g3);
 
-  m_pTrans->m_Swatches.push_back(newSwatch);
-
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  ss->cexec(m_trans.fqName() + ".swatchPushBack", newSwatch);
   UpdateSwatchTypes();
 
-  m_iActiveSwatchIndex = int(m_pTrans->m_Swatches.size()-1);
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  m_iActiveSwatchIndex = static_cast<int>(
+      ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount")-1);
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
   emit SwatchChange();
 }
 
 
 void Q2DTransferFunction::ComputeGradientForPseudoTris(TFPolygon& swatch, const FLOATVECTOR4& color) {
-  GradientStop g1(0,FLOATVECTOR4(0,0,0,0)),
+  GradientStop g1(0.0f,FLOATVECTOR4(0.0f,0.0f,0.0f,0.0f)),
                g2(0.5f,color),
-               g3(1.0f,FLOATVECTOR4(0,0,0,0));
+               g3(1.0f,FLOATVECTOR4(0.0f,0.0f,0.0f,0.0f));
   swatch.pGradientStops.clear();
   swatch.pGradientStops.push_back(g1);
   swatch.pGradientStops.push_back(g2);
@@ -1325,99 +1400,160 @@ void Q2DTransferFunction::Transfer2DAddPseudoTrisSwatch() {
   newSwatch.pPoints.push_back(p3);
   ComputeGradientForPseudoTris(newSwatch, FLOATVECTOR4(rand()/float(RAND_MAX),rand()/float(RAND_MAX),rand()/float(RAND_MAX),1));
 
-  m_pTrans->m_Swatches.push_back(newSwatch);
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  ss->cexec(m_trans.fqName() + ".swatchPushBack", newSwatch);
   UpdateSwatchTypes();
 
-  m_iActiveSwatchIndex = int(m_pTrans->m_Swatches.size()-1);
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  m_iActiveSwatchIndex = static_cast<int>(
+      ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount")-1);
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
   emit SwatchChange();
 }
 
 
 void Q2DTransferFunction::Transfer2DDeleteSwatch(){
   if (m_iActiveSwatchIndex != -1) {
-    m_pTrans->m_Swatches.erase(m_pTrans->m_Swatches.begin()+m_iActiveSwatchIndex);
+    shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+    ss->cexec(m_trans.fqName() + ".swatchErase", 
+              static_cast<size_t>(m_iActiveSwatchIndex));
 
-    m_iActiveSwatchIndex = min<int>(m_iActiveSwatchIndex, int(m_pTrans->m_Swatches.size()-1));
-    m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+    int lastSwatchIndex = static_cast<int>(
+        ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount")-1);
+    m_iActiveSwatchIndex = min<int>(m_iActiveSwatchIndex, lastSwatchIndex);
+    ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
     emit SwatchChange();
   }
 }
 
 void Q2DTransferFunction::Transfer2DUpSwatch(){
   if (m_iActiveSwatchIndex > 0) {
-    TFPolygon tmp = m_pTrans->m_Swatches[m_iActiveSwatchIndex-1];
-    m_pTrans->m_Swatches[m_iActiveSwatchIndex-1] = m_pTrans->m_Swatches[m_iActiveSwatchIndex];
-    m_pTrans->m_Swatches[m_iActiveSwatchIndex] = tmp;
+    shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+    shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+    TFPolygon tmp = (*swatches)[m_iActiveSwatchIndex-1];
+
+    ss->cexec(m_trans.fqName() + ".swatchUpdate", 
+              static_cast<size_t>(m_iActiveSwatchIndex-1),
+              (*swatches)[m_iActiveSwatchIndex]);
+    ss->cexec(m_trans.fqName() + ".swatchUpdate",
+              static_cast<size_t>(m_iActiveSwatchIndex),
+              tmp);
 
     m_iActiveSwatchIndex--;
-    m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+    ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
     emit SwatchChange();
   }
 }
 
 void Q2DTransferFunction::Transfer2DDownSwatch(){
-  if (m_iActiveSwatchIndex >= 0 && m_iActiveSwatchIndex < int(m_pTrans->m_Swatches.size()-1)) {
-    TFPolygon tmp = m_pTrans->m_Swatches[m_iActiveSwatchIndex+1];
-    m_pTrans->m_Swatches[m_iActiveSwatchIndex+1] = m_pTrans->m_Swatches[m_iActiveSwatchIndex];
-    m_pTrans->m_Swatches[m_iActiveSwatchIndex] = tmp;
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  int lastSwatchIndex = static_cast<int>(
+      ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount")-1);
+  if (m_iActiveSwatchIndex >= 0 && m_iActiveSwatchIndex < lastSwatchIndex) {
+    shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+    TFPolygon tmp = (*swatches)[m_iActiveSwatchIndex+1];
+
+    ss->cexec(m_trans.fqName() + ".swatchUpdate", 
+              static_cast<size_t>(m_iActiveSwatchIndex+1),
+              (*swatches)[m_iActiveSwatchIndex]);
+    ss->cexec(m_trans.fqName() + ".swatchUpdate",
+              static_cast<size_t>(m_iActiveSwatchIndex),
+              tmp);
 
     m_iActiveSwatchIndex++;
-    m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+    ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
     emit SwatchChange();
   }
 }
 
 void Q2DTransferFunction::SetActiveGradientType(bool bRadial) {
-  if(static_cast<size_t>(m_iActiveSwatchIndex) <
-     m_pTrans->m_Swatches.size()) {
-       if (m_pTrans->m_Swatches[m_iActiveSwatchIndex].bRadial != bRadial) {
-          m_pTrans->m_Swatches[m_iActiveSwatchIndex].bRadial = bRadial;
-          m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
-          update();
-       }
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  size_t swatchCount = 
+      ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount");
+  if (static_cast<size_t>(m_iActiveSwatchIndex) < swatchCount) {
+    if (ss->cexecRet<bool>(m_trans.fqName() + ".swatchIsRadial", 
+                           static_cast<size_t>(m_iActiveSwatchIndex)) 
+        != bRadial) {
+      ss->cexec(m_trans.fqName() + ".swatchSetRadial", bRadial);
+      ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
+      update();
+    }
   }
 }
 
 void Q2DTransferFunction::AddGradient(GradientStop stop) {
-  for (std::vector< GradientStop >::iterator i = m_pTrans->m_Swatches[m_iActiveSwatchIndex].pGradientStops.begin();i<m_pTrans->m_Swatches[m_iActiveSwatchIndex].pGradientStops.end();i++) {
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+  const TFPolygon& currentSwatch = (*swatches)[m_iActiveSwatchIndex];
+  for (std::vector< GradientStop >::const_iterator i = 
+       currentSwatch.pGradientStops.begin();
+       i<currentSwatch.pGradientStops.end();
+       i++) {
     if (i->first > stop.first) {
-      m_pTrans->m_Swatches[m_iActiveSwatchIndex].pGradientStops.insert(i, stop);
+      // Do not proceed iteration after making this Lua call. If the 
+      // pGradientStops vector resized itself, our iterator is invalid.
+      ss->cexec(m_trans.fqName() + ".swatchInsertGradient", 
+               static_cast<size_t>(m_iActiveSwatchIndex),
+               static_cast<size_t>(
+                   std::distance(currentSwatch.pGradientStops.begin(), i)), 
+               stop);
+      // We have modified the transfer function, make sure to display the 
+      // results...
+      ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
+      update();
       return;
     }
   }
-  m_pTrans->m_Swatches[m_iActiveSwatchIndex].pGradientStops.push_back(stop);
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  ss->cexec(m_trans.fqName() + ".swatchPushBackGradient",
+            static_cast<size_t>(m_iActiveSwatchIndex),
+            stop);
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
   update();
 }
 
 void Q2DTransferFunction::DeleteGradient(unsigned int i) {
-  m_pTrans->m_Swatches[m_iActiveSwatchIndex].pGradientStops.erase(m_pTrans->m_Swatches[m_iActiveSwatchIndex].pGradientStops.begin()+i);
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  shared_ptr<LuaScripting> ss(m_MasterController.LuaScript());
+  ss->cexec(m_trans.fqName() + ".swatchEraseGradient",
+            static_cast<size_t>(m_iActiveSwatchIndex),
+            static_cast<size_t>(i));
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
   update();
 }
 
 void Q2DTransferFunction::SetGradient(unsigned int i, GradientStop stop) {
-  m_pTrans->m_Swatches[m_iActiveSwatchIndex].pGradientStops[i] = stop;
-  m_MasterController.MemMan()->Changed2DTrans(NULL, m_pTrans);
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  ss->cexec(m_trans.fqName() + ".swatchUpdateGradient",
+            static_cast<size_t>(m_iActiveSwatchIndex),
+            static_cast<size_t>(i),
+            stop);
+  ss->cexec("tuvok.gpu.changed2DTrans", LuaClassInstance(), m_trans);
   update();
 }
 
 
 void Q2DTransferFunction::UpdateSwatchType(size_t i) {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
   std::string desc = m_vSimpleSwatchInfo[i].m_strDesc;
-  m_vSimpleSwatchInfo[i] = ClassifySwatch(m_pTrans->m_Swatches[i]);
+
+  shared_ptr<const vector<TFPolygon>> swatches = GetSwatches();
+  TFPolygon tmp = (*swatches)[i];
+  m_vSimpleSwatchInfo[i] = ClassifySwatch(tmp);
+  // Tmp might have been mutated by ClassifySwatch, so we need to inform
+  // TransferFunction2D that it happened.
+  ss->cexec(m_trans.fqName() + ".swatchUpdate", i, tmp);
 
   if (desc != m_vSimpleSwatchInfo[i].m_strDesc) emit SwatchTypeChange(int(i));
 }
 
 
 void Q2DTransferFunction::UpdateSwatchTypes() {
-  if (!m_pTrans) return;
+  if (m_trans.isValid(m_MasterController.LuaScript()) == false) return;
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  size_t swatchCount = 
+      ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount");
 
   m_vSimpleSwatchInfo.clear();
-  m_vSimpleSwatchInfo.resize(m_pTrans->m_Swatches.size());
-  for (size_t i = 0;i<m_pTrans->m_Swatches.size();i++) {
+  m_vSimpleSwatchInfo.resize(swatchCount);
+  for (size_t i = 0;i<swatchCount;i++) {
     UpdateSwatchType(i);
   }
 }
@@ -1440,3 +1576,63 @@ std::string Q2DTransferFunction::GetSwatchDesciption() const {
     return "";
   }
 }
+
+size_t Q2DTransferFunction::GetSwatchCount() {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if (m_trans.isValid(ss)) {
+    return ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetCount");
+  } else {
+    return 0;
+  }
+}
+
+size_t Q2DTransferFunction::GetSwatchSize(unsigned int i) {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if (m_trans.isValid(ss)) {
+    return ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetNumPoints", 
+                                static_cast<size_t>(i));
+  } else {
+    return 0;
+  }
+}
+
+bool Q2DTransferFunction::GetActiveGradientType() {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if ( !m_trans.isValid(ss)
+      || static_cast<size_t>(m_iActiveSwatchIndex) >= GetSwatchCount()) {
+    return false;
+  }
+  return ss->cexecRet<bool>(m_trans.fqName() + ".swatchIsRadial", 
+                            static_cast<size_t>(m_iActiveSwatchIndex));
+}
+
+size_t Q2DTransferFunction::GetGradientCount() {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if ( !m_trans.isValid(ss)
+      || static_cast<size_t>(m_iActiveSwatchIndex) >= GetSwatchCount()) {
+    return 0;
+  }
+ return ss->cexecRet<size_t>(m_trans.fqName() + ".swatchGetGradientCount", 
+                             static_cast<size_t>(m_iActiveSwatchIndex));
+}
+
+GradientStop Q2DTransferFunction::GetGradient(unsigned int i) {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  if ( !m_trans.isValid(ss)
+      || static_cast<size_t>(m_iActiveSwatchIndex) >= GetSwatchCount()) {
+    // need to return something invalid.
+    return GradientStop(0.0f, FLOATVECTOR4(0,0,0,0));
+  }
+  return ss->cexecRet<GradientStop>(m_trans.fqName() + ".swatchGetGradient", 
+                                    static_cast<size_t>(m_iActiveSwatchIndex),
+                                    static_cast<size_t>(i));
+}
+
+
+shared_ptr<const vector<TFPolygon>> Q2DTransferFunction::GetSwatches() const {
+  shared_ptr<LuaScripting> ss = m_MasterController.LuaScript();
+  shared_ptr<const vector<TFPolygon>> swatches = 
+      ss->cexecRet<shared_ptr<const vector<TFPolygon>>>(m_trans.fqName() +
+                                                        ".swatchGet");
+  return swatches;
+}
