@@ -53,7 +53,8 @@ SettingsDlg::SettingsDlg(bool bWarnAPIChange, MasterController& MasterController
   m_MasterController(MasterController),
   m_bInit(true),
   m_InitialGPUMemMax(0),
-  m_bWarnAPIChange(bWarnAPIChange)
+  m_bWarnAPIChange(bWarnAPIChange),
+  m_bBasic(false)
 {
   setupUi(this);
 }
@@ -63,14 +64,12 @@ SettingsDlg::~SettingsDlg(void)
 }
 
 void SettingsDlg::setupUi(QDialog *SettingsDlg) {
-
-
   Ui_SettingsDlg::setupUi(SettingsDlg);
 
   frame_ignoreMax->setVisible(false);
 
-  UINT64 iMaxCPUMemSize   = m_MasterController.SysInfo()->GetCPUMemSize();
-  UINT64 iMaxGPUMemSize   = m_MasterController.SysInfo()->GetGPUMemSize();
+  uint64_t iMaxCPUMemSize   = m_MasterController.SysInfo()->GetCPUMemSize();
+  uint64_t iMaxGPUMemSize   = m_MasterController.SysInfo()->GetGPUMemSize();
   unsigned int iProcCount = m_MasterController.SysInfo()->GetNumberOfCPUs();
   unsigned int iBitWidth  = m_MasterController.SysInfo()->GetProgramBitWidth();
 
@@ -110,16 +109,7 @@ void SettingsDlg::setupUi(QDialog *SettingsDlg) {
   horizontalSlider_GPUMem->setMinimum(32);
   horizontalSlider_CPUMem->setMinimum(512);
 
-
-#if defined(_WIN32) && defined(USE_DIRECTX)
-  if(m_MasterController.ExperimentalFeatures()) {
-    groupBox_7->setVisible(true);
-  } else {
-    groupBox_7->setVisible(false);
-  }
-#else
-  groupBox_7->setVisible(false);
-#endif
+  ToggleExperimentalFeatures();
 }
 
 void SettingsDlg::MaxToSliders(unsigned int iMaxCPUMB, unsigned int iMaxGPUMB) {
@@ -143,7 +133,6 @@ void SettingsDlg::MaxToSliders(unsigned int iMaxCPUMB, unsigned int iMaxGPUMB) {
   }
 }
 
-
 bool SettingsDlg::OverrideMaxMem() const {
   return checkBox_OverrideMax->isChecked();
 }
@@ -156,13 +145,12 @@ unsigned int SettingsDlg::GetMaxCPUMem() const {
   return checkBox_OverrideMax->isChecked() ? spinBox_CPUMax->value() : 0;
 }
 
-
-UINT64 SettingsDlg::GetGPUMem() const {
-  return UINT64(horizontalSlider_GPUMem->value())*1024*1024;
+uint64_t SettingsDlg::GetGPUMem() const {
+  return uint64_t(horizontalSlider_GPUMem->value())*1024*1024;
 }
 
-UINT64 SettingsDlg::GetCPUMem() const {
-  return UINT64(horizontalSlider_CPUMem->value())*1024*1024;
+uint64_t SettingsDlg::GetCPUMem() const {
+  return uint64_t(horizontalSlider_CPUMem->value())*1024*1024;
 }
 
 std::string SettingsDlg::GetTempDir() const {
@@ -361,21 +349,38 @@ void SettingsDlg::SetLogoLabel() {
   }
 }
 
-void SettingsDlg::Data2Form(bool bIsDirectX10Capable, UINT64 iMaxCPU, UINT64 iMaxGPU, bool bIgnoreMax,
-                            unsigned int iUserMaxCPUMB, unsigned int iUserMaxGPUMB, const std::string& tempDir,
-                            bool bQuickopen, unsigned int iMinFramerate, bool bUseAllMeans, unsigned int iLODDelay,
+void SettingsDlg::Data2Form(bool bIsDirectX10Capable, uint64_t iMaxCPU, 
+                            uint64_t iMaxGPU, bool bIgnoreMax,
+                            unsigned int iUserMaxCPUMB, 
+                            unsigned int iUserMaxGPUMB, 
+                            const std::string& tempDir,
+                            bool bQuickopen, unsigned int iMinFramerate, 
+                            bool bRenderLowResIntermediateResults, 
+                            unsigned int iLODDelay,
                             unsigned int iActiveTS, unsigned int iInactiveTS,
-                            bool bWriteLogFile, bool bShowCrashDialog, const std::string& strLogFileName, UINT32 iLogLevel,
+                            bool bWriteLogFile, bool bShowCrashDialog,
+                            const std::string& strLogFileName, 
+                            uint32_t iLogLevel,
                             bool bShowVersionInTitle,
-                            bool bAutoSaveGEO, bool bAutoSaveWSP, bool bAutoLockClonedWindow, bool bAbsoluteViewLocks,
-                            bool bCheckForUpdatesOnStartUp, bool bCheckForDevBuilds, bool bShowWelcomeScreen,
+                            bool bAutoSaveGEO, bool bAutoSaveWSP, 
+                            bool bAutoLockClonedWindow, bool bAbsoluteViewLocks,
+                            bool bCheckForUpdatesOnStartUp, 
+                            bool bCheckForDevBuilds, bool bShowWelcomeScreen,
                             bool bInvWheel, bool bI3MFeatures,
-                            unsigned int iVolRenType, unsigned int iBlendPrecision, bool bPowerOfTwo, bool bDownSampleTo8Bits,
-                            bool bDisableBorder, bool bNoRCClipplanes,
-                            const FLOATVECTOR3& vBackColor1, const FLOATVECTOR3& vBackColor2, const FLOATVECTOR4& vTextColor,
+                            unsigned int iVolRenType, 
+                            unsigned int iBlendPrecision, bool bPowerOfTwo, 
+                            bool bDownSampleTo8Bits,
+                            bool bDisableBorder,
+                            const FLOATVECTOR3& vBackColor1,
+                            const FLOATVECTOR3& vBackColor2,
+                            const FLOATVECTOR4& vTextColor,
                             const QString& strLogo, int iLogoPos,
                             unsigned int iMaxBrickSize,
+                            unsigned int iBuilderBrickSize,
                             unsigned int iMaxMaxBrickSize,
+                            bool bMedianFilter,
+                            bool bClampToEdge,
+                            uint32_t iCompression,
                             bool expFeatures) {
   m_bInit = true;
 
@@ -392,12 +397,31 @@ void SettingsDlg::Data2Form(bool bIsDirectX10Capable, UINT64 iMaxCPU, UINT64 iMa
   horizontalSlider_GPUMem->setValue(iMaxGPU / (1024*1024));
 
   lineEdit_TempDir->setText(tempDir.c_str());
-  horizontalSlider_BS->setValue(iMaxBrickSize);
-  horizontalSlider_BS->setMaximum(iMaxMaxBrickSize);
+  horizontalSlider_BSMax->setMaximum(iMaxMaxBrickSize);
+  horizontalSlider_BSMax->setValue(iMaxBrickSize);
+  MaxBSChanged(iMaxBrickSize);
+
+  horizontalSlider_BSBuilder->setMaximum(iMaxMaxBrickSize);
+  horizontalSlider_BSBuilder->setValue(iBuilderBrickSize);
+  BuilderBSChanged(iBuilderBrickSize);
+
+  radioButton_Mean->setChecked(!bMedianFilter);
+  radioButton_Median->setChecked(bMedianFilter);
+  radioButton_Border0->setChecked(!bClampToEdge);
+  radioButton_BorderClamp->setChecked(bClampToEdge);
+
+  switch (iCompression) {
+    default : radioButton_noCompression->setChecked(true);
+              radioButton_zlibCompression->setChecked(false);
+              break;
+    case 1 : radioButton_noCompression->setChecked(false);
+             radioButton_zlibCompression->setChecked(true);
+             break;
+  }
 
   checkBoxQuickload->setChecked(bQuickopen);
   horizontalSlider_MinFramerate->setValue(iMinFramerate);
-  checkBox_useAllMeans->setChecked(bUseAllMeans);
+  checkBox_useAllMeans->setChecked(bRenderLowResIntermediateResults);
   horizontalSlider_LODDelay->setValue(iLODDelay);
   horizontalSlider_ActTS->setValue(iActiveTS);
   horizontalSlider_InactTS->setValue(iInactiveTS);
@@ -419,9 +443,16 @@ void SettingsDlg::Data2Form(bool bIsDirectX10Capable, UINT64 iMaxCPU, UINT64 iMa
   checkBox_I3MFeatures->setChecked(bI3MFeatures);
   checkBox_ExperimentalFeatures->setChecked(expFeatures);
 
-  m_cBackColor1 = QColor(int(vBackColor1.x*255), int(vBackColor1.y*255),int(vBackColor1.z*255));
-  m_cBackColor2 = QColor(int(vBackColor2.x*255), int(vBackColor2.y*255),int(vBackColor2.z*255));
-  m_cTextColor  = QColor(int(vTextColor.x*255), int(vTextColor.y*255),int(vTextColor.z*255),int(vTextColor.w*255));
+  m_cBackColor1 = QColor(int(vBackColor1.x*255),
+                         int(vBackColor1.y*255),
+                         int(vBackColor1.z*255));
+  m_cBackColor2 = QColor(int(vBackColor2.x*255),
+                         int(vBackColor2.y*255),
+                         int(vBackColor2.z*255));
+  m_cTextColor  = QColor(int(vTextColor.x*255),
+                         int(vTextColor.y*255),
+                         int(vTextColor.z*255),
+                         int(vTextColor.w*255));
 
   switch (iBlendPrecision) {
     case 2    : radioButton_Prec32Bit->setChecked(true); break;
@@ -433,8 +464,8 @@ void SettingsDlg::Data2Form(bool bIsDirectX10Capable, UINT64 iMaxCPU, UINT64 iMa
 
   // adjust rendermode to GL if no DX support is present but set
   if (!bIsDirectX10Capable) {
-    if (iVolRenType == 2) iVolRenType = 0;
-    if (iVolRenType == 3) iVolRenType = 1;
+    if (iVolRenType == 2) iVolRenType = 0; else 
+    if (iVolRenType == 3) iVolRenType = 1; else 
     if (iVolRenType == 5) iVolRenType = 4;
   }
 
@@ -455,9 +486,15 @@ void SettingsDlg::Data2Form(bool bIsDirectX10Capable, UINT64 iMaxCPU, UINT64 iMa
     case 4    : radioButton_APIGL->setChecked(true);
                 radioButton_SBVR2D->setChecked(true);
                 break;
-    default   : radioButton_APIDX->setChecked(true);
+    case 5    : radioButton_APIDX->setChecked(true);
                 radioButton_SBVR2D->setChecked(true);
                 break;
+    case 6    : radioButton_APIGL->setChecked(true);
+                radioButton_TRaycast->setChecked(true);
+                break;
+    default   : radioButton_APIDX->setChecked(true);
+                radioButton_TRaycast->setChecked(true);
+                break;
   }
 
   switch (iLogoPos) {
@@ -477,7 +514,6 @@ void SettingsDlg::Data2Form(bool bIsDirectX10Capable, UINT64 iMaxCPU, UINT64 iMa
   checkBox_PowerOfTwo->setChecked(bPowerOfTwo);
   checkBox_DisableBorder->setChecked(bDisableBorder);
   checkBox_DownSampleTo8Bits->setChecked(bDownSampleTo8Bits);
-  checkBox_NoRCClipplanes->setChecked(bNoRCClipplanes);
 
   QString strStyle =
   tr("QPushButton { background: rgb(%1, %2, %3); color: rgb(%4, %5, %6) }").arg(m_cBackColor1.red())
@@ -523,21 +559,43 @@ const string SettingsDlg::GetLogFileName() const {
   return string(lineEdit_filename->text().toAscii());
 }
 
-UINT32 SettingsDlg::GetLogLevel() const {
+uint32_t SettingsDlg::GetLogLevel() const {
   return horizontalSlider_loglevel->value();
 }
 
 void SettingsDlg::WarnAPIMethodChange() {
   if (m_bWarnAPIChange && !m_bInit) {
-    QMessageBox::warning(this, "Warning", "A change to the render API, the rendermode, or the compatibiliy settings only affects new render windows.");
+    QMessageBox::warning(this, "Warning", "A change to the render API, the rendermode, or the compatibility settings only affects new render windows.");
     m_bWarnAPIChange = false;
   }
 }
 
+
+bool SettingsDlg::GetMedianFilter() const {
+  return radioButton_Median->isChecked();
+}
+
+bool SettingsDlg::GetClampToEdge() const {
+  return radioButton_BorderClamp->isChecked();
+}
+
+uint32_t SettingsDlg::GetCompression() const {
+  if (radioButton_noCompression->isChecked()) {
+    return 0;
+  } else {
+    return 1;
+  }
+}
+
+bool SettingsDlg::GetUseBasicSettings() const {
+  return m_bBasic;
+}
+
 unsigned int SettingsDlg::GetVolrenType() const {
   if (radioButton_APIGL->isChecked()) {
     if (radioButton_SBVR->isChecked()) return 0;
-    if (radioButton_SBVR2D->isChecked()) return 4; else return 1;
+    if (radioButton_SBVR2D->isChecked()) return 4;
+    if (radioButton_TRaycast->isChecked()) return 6; else return 1;
   } else {
     if (radioButton_SBVR->isChecked()) return 2;
     if (radioButton_SBVR2D->isChecked()) return 6; else return 3;
@@ -556,10 +614,6 @@ bool SettingsDlg::GetDisableBorder() const {
   return checkBox_DisableBorder->isChecked();
 }
 
-bool SettingsDlg::GetNoRCClipplanes() const {
-  return checkBox_NoRCClipplanes->isChecked();
-}
-
 QString SettingsDlg::GetLogoFilename() const {
   return m_strLogoFilename;
 }
@@ -571,15 +625,30 @@ int SettingsDlg::GetLogoPos() const {
   return 3;
 }
 
-
 void SettingsDlg::MaxBSChanged(int iValue) {
-  QString str = tr("%1^3").arg(MathTools::Pow2(UINT32(iValue)));
+  QString str = tr("%1^3").arg(MathTools::Pow2(uint32_t(iValue)));
+  if (iValue < horizontalSlider_BSBuilder->value()) {
+    horizontalSlider_BSBuilder->setValue(iValue);
+    BuilderBSChanged(iValue);
+  }
+  label_BSMaxUnit->setText(str);
+}
 
-  label_BS->setText(str);
+void SettingsDlg::BuilderBSChanged(int iValue) {
+  QString str = tr("%1^3").arg(MathTools::Pow2(uint32_t(iValue)));
+  if (horizontalSlider_BSMax->value() < iValue) {
+    horizontalSlider_BSMax->setValue(iValue);
+    MaxBSChanged(iValue);
+  }
+  label_BSBuilderUnit->setText(str);
 }
 
 unsigned int SettingsDlg::GetMaxBrickSize() const {
-  return horizontalSlider_BS->value();
+  return horizontalSlider_BSMax->value();
+}
+
+unsigned int SettingsDlg::GetBuilderBrickSize() const {
+  return horizontalSlider_BSBuilder->value();
 }
 
 void SettingsDlg::PickLogFile() {
@@ -667,8 +736,37 @@ void SettingsDlg::OverrideMaxToggled(bool bOverride) {
     horizontalSlider_CPUMem->setMaximum(spinBox_CPUMax->value());
     horizontalSlider_GPUMem->setMaximum(spinBox_GPUMax->value());
   } else {
-    UINT64 iMaxCPUMemSize   = m_MasterController.SysInfo()->GetCPUMemSize();
-    UINT64 iMaxGPUMemSize   = m_MasterController.SysInfo()->GetGPUMemSize();
+    uint64_t iMaxCPUMemSize   = m_MasterController.SysInfo()->GetCPUMemSize();
+    uint64_t iMaxGPUMemSize   = m_MasterController.SysInfo()->GetGPUMemSize();
     MaxToSliders(iMaxCPUMemSize/(1024*1024), iMaxGPUMemSize/(1024*1024));
   }
 }
+
+void SettingsDlg::ToggleExperimentalFeatures() {
+#if defined(_WIN32) && defined(USE_DIRECTX)
+  if(GetExperimentalFeatures()) {
+    groupBox_7->setVisible(true);
+  } else {
+    groupBox_7->setVisible(false);
+    radioButton_APIGL->setChecked(true);
+  }
+#else
+  groupBox_7->setVisible(false);
+  radioButton_APIGL->setChecked(true);
+#endif
+
+  if(GetExperimentalFeatures()) {
+    radioButton_TRaycast->setVisible(true);
+    groupBox_Filter->setVisible(true);
+  } else {
+    radioButton_TRaycast->setVisible(false);
+    groupBox_Filter->setVisible(false);
+    if (radioButton_TRaycast->isChecked()) 
+      radioButton_Raycast->setChecked(true);
+  }
+}
+
+void SettingsDlg::SwitchToBasic() {
+  m_bBasic = true;
+  this->reject();
+}
