@@ -44,6 +44,7 @@
 #include <cctype>
 #include <functional>
 #include <iterator>
+#include <limits.h>
 #include <sstream>
 #include <sys/stat.h>
 
@@ -52,8 +53,10 @@
   #include <dirent.h>
   #include <unistd.h>
   #include <pwd.h>
+  #define LARGE_STAT(name,buffer) stat(name,buffer)
 #else
   #include <shlwapi.h>
+  #define LARGE_STAT(name,buffer) _stat64(name,buffer)
 #endif
 
 #ifdef DETECTED_OS_APPLE
@@ -241,22 +244,22 @@ namespace SysTools {
     return Src.substr(p1, (p2-p1)+1);
   }
 
-  bool GetFileStats(const string& strFileName, struct ::stat& stat_buf) {
-    return (::stat( strFileName.c_str(), &stat_buf) >= 0);
+  bool GetFileStats(const string& strFileName, LARGE_STAT_BUFFER& stat_buf) {
+    return (LARGE_STAT( strFileName.c_str(), &stat_buf) >= 0);
   }
 
-  bool GetFileStats(const wstring& wstrFileName, struct ::stat& stat_buf) {
+  bool GetFileStats(const wstring& wstrFileName, LARGE_STAT_BUFFER& stat_buf) {
     string strFileName(wstrFileName.begin(), wstrFileName.end());
-    return (::stat( strFileName.c_str(), &stat_buf) >= 0);
+    return (LARGE_STAT( strFileName.c_str(), &stat_buf) >= 0);
   }
 
   bool FileExists(const wstring& wstrFileName) {
-    struct ::stat stat_buf;
+    LARGE_STAT_BUFFER stat_buf;
     return GetFileStats(wstrFileName, stat_buf);
   }
 
   bool FileExists(const string& strFileName) {
-    struct ::stat stat_buf;
+    LARGE_STAT_BUFFER stat_buf;
     return GetFileStats(strFileName, stat_buf);
   }
 
@@ -304,11 +307,13 @@ namespace SysTools {
 
   string GetPath(const string& fileName) {
     string path = fileName.substr(0,MAX(int(fileName.find_last_of("\\")),int(fileName.find_last_of("/")))+1);
+    if(path.empty()) { path = "./"; }
     return path;
   }
 
   wstring GetPath(const wstring& fileName) {
     wstring path = fileName.substr(0,MAX(int(fileName.find_last_of(L"\\")),int(fileName.find_last_of(L"/")))+1);
+    if(path.empty()) { path = L"./"; }
     return path;
   }
 
@@ -410,14 +415,24 @@ namespace SysTools {
     return L"";
   }
 
-  string  RemoveExt(const string& fileName) {
-    return fileName.substr(0,fileName.find_last_of("."));
+
+  std::string  RemoveExt(const std::string& fileName) {
+    size_t indexDot = fileName.find_last_of(".");
+    size_t indexSlash = MAX(int(fileName.find_last_of("\\")),int(fileName.find_last_of("/")));
+    if (indexDot == string::npos || (indexSlash != string::npos && indexDot < indexSlash)) return fileName;
+
+    return fileName.substr(0,indexDot);
   }
 
-  wstring RemoveExt(const wstring& fileName) {
-    return fileName.substr(0,fileName.find_last_of(L"."));
+  std::wstring  RemoveExt(const std::wstring& fileName) {
+    size_t indexDot = fileName.find_last_of(L".");
+    size_t indexSlash = MAX(int(fileName.find_last_of(L"\\")),int(fileName.find_last_of(L"/")));
+    if (indexDot == wstring::npos || (indexSlash != wstring::npos && indexDot < indexSlash)) return fileName;
+
+    return fileName.substr(0,indexDot);
   }
 
+
   string  ChangeExt(const string& fileName, const std::string& newext) {
     return RemoveExt(fileName)+ "." + newext;
   }
@@ -526,13 +541,9 @@ namespace SysTools {
   }
 #endif
 
-    vector<wstring> completeSubDirs;
+    vector<wstring> completeSubDirs(subDirs.size());
     for (size_t i = 0;i<subDirs.size();i++) {
       completeSubDirs.push_back(rootdir+subDirs[i]);
-      vector<wstring> subSubDirs = GetSubDirList(rootdir+subDirs[i]);
-      for (size_t j = 0;j<subSubDirs.size();j++) {
-        completeSubDirs.push_back(subSubDirs[j]);
-      }
     }
 
     return completeSubDirs;
@@ -543,7 +554,6 @@ namespace SysTools {
     vector<string> subDirs;
     string rootdir;
 
-
 #ifdef _WIN32
     stringstream s;
     if (dir == "") {
@@ -597,13 +607,9 @@ namespace SysTools {
   }
 #endif
 
-    vector<string> completeSubDirs;
+    vector<string> completeSubDirs(subDirs.size());
     for (size_t i = 0;i<subDirs.size();i++) {
       completeSubDirs.push_back(rootdir+subDirs[i]);
-      vector<string> subSubDirs = GetSubDirList(rootdir+subDirs[i]);
-      for (size_t j = 0;j<subSubDirs.size();j++) {
-        completeSubDirs.push_back(subSubDirs[j]);
-      }
     }
     return completeSubDirs;
   }
@@ -889,27 +895,27 @@ namespace SysTools {
   }
 
 
-  UINT32 FindNextSequenceIndex(const string& fileName, const string& ext, const string& dir) {
+  uint32_t FindNextSequenceIndex(const string& fileName, const string& ext, const string& dir) {
     vector<string> files = GetDirContents(dir, fileName+"*", ext);
 
-    UINT32 iMaxIndex = 0;
+    uint32_t iMaxIndex = 0;
     for (size_t i = 0; i<files.size();i++) {
       string curFilename = RemoveExt(files[i]);
-      UINT32 iCurrIndex = UINT32(atoi(curFilename.substr(fileName.length()).c_str()));
+      uint32_t iCurrIndex = uint32_t(atoi(curFilename.substr(fileName.length()).c_str()));
       iMaxIndex = (iMaxIndex <= iCurrIndex) ? iCurrIndex+1 : iMaxIndex;
     }
 
     return iMaxIndex;
   }
 
-  UINT32 FindNextSequenceIndex(const wstring& fileName, const wstring& ext, const wstring& dir) {
+  uint32_t FindNextSequenceIndex(const wstring& fileName, const wstring& ext, const wstring& dir) {
     vector<wstring> files = GetDirContents(dir, fileName+L"*", ext);
 
-    UINT32 iMaxIndex = 0;
+    uint32_t iMaxIndex = 0;
     for (size_t i = 0; i<files.size();i++) {
       wstring wcurFilename = RemoveExt(files[i]);
       string curFilename(wcurFilename.begin(), wcurFilename.end());
-      UINT32 iCurrIndex = UINT32(atoi(curFilename.substr(fileName.length()).c_str()));
+      uint32_t iCurrIndex = uint32_t(atoi(curFilename.substr(fileName.length()).c_str()));
       iMaxIndex = (iMaxIndex <= iCurrIndex) ? iCurrIndex+1 : iMaxIndex;
     }
 
