@@ -85,10 +85,10 @@
 #ifdef WIN32
   #pragma warning( disable : 4201 )  // Disable warning messages about nameless union
   #ifdef USEDX
-    #pragma message("    [vectors.h] Using DX extensions.\n")
+    #pragma message("    [vectors.h] Using DX extensions.")
   #endif
   #ifdef USEGL
-    #pragma message("    [vectors.h] Using GL extensions.\n")
+    #pragma message("    [vectors.h] Using GL extensions.")
   #endif
 #endif
 
@@ -288,11 +288,11 @@ public:
     }
   }
 
-  VECTOR3<T> abs() const {return VECTOR3<T>(fabs(x),fabs(y),fabs(z));}
-  T maxVal() const {return MAX(x,MAX(y,z));}
-  T minVal() const {return MIN(x,MIN(y,z));}
-  T volume() const {return x*y*z;}
-  T length() const {return sqrt(T(x*x+y*y+z*z));}
+  VECTOR3<T> abs() const FUNC_CONST {return VECTOR3<T>(fabs(x),fabs(y),fabs(z));}
+  T maxVal() const FUNC_CONST {return MAX(x,MAX(y,z));}
+  T minVal() const FUNC_CONST {return MIN(x,MIN(y,z));}
+  T volume() const FUNC_CONST {return x*y*z;}
+  T length() const FUNC_CONST {return sqrt(T(x*x+y*y+z*z));}
   void normalize() {T len = length(); x/=len;y/=len;z/=len;}
   void normalize(T epsilon, const VECTOR3<T> replacement=VECTOR3<T>(T(0),T(0),T(1))) {
     T len = length();
@@ -306,7 +306,7 @@ public:
       z = replacement.z;
     }
   }
-  VECTOR3<T> normalized() {
+  VECTOR3<T> normalized() const FUNC_CONST {
     T len = length(); 
     return VECTOR3<T>(x/len,y/len,z/len);
   }
@@ -321,7 +321,6 @@ public:
     }
   }
 
-
   VECTOR2<T> xx() const {return VECTOR2<T>(x,x);}
   VECTOR2<T> xy() const {return VECTOR2<T>(x,y);}
   VECTOR2<T> xz() const {return VECTOR2<T>(x,z);}
@@ -574,11 +573,13 @@ typedef VECTOR4<> INTVECTOR4;
 typedef VECTOR3<> INTVECTOR3;
 typedef VECTOR2<> INTVECTOR2;
 
-typedef VECTOR4<UINT32> UINTVECTOR4;
-typedef VECTOR3<UINT32> UINTVECTOR3;
-typedef VECTOR2<UINT32> UINTVECTOR2;
+typedef VECTOR4<uint32_t> UINTVECTOR4;
+typedef VECTOR3<uint32_t> UINTVECTOR3;
+typedef VECTOR2<uint32_t> UINTVECTOR2;
 
-typedef VECTOR3<UINT64> UINT64VECTOR3;
+typedef VECTOR2<uint64_t> UINT64VECTOR2;
+typedef VECTOR3<uint64_t> UINT64VECTOR3;
+typedef VECTOR4<uint64_t> UINT64VECTOR4;
 
 typedef VECTOR4<unsigned short> USHORTVECTOR4;
 typedef VECTOR3<unsigned short> USHORTVECTOR3;
@@ -903,6 +904,8 @@ public:
                                        << m.m41 << '\t' << m.m42 << '\t' << m.m43 << '\t' << m.m44 ; return os;}
 
   operator T*(void) {return &m11;}
+  operator const T*(void) const  {return &m11;}
+
   const T *operator *(void) const {return &m11;}
   T *operator *(void) {return &m11;}
 
@@ -1179,8 +1182,6 @@ public:
 
   // OpenGL
   #ifdef USEGL
-
-
     static void BuildStereoLookAtAndProjection(const VECTOR3<T> vEye, const VECTOR3<T> vAt, const VECTOR3<T> vUp,
                                               T fFOVY, T fAspect, T fZNear, T fZFar, T fFocalLength,
                                               T fEyeDist, int iEyeID, MATRIX4<T>& mView, MATRIX4<T>& mProj) {
@@ -1320,7 +1321,6 @@ public:
       glMatrixMode(GL_TEXTURE);
       glLoadMatrixf(M);
     }
-
   #endif
 };
 
@@ -1340,7 +1340,7 @@ public:
 
   QUATERNION4<T>(): x(0), y(0),z(0), w(0) {}
   QUATERNION4<T>(T _x, T _y, T _z, T _w): x(_x), y(_y), z(_z), w(_w) {}
-  QUATERNION4<T>(const T* other) : x(other.x), y(other.y), z(other.z), w(other.w) {}
+  QUATERNION4<T>(const T& other) : x(other.x), y(other.y), z(other.z), w(other.w) {}
 
   MATRIX4<T> ComputeRotation() {
     float n, s;
