@@ -160,7 +160,7 @@ void DICOMParser::GetDirInfo(wstring wstrDirectory) {
   GetDirInfo(strDirectory);
 }
 
-void DICOMParser::ReadHeaderElemStart(ifstream& fileDICOM, short& iGroupID, short& iElementID, DICOM_eType& eElementType, UINT32& iElemLength, bool bImplicit, bool bNeedsEndianConversion) {
+void DICOMParser::ReadHeaderElemStart(ifstream& fileDICOM, short& iGroupID, short& iElementID, DICOM_eType& eElementType, uint32_t& iElemLength, bool bImplicit, bool bNeedsEndianConversion) {
   string typeString = "  ";
 
   fileDICOM.read((char*)&iGroupID,2);
@@ -179,7 +179,7 @@ void DICOMParser::ReadHeaderElemStart(ifstream& fileDICOM, short& iGroupID, shor
   if (bImplicit) {
     eElementType = TYPE_Implicit;
     fileDICOM.read((char*)&iElemLength,4);
-    if (bNeedsEndianConversion) iElemLength = EndianConvert::Swap<UINT32>(iElemLength);
+    if (bNeedsEndianConversion) iElemLength = EndianConvert::Swap<uint32_t>(iElemLength);
     DICOM_DBG("Reader read implict field iGroupID=%i, iElementID=%i, iElemLength=%i\n", int(iGroupID), int(iElementID), iElemLength);
   } else {
     fileDICOM.read(&typeString[0],2);
@@ -188,7 +188,7 @@ void DICOMParser::ReadHeaderElemStart(ifstream& fileDICOM, short& iGroupID, shor
     if (bNeedsEndianConversion) tmp = EndianConvert::Swap<short>(tmp);
     iElemLength = tmp;
     eElementType = TYPE_UN;
-    UINT32 i=0;
+    uint32_t i=0;
     for (;i<27;i++) {
       if (typeString == DICOM_TypeStrings[i]) {
         eElementType = DICOM_eType(i);
@@ -204,26 +204,25 @@ void DICOMParser::ReadHeaderElemStart(ifstream& fileDICOM, short& iGroupID, shor
 
   if ((eElementType == TYPE_OF || eElementType == TYPE_OW || eElementType == TYPE_OB || eElementType == TYPE_UT) && iElemLength == 0) {
     fileDICOM.read((char*)&iElemLength,4);
-    if (bNeedsEndianConversion) iElemLength = EndianConvert::Swap<UINT32>(iElemLength);
+    if (bNeedsEndianConversion) iElemLength = EndianConvert::Swap<uint32_t>(iElemLength);
     DICOM_DBG("Reader found zero length %c%c field and read the length again which is now (iElemLength=%i)\n", typeString[0], typeString[1], iElemLength);
   }
 }
 
 
-UINT32 DICOMParser::GetUInt(ifstream& fileDICOM, const DICOM_eType eElementType, const UINT32 iElemLength, const bool bNeedsEndianConversion) {
+uint32_t DICOMParser::GetUInt(ifstream& fileDICOM, const DICOM_eType eElementType, const uint32_t iElemLength, const bool bNeedsEndianConversion) {
   string value;
-  UINT32 result;
+  uint32_t result;
   switch (eElementType) {
     case TYPE_Implicit :
     case TYPE_IS  : {
-              value.resize(iElemLength);
-              fileDICOM.read(&value[0],iElemLength);
+              ReadSizedElement(fileDICOM, value, iElemLength);
               result = atoi(value.c_str());
               break;
             }
     case TYPE_UL  : {
               fileDICOM.read((char*)&result,4);
-              if (bNeedsEndianConversion) result = EndianConvert::Swap<UINT32>(result);
+              if (bNeedsEndianConversion) result = EndianConvert::Swap<uint32_t>(result);
               break;
             }
     case TYPE_US  : {
@@ -240,14 +239,14 @@ UINT32 DICOMParser::GetUInt(ifstream& fileDICOM, const DICOM_eType eElementType,
 
 
 #ifdef DEBUG_DICOM
-void DICOMParser::ParseUndefLengthSequence(ifstream& fileDICOM, short& iSeqGroupID, short& iSeqElementID, DICOMFileInfo& info, const bool bImplicit, const bool bNeedsEndianConversion, UINT32 iDepth) {
+void DICOMParser::ParseUndefLengthSequence(ifstream& fileDICOM, short& iSeqGroupID, short& iSeqElementID, DICOMFileInfo& info, const bool bImplicit, const bool bNeedsEndianConversion, uint32_t iDepth) {
   for (int i = 0;i<int(iDepth)-1;i++) Console::printf("  ");
   Console::printf("iGroupID=%x iElementID=%x elementType=SEQUENCE (undef length)\n", iSeqGroupID, iSeqElementID);
 #else
 void DICOMParser::ParseUndefLengthSequence(ifstream& fileDICOM, short& , short& , DICOMFileInfo& info, const bool bImplicit, const bool bNeedsEndianConversion) {
 #endif
   int iItemCount = 0;
-  UINT32 iData;
+  uint32_t iData;
 
   string value;
   short iGroupID, iElementID;
@@ -260,14 +259,14 @@ void DICOMParser::ParseUndefLengthSequence(ifstream& fileDICOM, short& , short&
       iItemCount++;
       fileDICOM.read((char*)&iData,4);
       #ifdef DEBUG_DICOM
-        for (UINT32 i = 0;i<iDepth;i++) Console::printf("  ");
+        for (uint32_t i = 0;i<iDepth;i++) Console::printf("  ");
         Console::printf("START ITEM\n");
       #endif
     } else if (iData == 0xE00DFFFE) {
       iItemCount--;
       fileDICOM.read((char*)&iData,4);
       #ifdef DEBUG_DICOM
-        for (UINT32 i = 0;i<iDepth;i++) Console::printf("  ");
+        for (uint32_t i = 0;i<iDepth;i++) Console::printf("  ");
         Console::printf("END ITEM\n");
       #endif
     } else if (iData != 0xE0DDFFFE) fileDICOM.seekg(-4, ios_base::cur);
@@ -303,7 +302,7 @@ void DICOMParser::ParseUndefLengthSequence(ifstream& fileDICOM, short& , short&
             value.resize(iData);
             fileDICOM.read(&value[0],iData);
             #ifdef DEBUG_DICOM
-              for (UINT32 i = 0;i<iDepth;i++) Console::printf("  ");
+              for (uint32_t i = 0;i<iDepth;i++) Console::printf("  ");
               Console::printf("iGroupID=%x iElementID=%x elementType=%s value=%s\n", iGroupID, iElementID, DICOM_TypeStrings[int(elementType)].c_str(), value.c_str());
             #endif
           } else {
@@ -318,22 +317,28 @@ void DICOMParser::ParseUndefLengthSequence(ifstream& fileDICOM, short& , short&
   fileDICOM.read((char*)&iData,4);
 
 #ifdef DEBUG_DICOM
-  for (UINT32 i = 0;i<iDepth;i++) Console::printf("  ");
+  for (uint32_t i = 0;i<iDepth;i++) Console::printf("  ");
   Console::printf("END SEQUENCE\n");
 #endif
 
 }
 
-void DICOMParser::SkipUnusedElement(ifstream& fileDICOM, string& value, const UINT32 iElemLength) {
+void DICOMParser::ReadSizedElement(ifstream& fileDICOM, string& value, const uint32_t iElemLength) {
   value.resize(iElemLength);
-  fileDICOM.read(&value[0],iElemLength);
+  if (iElemLength) {
+    fileDICOM.read(&value[0],iElemLength);
+  }
+}
+
+void DICOMParser::SkipUnusedElement(ifstream& fileDICOM, string& value, const uint32_t iElemLength) {
+  ReadSizedElement(fileDICOM, value, iElemLength);
 }
 
 bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                                    DICOMFileInfo& info) {
   DICOM_DBG("Processing file %s\n",strFilename.c_str());
 
-  struct stat stat_buf;
+  LARGE_STAT_BUFFER stat_buf;
 
   bool bImplicit    = false;
   info.m_bIsJPEGEncoded = false;
@@ -341,7 +346,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
 
   info.m_strFileName = strFilename;
   info.m_wstrFileName = wstring(strFilename.begin(), strFilename.end());
-  info.m_ivSize.z = 1; // default if slices does not apear in the dicom
+  info.m_ivSize.z = 1; // default if slices does not appear in the dicom
 
   // check for basic properties
   if (!SysTools::GetFileStats(strFilename, stat_buf)) {// file must exist
@@ -359,16 +364,18 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
   fileDICOM.seekg(128);  // skip first 128 bytes
 
   string value;
+#ifdef DEBUG_DICOM
   float fSliceSpacing = 0;
+#endif
   short iGroupID, iElementID;
-  UINT32 iElemLength;
+  uint32_t iElemLength;
   DICOM_eType elementType;  
 
   // check for DICM magic.
   char DICM[4];
   fileDICOM.read(DICM,4);
   if (DICM[0] != 'D' || DICM[1] != 'I' || DICM[2] != 'C' || DICM[3] != 'M') {
-    MESSAGE("File '%s' does not conatin DICM meta header.", strFilename.c_str());
+    MESSAGE("File '%s' does not contain DICM meta header.", strFilename.c_str());
 
     // DICOM supports files without the meta header, 
     // in that case you have to guess the parameters
@@ -389,7 +396,9 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
   } else {
     // Ok, at this point we are very sure that we are dealing with a DICOM File,
     // lets find out the dimensions, the sequence numbers
+#ifdef DEBUG_DICOM
     fSliceSpacing = 0;
+#endif
     int iMetaHeaderEnd=0;
     bool bParsingMetaHeader = true;
 
@@ -407,17 +416,15 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
               }
               int iMetaHeaderLength;
               fileDICOM.read((char*)&iMetaHeaderLength,4);
-              iMetaHeaderEnd  = iMetaHeaderLength + UINT32(fileDICOM.tellg());
+              iMetaHeaderEnd  = iMetaHeaderLength + uint32_t(fileDICOM.tellg());
              } break;
         case 0x1 : {  // Version
               assert(iElemLength > 0);
               if(iElemLength == 0) { iElemLength = 1; } // guarantee progress.
-              value.resize(iElemLength);
-              fileDICOM.read(&value[0],iElemLength);
+              ReadSizedElement(fileDICOM, value, iElemLength);
              } break;
         case 0x10 : {  // Parse Type to find out endianess
-              value.resize(iElemLength);
-              fileDICOM.read(&value[0],iElemLength);
+              ReadSizedElement(fileDICOM, value, iElemLength);
               if (value[iElemLength-1] == 0) value.resize(iElemLength-1);
 
               if (value == "1.2.840.10008.1.2") {   // Implicit VR Little Endian
@@ -460,8 +467,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
               bParsingMetaHeader = false;
              } break;
         default : {
-          value.resize(iElemLength);
-          fileDICOM.read(&value[0],iElemLength);
+          SkipUnusedElement(fileDICOM, value, iElemLength);
         } break;
       }
       ReadHeaderElemStart(fileDICOM, iGroupID, iElementID, elementType,
@@ -481,8 +487,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
         value = "SEQUENCE";
       } else {
         // HACK: here we simply skip over the entire sequence
-        value.resize(iElemLength);
-        fileDICOM.read(&value[0],iElemLength);
+        SkipUnusedElement(fileDICOM, value, iElemLength);
         value = "SKIPPED EXPLICIT SEQUENCE";
       }
     } else if (elementType == TYPE_Implicit && iElemLength == 0xFFFFFFFF) { // read implicit sequence
@@ -544,9 +549,8 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
               } break;
              } break;
         case 0x18 : switch (iElementID) {
-              case 0x50 : { // Slice Thinkness
-                    value.resize(iElemLength);
-                    fileDICOM.read(&value[0],iElemLength);
+              case 0x50 : { // Slice Thickness
+                    ReadSizedElement(fileDICOM, value, iElemLength);
                     info.m_fvfAspect.z = float(atof(value.c_str()));
                     #ifdef DEBUG_DICOM
                     {
@@ -557,10 +561,9 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                     #endif
                     } break;
               case 0x88 : { // Spacing
-                    value.resize(iElemLength);
-                    fileDICOM.read(&value[0],iElemLength);
-                    fSliceSpacing = float(atof(value.c_str()));
+                    ReadSizedElement(fileDICOM, value, iElemLength);
                     #ifdef DEBUG_DICOM
+                    fSliceSpacing = float(atof(value.c_str()));
                     {
                       stringstream ss;
                       ss << fSliceSpacing << " (Slice Spacing: recognized)";
@@ -569,8 +572,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                     #endif
                     } break;
                default : {
-                value.resize(iElemLength);
-                fileDICOM.read(&value[0],iElemLength);
+                SkipUnusedElement(fileDICOM, value, iElemLength);
               } break;
             }  break;
         case 0x20 : switch (iElementID) {
@@ -596,9 +598,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                     } break;
               case 0x32 : // patient position
                 {
-                    value.resize(iElemLength);
-                    fileDICOM.read(&value[0],iElemLength);
-
+                    ReadSizedElement(fileDICOM, value, iElemLength);
                     size_t iDelimiter = value.find_first_of("\\");
 
                     info.m_fvPatientPosition.x = float(atof(value.substr(0,iDelimiter).c_str()));
@@ -621,8 +621,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                     #endif
                 }  break;
               default : {
-                value.resize(iElemLength);
-                fileDICOM.read(&value[0],iElemLength);
+                SkipUnusedElement(fileDICOM, value, iElemLength);
               } break;
              } break;
         case 0x28 : switch (iElementID) {
@@ -672,8 +671,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                       break;
               case 0x30 : // x,y spacing
                 {
-                    value.resize(iElemLength);
-                    fileDICOM.read(&value[0],iElemLength);
+                    ReadSizedElement(fileDICOM, value, iElemLength);
 
                     size_t iDelimiter = value.find_first_of("\\");
 
@@ -721,9 +719,30 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                     }
                     #endif
                     break;
+              case 0x1050: // Window Center
+                ReadSizedElement(fileDICOM, value, iElemLength);
+                info.m_fWindowCenter = float(atof(value.c_str()));
+                #ifdef DEBUG_DICOM
+                  {
+                    stringstream ss;
+                    ss << info.m_fWindowCenter << " (Window Center: recognized and stored)";
+                    value = ss.str();
+                  }
+                #endif
+                break;
+              case 0x1051: // Window Width
+                ReadSizedElement(fileDICOM, value, iElemLength);
+                info.m_fWindowWidth =-float(atof(value.c_str()));
+                #ifdef DEBUG_DICOM
+                  {
+                    stringstream ss;
+                    ss << info.m_fWindowWidth << " (Window Width: recognized and stored)";
+                    value = ss.str();
+                  }
+                #endif
+                break;
               case 0x1052 : // Rescale Intercept (Bias)
-                    value.resize(iElemLength);
-                    fileDICOM.read(&value[0],iElemLength);
+                    ReadSizedElement(fileDICOM, value, iElemLength);
                     info.m_fBias = float(atof(value.c_str()));
                     #ifdef DEBUG_DICOM
                     {
@@ -734,8 +753,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                     #endif
                     break;
               case 0x1053 : // Rescale Slope (Scale)
-                    value.resize(iElemLength);
-                    fileDICOM.read(&value[0],iElemLength);
+                    ReadSizedElement(fileDICOM, value, iElemLength);
                     info.m_fScale = float(atof(value.c_str()));
                     #ifdef DEBUG_DICOM
                     {
@@ -746,13 +764,11 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
                     #endif
                     break;
               default : {
-                value.resize(iElemLength);
-                fileDICOM.read(&value[0],iElemLength);
+                SkipUnusedElement(fileDICOM, value, iElemLength);
               } break;
              } break;
         default : {
-          value.resize(iElemLength);
-          fileDICOM.read(&value[0],iElemLength);
+          SkipUnusedElement(fileDICOM, value, iElemLength);
         } break;
 
       }
@@ -768,8 +784,8 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
     if (!bImplicit) {
       // for an explicit file we can actually check if we found the pixel
       // data block (and not some color table)
-      UINT32 iPixelDataSize = info.m_ivSize.volume() * info.m_iAllocated / 8;
-      UINT32 iDataSizeInFile = iElemLength;
+      uint32_t iPixelDataSize = info.m_ivSize.volume() * info.m_iAllocated / 8;
+      uint32_t iDataSizeInFile = iElemLength;
       if (iDataSizeInFile == 0) fileDICOM.read((char*)&iDataSizeInFile,4);
 
       if (info.m_bIsJPEGEncoded) {
@@ -787,16 +803,16 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
           offset = 4;  // make sure it won't underflow in the next line.
         }
         offset -= 4;
-        MESSAGE("JPEG is at offset: %u", static_cast<UINT32>(offset));
-        info.SetOffsetToData(static_cast<UINT32>(offset));
+        MESSAGE("JPEG is at offset: %u", static_cast<uint32_t>(offset));
+        info.SetOffsetToData(static_cast<uint32_t>(offset));
       } else {
         if (iPixelDataSize != iDataSizeInFile) {
           elementType = TYPE_UN;
-        } else info.SetOffsetToData(UINT32(fileDICOM.tellg()));
+        } else info.SetOffsetToData(uint32_t(fileDICOM.tellg()));
       }
     } else {
       // otherwise just believe we have found the right data block
-      info.SetOffsetToData(UINT32(fileDICOM.tellg()));
+      info.SetOffsetToData(uint32_t(fileDICOM.tellg()));
     }
   }
 
@@ -813,7 +829,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
 
     DICOM_DBG("volume size: %u\n", info.m_ivSize.volume());
     DICOM_DBG("n components: %u\n", info.m_iComponentCount);
-    UINT32 iPixelDataSize = info.m_iComponentCount *
+    uint32_t iPixelDataSize = info.m_iComponentCount *
                             info.m_ivSize.volume() *
                             info.m_iAllocated / 8;
     bool bOK = false;
@@ -840,8 +856,8 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
         if (bOK) {
           DICOM_DBG("Manual search for GroupID seemed to work.\n");
           if (!bImplicit) {
-            UINT32 iVolumeDataSize = info.m_ivSize.volume() * info.m_iAllocated / 8;
-            UINT32 iDataSizeInFile;
+            uint32_t iVolumeDataSize = info.m_ivSize.volume() * info.m_iAllocated / 8;
+            uint32_t iDataSizeInFile;
             fileDICOM.read((char*)&iDataSizeInFile,4);
 
             if (iVolumeDataSize != iDataSizeInFile) bOK = false;
@@ -861,7 +877,7 @@ bool DICOMParser::GetDICOMFileInfo(const string& strFilename,
       // and let's hope that the file ends with the data
       WARNING("Trouble parsing DICOM file; assuming data starts at %u",
               static_cast<unsigned int>(iFileLength - size_t(iPixelDataSize)));
-      info.SetOffsetToData(UINT32(iFileLength - size_t(iPixelDataSize)));
+      info.SetOffsetToData(uint32_t(iFileLength - size_t(iPixelDataSize)));
     }
   }
 
@@ -879,6 +895,8 @@ SimpleDICOMFileInfo::SimpleDICOMFileInfo(const std::string& strFileName) :
   m_iComponentCount(1),
   m_fScale(1.0f),
   m_fBias(0.0f),
+  m_fWindowWidth(0),
+  m_fWindowCenter(0),
   m_bSigned(false),
   m_iOffsetToData(0)
 {
@@ -921,8 +939,8 @@ uint32_t SimpleDICOMFileInfo::GetComponentCount() const {
   return m_iComponentCount;
 }
 
-bool SimpleDICOMFileInfo::GetData(std::vector<char>& vData, UINT32 iLength,
-                                  UINT32 iOffset)
+bool SimpleDICOMFileInfo::GetData(std::vector<char>& vData, uint32_t iLength,
+                                  uint32_t iOffset)
 {
   ifstream fs;
   fs.open(m_strFileName.c_str(),fstream::binary);
@@ -989,7 +1007,7 @@ DICOMFileInfo::DICOMFileInfo(const std::wstring& wstrFileName) :
   m_strDesc("")
 {}
 
-void DICOMFileInfo::SetOffsetToData(const UINT32 iOffset) {
+void DICOMFileInfo::SetOffsetToData(const uint32_t iOffset) {
   m_iOffsetToData = iOffset;
   m_iDataSize = m_iComponentCount*m_ivSize.volume()*m_iAllocated/8;
 }
