@@ -54,13 +54,12 @@ NRRDConverter::NRRDConverter()
 
 bool NRRDConverter::ConvertToRAW(const std::string& strSourceFilename,
                                  const std::string& strTempDir, bool,
-                                 UINT64& iHeaderSkip, UINT64& iComponentSize,
-                                 UINT64& iComponentCount,
+                                 uint64_t& iHeaderSkip, uint64_t& iComponentSize,
+                                 uint64_t& iComponentCount,
                                  bool& bConvertEndianess, bool& bSigned,
                                  bool& bIsFloat, UINT64VECTOR3& vVolumeSize,
                                  FLOATVECTOR3& vVolumeAspect,
                                  std::string& strTitle,
-                                 UVFTables::ElementSemanticTable& eType,
                                  std::string& strIntermediateFile,
                                  bool& bDeleteIntermediateFile)
 {
@@ -84,7 +83,6 @@ bool NRRDConverter::ConvertToRAW(const std::string& strSourceFilename,
   fileData.close();
 
   // init data
-  eType    = UVFTables::ES_UNDEFINED;
   strTitle = "NRRD data";
   iComponentSize=8;
   iComponentCount=1;
@@ -93,6 +91,7 @@ bool NRRDConverter::ConvertToRAW(const std::string& strSourceFilename,
   vVolumeSize = UINT64VECTOR3(1,1,1);
   vVolumeAspect = FLOATVECTOR3(1,1,1);
   string        strRAWFile;
+  bDeleteIntermediateFile = false;
 
   // read data
   KeyValueFileParser parser(strSourceFilename, true);
@@ -107,35 +106,64 @@ bool NRRDConverter::ConvertToRAW(const std::string& strSourceFilename,
     T_ERROR("Could not open find token \"type\" in file %s", strSourceFilename.c_str());
     return false;
   } else {
-    if (kvpType->strValueUpper == "SIGNED CHAR" || kvpType->strValueUpper == "INT8" || kvpType->strValueUpper == "INT8_T") {
+    if (kvpType->strValueUpper == "SIGNED CHAR" ||
+        kvpType->strValueUpper == "INT8" || kvpType->strValueUpper == "INT8_T") {
       bSigned = true;
       bIsFloat = false;
       iComponentSize = 8;
-    } else if (kvpType->strValueUpper == "UCHAR" || kvpType->strValueUpper == "UNSIGNED CHAR" ||  kvpType->strValueUpper == "UINT8" || kvpType->strValueUpper == "UINT8_T") {
+    } else if (kvpType->strValueUpper == "UCHAR" ||
+               kvpType->strValueUpper == "UNSIGNED CHAR" ||
+               kvpType->strValueUpper == "UINT8" ||
+               kvpType->strValueUpper == "UINT8_T") {
       bSigned = false;
       bIsFloat = false;
       iComponentSize = 8;
-    } else if (kvpType->strValueUpper == "SHORT" || kvpType->strValueUpper == "SHORT INT" ||  kvpType->strValueUpper == "SIGNED SHORT" || kvpType->strValueUpper == "SIGNED SHORT INT" || kvpType->strValueUpper == "INT16" || kvpType->strValueUpper == "INT16_T") {
+    } else if (kvpType->strValueUpper == "SHORT" ||
+               kvpType->strValueUpper == "SHORT INT" ||
+               kvpType->strValueUpper == "SIGNED SHORT" ||
+               kvpType->strValueUpper == "SIGNED SHORT INT" ||
+               kvpType->strValueUpper == "INT16" ||
+               kvpType->strValueUpper == "INT16_T") {
       bSigned = true;
       bIsFloat = false;
       iComponentSize = 16;
-    } else if (kvpType->strValueUpper == "USHORT" || kvpType->strValueUpper == "UNSIGNED SHORT" || kvpType->strValueUpper == "UNSIGNED SHORT INT" || kvpType->strValueUpper == "UINT16" || kvpType->strValueUpper == "UINT16_T") {
+    } else if (kvpType->strValueUpper == "USHORT" ||
+               kvpType->strValueUpper == "UNSIGNED SHORT" ||
+               kvpType->strValueUpper == "UNSIGNED SHORT INT" ||
+               kvpType->strValueUpper == "UINT16" ||
+               kvpType->strValueUpper == "UINT16_T") {
       bSigned = false;
       bIsFloat = false;
       iComponentSize = 16;
-    } else if (kvpType->strValueUpper == "INT" || kvpType->strValueUpper == "SIGNED INT" || kvpType->strValueUpper == "INT32" || kvpType->strValueUpper == "INT32_T") {
+    } else if (kvpType->strValueUpper == "INT" ||
+               kvpType->strValueUpper == "SIGNED INT" ||
+               kvpType->strValueUpper == "INT32" ||
+               kvpType->strValueUpper == "INT32_T") {
       bSigned = true;
       bIsFloat = false;
       iComponentSize = 32;
-    } else if (kvpType->strValueUpper == "UINT" || kvpType->strValueUpper == "UNSIGNED INT" || kvpType->strValueUpper == "UINT32" || kvpType->strValueUpper == "UINT32_T") {
+    } else if (kvpType->strValueUpper == "UINT" ||
+               kvpType->strValueUpper == "UNSIGNED INT" ||
+               kvpType->strValueUpper == "UINT32" ||
+               kvpType->strValueUpper == "UINT32_T") {
       bSigned = false;
       bIsFloat = false;
       iComponentSize = 32;
-    } else if (kvpType->strValueUpper == "LONGLONG" || kvpType->strValueUpper == "LONG LONG" || kvpType->strValueUpper == "LONG LONG INT" || kvpType->strValueUpper == "SIGNED LONG LONG" || kvpType->strValueUpper == "SIGNED LONG LONG INT" || kvpType->strValueUpper == "INT64" || kvpType->strValueUpper == "INT64_T") {
+    } else if (kvpType->strValueUpper == "LONGLONG" ||
+               kvpType->strValueUpper == "LONG LONG" ||
+               kvpType->strValueUpper == "LONG LONG INT" ||
+               kvpType->strValueUpper == "SIGNED LONG LONG" ||
+               kvpType->strValueUpper == "SIGNED LONG LONG INT" ||
+               kvpType->strValueUpper == "INT64" ||
+               kvpType->strValueUpper == "INT64_T") {
       bSigned = true;
       bIsFloat = false;
       iComponentSize = 64;
-    } else if (kvpType->strValueUpper == "ULONGLONG" || kvpType->strValueUpper == "UNSIGNED LONG LONG" || kvpType->strValueUpper == "UNSIGNED LONG LONG INT" || kvpType->strValueUpper == "UINT64" || kvpType->strValueUpper == "UINT64_T") {
+    } else if (kvpType->strValueUpper == "ULONGLONG" ||
+               kvpType->strValueUpper == "UNSIGNED LONG LONG" ||
+               kvpType->strValueUpper == "UNSIGNED LONG LONG INT" ||
+               kvpType->strValueUpper == "UINT64" ||
+               kvpType->strValueUpper == "UINT64_T") {
       bSigned = true;
       bIsFloat = false;
       iComponentSize = 64;
@@ -176,7 +204,8 @@ bool NRRDConverter::ConvertToRAW(const std::string& strSourceFilename,
 
   KeyValPair* kvpDim = parser.GetData("DIMENSION");
   if (kvpDim == NULL) {
-    T_ERROR("Could not open find token \"dimension\" in file %s", strSourceFilename.c_str());
+    T_ERROR("Could not open find token \"dimension\" in file %s",
+            strSourceFilename.c_str());
     return false;
   } else {
     if (kvpDim->iValue < 3)  {
@@ -192,12 +221,12 @@ bool NRRDConverter::ConvertToRAW(const std::string& strSourceFilename,
   KeyValPair* kvpDataFile1 = parser.GetData("DATA FILE");
   KeyValPair* kvpDataFile2 = parser.GetData("DATAFILE");
   if (kvpDataFile1 == NULL && kvpDataFile2 == NULL) {
-    iHeaderSkip = UINT64(parser.GetStopPos());
+    iHeaderSkip = uint64_t(parser.GetStopPos());
     strRAWFile = strSourceFilename;
     bDetachedHeader = false;
 
     if (iHeaderSkip == 0) { // parser read the entire file and did not find a 
-                            // data part (seperated by an empty line) 
+                            // data part (separated by an empty line)
       T_ERROR("NRRD file does neither specify detached data file nor does it "
               "contain data after the header, separated by an empty line.");
       return false;
@@ -349,10 +378,10 @@ bool NRRDConverter::ConvertToRAW(const std::string& strSourceFilename,
       if (iByteSkip == -1) {
         LargeRAWFile f(strRAWFile);
         f.Open(false);
-        UINT64 iSize = f.GetCurrentSize();
+        uint64_t iSize = f.GetCurrentSize();
         f.Close();
 
-        iHeaderSkip = iSize - (iComponentSize/8 * iComponentCount * UINT64(vVolumeSize.volume()));
+        iHeaderSkip = iSize - (iComponentSize/8 * iComponentCount * uint64_t(vVolumeSize.volume()));
       } else {
         if (iByteSkip != 0) {
           if (bDetachedHeader)
@@ -414,8 +443,8 @@ bool NRRDConverter::ConvertToRAW(const std::string& strSourceFilename,
   }
 }
 
-bool NRRDConverter::ConvertToNative(const std::string& strRawFilename, const std::string& strTargetFilename, UINT64 iHeaderSkip,
-                             UINT64 iComponentSize, UINT64 iComponentCount, bool bSigned, bool bFloatingPoint,
+bool NRRDConverter::ConvertToNative(const std::string& strRawFilename, const std::string& strTargetFilename, uint64_t iHeaderSkip,
+                             uint64_t iComponentSize, uint64_t iComponentCount, bool bSigned, bool bFloatingPoint,
                              UINT64VECTOR3 vVolumeSize,FLOATVECTOR3 vVolumeAspect, bool bNoUserInteraction,
                              const bool bQuantizeTo8Bit) {
 
@@ -505,7 +534,8 @@ bool NRRDConverter::ConvertToNative(const std::string& strRawFilename, const std
     if (bRAWSuccess) {
       return true;
     } else {
-      T_ERROR("Error appaneding raw data to header file %s.", strTargetFilename.c_str());
+      T_ERROR("Error appending raw data to header file %s.",
+              strTargetFilename.c_str());
       remove(strTargetFilename.c_str());
       return false;
     }
