@@ -4,17 +4,18 @@
 #define UVFBASIC_H
 
 #ifndef UVFVERSION
-  #define UVFVERSION 3
+  #define UVFVERSION 4
 #else
-  #if UVFVERSION != 3
+  #if UVFVERSION != 4
     #error [UVFBasic.h] Version mismatch
   #endif
 #endif
 
-#include "StdDefines.h"
+#include "Basics/StdDefines.h"
 #define UVF_INVALID UINT64_INVALID
 
 #include "Basics/LargeRAWFile.h"
+
 class DataBlock;
 
 #endif // UVFBASIC_H
