@@ -1,9 +1,9 @@
+#include <cstdint>
 #include <cstdlib>
 #include <time.h>
 #include <algorithm>
 #include <functional>
 #include <vector>
-#include <boost/cstdint.hpp>
 
 #include <cxxtest/TestSuite.h>
 
@@ -41,7 +41,7 @@ struct test_quant : public std::unary_function<testfile<T>, void> {
     const size_t filesz = filesize(fn.c_str());
     const size_t n_elems = filesz / sizeof(T);
 
-    std::vector<UINT64> hist;
+    std::vector<uint64_t> hist;
     {
 #ifdef VERBOSE
       TS_TRACE("raw_data_src");
@@ -50,7 +50,7 @@ struct test_quant : public std::unary_function<testfile<T>, void> {
       LargeRAWFile raw(fn);
       raw.Open(false);
       std::pair<T,T> mm = io_minmax<T>(raw_data_src<T>(raw), histw,
-                                       TuvokProgress<UINT64>(n_elems), sz);
+                                       TuvokProgress<uint64_t>(n_elems), sz);
       check_equality(tf.data_min, mm.first);
       check_equality(tf.data_max, mm.second);
       raw.Close();
@@ -62,7 +62,7 @@ struct test_quant : public std::unary_function<testfile<T>, void> {
       Unsigned12BitHistogram<T> histw(hist);
       std::ifstream fs(fn.c_str());
       std::pair<T,T> mm = io_minmax<T>(ios_data_src<T>(fs), histw,
-                                       TuvokProgress<UINT64>(n_elems), sz);
+                                       TuvokProgress<uint64_t>(n_elems), sz);
       check_equality(tf.data_min, mm.first);
       check_equality(tf.data_max, mm.second);
       fs.close();
@@ -76,8 +76,8 @@ namespace {
   template<typename T>
   void t(size_t sz, T mean, T stddev) {
     std::ofstream dataf;
-    std::string fn = mk_tmpfile(dataf, std::ios::out | std::ios::binary);
-    std::pair<T,T> minmax = gen_normal<T>(dataf, sz, mean, stddev);
+    const std::string fn = mk_tmpfile(dataf, std::ios::out | std::ios::binary);
+    const std::pair<T,T> minmax = gen_normal<T>(dataf, sz, mean, stddev);
     dataf.close();
     {
       struct testfile<T> tf = { fn.c_str(), 0, minmax.first, minmax.second };
@@ -90,7 +90,7 @@ namespace {
   template <typename T>
   void t_constant(size_t sz, T value) {
     std::ofstream dataf;
-    std::string fn = mk_tmpfile(dataf, std::ios::out | std::ios::binary);
+    const std::string fn = mk_tmpfile(dataf, std::ios::out | std::ios::binary);
     gen_constant<T>(dataf, sz, value);
     dataf.close();
     {
