@@ -41,11 +41,7 @@
 
 #include "StdTuvokDefines.h"
 #include <string>
-#ifdef DETECTED_OS_WINDOWS
-# include <memory>
-#else
-# include <tr1/memory>
-#endif
+#include <memory>
 
 #include "../StdTuvokDefines.h"
 #include "../Renderer/CullingLOD.h"
@@ -53,8 +49,11 @@
 #include "../IO/Dataset.h"
 #include "../Basics/Plane.h"
 #include "../Basics/GeometryGenerator.h"
-#include "../Scripting/Scriptable.h"
 #include "Context.h"
+#include "Basics/Interpolant.h"
+
+#include "LuaScripting/LuaScripting.h"
+#include "LuaScripting/LuaClassRegistration.h"
 
 class TransferFunction1D;
 class TransferFunction2D;
@@ -63,6 +62,9 @@ namespace tuvok {
 
 class MasterController;
 class RenderMesh;
+class LuaDatasetProxy;
+class LuaTransferFun1DProxy;
+class LuaTransferFun2DProxy;
 
 class Brick {
 public:
@@ -77,8 +79,8 @@ public:
   {
   }
 
-  Brick(UINT32 x, UINT32 y, UINT32 z,
-        UINT32 iSizeX, UINT32 iSizeY, UINT32 iSizeZ,
+  Brick(uint32_t x, uint32_t y, uint32_t z,
+        uint32_t iSizeX, uint32_t iSizeY, uint32_t iSizeZ,
         const BrickKey& k) :
     vCenter(0,0,0),
     vExtension(0,0,0),
@@ -102,13 +104,12 @@ public:
 };
 
 inline bool operator < (const Brick& left, const Brick& right) {
-  if  (left.fDistance < right.fDistance) return true;
-  return false;
+  return (left.fDistance < right.fDistance);
 }
 
 /** \class AbstrRenderer
  * Base for all renderers. */
-class AbstrRenderer: public Scriptable {
+class AbstrRenderer {
   public:
 
     enum ERendererTarget {
@@ -131,7 +132,12 @@ class AbstrRenderer: public Scriptable {
       SM_AF,            // alternating frame
       SM_INVALID
     };
-    virtual ERendererType GetRendererType() const {return RT_INVALID;}
+
+    enum EStereoID {
+      SI_LEFT_OR_MONO = 0,
+      SI_RIGHT = 1,
+      SI_INVALID
+    };
 
     enum ERenderMode {
       RM_1DTRANS = 0,  /**< one dimensional transfer function */
@@ -139,8 +145,6 @@ class AbstrRenderer: public Scriptable {
       RM_ISOSURFACE,   /**< render isosurfaces                */
       RM_INVALID
     };
-    ERenderMode GetRendermode() {return m_eRenderMode;}
-    virtual void SetRendermode(ERenderMode eRenderMode);
 
     enum EBlendPrecision {
       BP_8BIT = 0,
@@ -148,11 +152,6 @@ class AbstrRenderer: public Scriptable {
       BP_32BIT,
       BP_INVALID
     };
-    EBlendPrecision GetBlendPrecision() {return m_eBlendPrecision;}
-    virtual void SetBlendPrecision(EBlendPrecision eBlendPrecision);
-
-    bool GetUseLighting() const { return m_bUseLighting; }
-    virtual void SetUseLighting(bool bUseLighting);
 
     enum ScalingMethod {
       SMETH_BIT_WIDTH=0,    ///< scaled based on DS and TF bit width
@@ -175,59 +174,19 @@ class AbstrRenderer: public Scriptable {
                   enum ScalingMethod smeth = SMETH_BIT_WIDTH);
     /** Deallocates dataset and transfer functions. */
     virtual ~AbstrRenderer();
-    /** Sends a message to the master to ask for a dataset to be loaded.
-     * @param strFilename path to a file */
-    virtual bool LoadDataset(const std::string& strFilename);
-    /** Query whether or not we should redraw the next frame, else we should
-     * reuse what is already rendered or continue with the current frame if it
-     * is not complete yet. */
-    virtual bool CheckForRedraw();
 
-    UINTVECTOR2 GetSize() const {return m_vWinSize;}
-
-    virtual bool Paint() {
-      if (m_bDatasetIsInvalid) return true;
-
-      if (renderRegions.empty()) {
-        renderRegions.push_back(&simpleRenderRegion3D);
-      }
-      if (renderRegions.size() == 1 && renderRegions[0] == &simpleRenderRegion3D) {
-        simpleRenderRegion3D.maxCoord = m_vWinSize; //minCoord is always (0,0)
-      }
-
-      // check if we are rendering a stereo frame
-      m_bDoStereoRendering = m_bRequestStereoRendering &&
-                             renderRegions.size() == 1 &&
-                             renderRegions[0]->is3D();
-
-      return true; // nothing can go wrong here
-    }
-
-    void SetDatasetIsInvalid(bool bDatasetIsInvalid) {
-      m_bDatasetIsInvalid = bDatasetIsInvalid;
-    }
-
-
-    virtual bool Initialize(std::tr1::shared_ptr<Context> ctx);
-
-    /** Deallocates GPU memory allocated during the rendering process. */
-    virtual void Cleanup() = 0;
 
     /// Sets the dataset from external source; only meant to be used by clients
     /// which don't want to use the LOD subsystem.
     void SetDataset(Dataset *vds);
 /*    /// Modifies previously uploaded data.
     void UpdateData(const BrickKey&,
-                    std::tr1::shared_ptr<float> fp, size_t len);
+                    std::shared_ptr<float> fp, size_t len);
 */
-    Dataset&       GetDataset()       { return *m_pDataset; }
-    const Dataset& GetDataset() const { return *m_pDataset; }
     void ClearBricks() { m_pDataset->Clear(); }
 
-    TransferFunction1D* Get1DTrans() {return m_p1DTrans;}
-    TransferFunction2D* Get2DTrans() {return m_p2DTrans;}
-    virtual void Set1DTrans(const std::vector<unsigned char>& rgba) = 0;
 
+    virtual void Set1DTrans(const std::vector<unsigned char>& rgba) = 0;
     /** Notify renderer that 1D TF has changed.  In most cases, this will cause
      * the renderer to start anew. */
     virtual void Changed1DTrans();
@@ -235,224 +194,58 @@ class AbstrRenderer: public Scriptable {
      * the renderer to start anew. */
     virtual void Changed2DTrans();
 
-    /** Sets up a gradient background which fades vertically.
-     * @param vColors[0] is the color at the bottom;
-     * @param vColors[1] is the color at the top. */
-    virtual bool SetBackgroundColors(FLOATVECTOR3 vColors[2]) {
-      if (vColors[0] != m_vBackgroundColors[0] ||
-          vColors[1] != m_vBackgroundColors[1]) {
-        m_vBackgroundColors[0]=vColors[0];
-        m_vBackgroundColors[1]=vColors[1];
-        ScheduleCompleteRedraw();
-        return true;
-      } return false;
-    }
 
-    virtual bool SetTextColor(FLOATVECTOR4 vColor) {
-      if (vColor != m_vTextColor) {
-        m_vTextColor=vColor;ScheduleCompleteRedraw();
-        return true;
-      } return false;
-    }
-
-    FLOATVECTOR3 GetBackgroundColor(int i) const {
-      return m_vBackgroundColors[i];
-    }
     FLOATVECTOR4 GetTextColor() const {return m_vTextColor;}
 
-    virtual void SetSampleRateModifier(float fSampleRateModifier);
-    float GetSampleRateModifier() const { return m_fSampleRateModifier; }
-
-    virtual void SetIsoValue(float fIsovalue);
-    float GetIsoValue() const { return m_fIsovalue; }
-
-    virtual void SetIsosufaceColor(const FLOATVECTOR3& vColor);
-    virtual FLOATVECTOR3 GetIsosufaceColor() const { return m_vIsoColor; }
-
-    virtual void SetOrthoView(bool bOrthoView);
-    virtual bool GetOrthoView() const {return m_bOrthoView;}
 
     virtual void SetViewPort(UINTVECTOR2 lower_left, UINTVECTOR2 upper_right,
                              bool decrease_screen_res) = 0;
 
-    virtual void Transfer3DRotationToMIP();
-    virtual void SetRenderCoordArrows(bool bRenderCoordArrows);
-    virtual bool GetRenderCoordArrows() const {return m_bRenderCoordArrows;}
 
-    virtual void Set2DPlanesIn3DView(bool bRenderPlanesIn3D,
-                                     RenderRegion* renderRegion=NULL);
-    virtual bool Get2DPlanesIn3DView(RenderRegion* = NULL) const {
-      /// @todo: Make this bool a per 3d render region toggle.
-      return m_bRenderPlanesIn3D;}
 
-    /** Change the size of the render window.  Any previous image is
-     * destroyed, causing a full redraw on the next render.
-     * \param vWinSize  new width and height of the view window */
-    virtual void Resize(const UINTVECTOR2& vWinSize);
-
-    virtual void SetRotation(RenderRegion *renderRegion,
-                             const FLOATMATRIX4& mRotation);
+    virtual void SetRotation(RenderRegion * region,
+                             const FLOATMATRIX4& rotation);
     virtual const FLOATMATRIX4& GetRotation(const RenderRegion *renderRegion) const;
 
     virtual void SetTranslation(RenderRegion *renderRegion,
-                                const FLOATMATRIX4& mTranslation);
+                                const FLOATMATRIX4& translation);
     virtual const FLOATMATRIX4& GetTranslation(
                                         const RenderRegion *renderRegion) const;
 
-    void SetClipPlane(RenderRegion *renderRegion,
+    virtual void SetClipPlane(RenderRegion *renderRegion,
                       const ExtendedPlane& plane);
     virtual bool IsClipPlaneEnabled(RenderRegion *renderRegion=NULL);
     virtual void EnableClipPlane(RenderRegion *renderRegion=NULL);
     virtual void DisableClipPlane(RenderRegion *renderRegion=NULL);
-    virtual void ShowClipPlane(bool, RenderRegion *renderRegion=NULL);
-    const ExtendedPlane& GetClipPlane() const;
-
-    virtual void ClipPlaneRelativeLock(bool);
-    virtual bool CanDoClipPlane() {return true;}
-    bool ClipPlaneEnabled() const { return m_bClipPlaneOn; }
-    bool ClipPlaneShown() const   { return m_bClipPlaneDisplayed; }
-    bool ClipPlaneLocked() const  { return m_bClipPlaneLocked; }
 
     /// slice parameter for slice views.
-    virtual void SetSliceDepth(RenderRegion *renderRegion, UINT64 fSliceDepth);
-    virtual UINT64 GetSliceDepth(const RenderRegion *renderRegion) const;
+    virtual void SetSliceDepth(RenderRegion *renderRegion, uint64_t fSliceDepth);
+    virtual uint64_t GetSliceDepth(const RenderRegion *renderRegion) const;
 
     void SetClearFramebuffer(bool bClearFramebuffer) {
       m_bClearFramebuffer = bClearFramebuffer;
     }
     bool GetClearFramebuffer() const {return m_bClearFramebuffer;}
-    void SetGlobalBBox(bool bRenderBBox);
-    bool GetGlobalBBox() const {return m_bRenderGlobalBBox;}
-    void SetLocalBBox(bool bRenderBBox);
-    bool GetLocalBBox() const {return m_bRenderLocalBBox;}
 
-    virtual void SetLogoParams(std::string strLogoFilename, int iLogoPos);
     void Set2DFlipMode(RenderRegion *renderRegion, bool bFlipX, bool bFlipY);
     void Get2DFlipMode(const RenderRegion *renderRegion, bool& bFlipX,
                        bool& bFlipY) const;
     bool GetUseMIP(const RenderRegion *renderRegion) const;
     void SetUseMIP(RenderRegion *renderRegion, bool bUseMIP);
 
-    UINT64 GetMaxLODIndex() const { return m_iMaxLODIndex; }
-    UINT64 GetMinLODIndex() const { return m_iMinLODForCurrentView; }
-
-    UINTVECTOR2 GetLODLimits() const { return m_iLODLimits; }
-    void SetLODLimits(const UINTVECTOR2 iLODLimits);
-
-    // scheduling routines
-    UINT64 GetCurrentSubFrameCount() const
-        { return 1+m_iMaxLODIndex-m_iMinLODForCurrentView; }
-    UINT32 GetWorkingSubFrame() const
-        { return 1+static_cast<UINT32>(m_iMaxLODIndex-m_iCurrentLOD); }
-
-    UINT32 GetCurrentBrickCount() const
-        { return UINT32(m_vCurrentBrickList.size()); }
-    UINT32 GetWorkingBrick() const
-        { return static_cast<UINT32>(m_iBricksRenderedInThisSubFrame); }
-
-    UINT32 GetFrameProgress() const {
-        return UINT32(100.0f * float(GetWorkingSubFrame()) /
-                      float(GetCurrentSubFrameCount()));
-    }
-    UINT32 GetSubFrameProgress() const {
-        return (m_vCurrentBrickList.empty()) ? 100 :
-                UINT32(100.0f * m_iBricksRenderedInThisSubFrame /
-                float(m_vCurrentBrickList.size()));
-    }
-
-    void SetTimeSlice(UINT32 iMSecs) {m_iTimeSliceMSecs = iMSecs;}
-    void SetPerfMeasures(UINT32 iMinFramerate, bool bUseAllMeans,
-                         float fScreenResDecFactor,
-                         float fSampleDecFactor, UINT32 iStartDelay);
-    void SetRescaleFactors(const DOUBLEVECTOR3& vfRescale) {
-      m_pDataset->SetRescaleFactors(vfRescale);
-      ScheduleCompleteRedraw();
-    }
-    DOUBLEVECTOR3 GetRescaleFactors() const {
-      return m_pDataset->GetRescaleFactors();
-    }
-
-    void SetRendererTarget(ERendererTarget eRendererTarget) {
-      m_eRendererTarget = eRendererTarget;
-    }
-    ERendererTarget GetRendererTarget() const {
-      return m_eRendererTarget;
-    }
-    void SetMIPLOD(bool bMIPLOD) {m_bMIPLOD = bMIPLOD;}
-
-    virtual void  SetStereo(bool bStereoRendering);
-    virtual void  SetStereoEyeDist(float fStereoEyeDist);
-    virtual void  SetStereoFocalLength(float fStereoFocalLength);
-    virtual void  SetStereoMode(EStereoMode mode);
-    virtual void  SetStereoEyeSwap(bool bSwap);
-    virtual bool  GetStereo() const {return m_bRequestStereoRendering;}
-    virtual float GetStereoEyeDist() const {return m_fStereoEyeDist;}
-    virtual float GetStereoFocalLength() const {return m_fStereoFocalLength;}
-    virtual EStereoMode GetStereoMode() const {return m_eStereoMode;}
-    virtual bool  GetStereoEyeSwap() const {return m_bStereoEyeSwap;}
-
-    virtual void InitStereoFrame();
-    virtual void ToggleStereoFrame();
-
-    virtual void  SetConsiderPreviousDepthbuffer(bool bConsiderPreviousDepthbuffer);
-    virtual bool  GetConsiderPreviousDepthbuffer() const {
-      return m_bConsiderPreviousDepthbuffer;
-    }
-
-    void SetColors(const FLOATVECTOR4& ambient,
-                   const FLOATVECTOR4& diffuse,
-                   const FLOATVECTOR4& specular,
-                   const FLOATVECTOR3& lightDir);
-
-    FLOATVECTOR4 GetAmbient() const;
-    FLOATVECTOR4 GetDiffuse() const;
-    FLOATVECTOR4 GetSpecular()const;
-    FLOATVECTOR3 GetLightDir()const;
+    virtual void SetUserMatrices(const FLOATMATRIX4& view, const FLOATMATRIX4& projection,
+                                 const FLOATMATRIX4& viewLeft, const FLOATMATRIX4& projectionLeft,
+                                 const FLOATMATRIX4& viewRight, const FLOATMATRIX4& projectionRight);
+    virtual void UnsetUserMatrices();
 
     // ClearView
-    virtual bool SupportsClearView() {return false;}
-    virtual std::string ClearViewDisableReason() const {
-      return "this renderer does not support ClearView";
-    }
 
-    virtual void SetCV(bool bEnable);
-    virtual bool GetCV() const {return m_bDoClearView;}
-    virtual void SetCVIsoValue(float fIsovalue);
-    virtual float GetCVIsoValue() const { return m_fCVIsovalue; }
-    virtual void SetCVColor(const FLOATVECTOR3& vColor);
-    virtual FLOATVECTOR3 GetCVColor() const {return m_vCVColor;}
-    virtual void SetCVSize(float fSize);
-    virtual float GetCVSize() const {return m_fCVSize;}
-    virtual void SetCVContextScale(float fScale);
-    virtual float GetCVContextScale() const {return m_fCVContextScale;}
-    virtual void SetCVBorderScale(float fScale);
-    virtual float GetCVBorderScale() const {return m_fCVBorderScale;}
-    virtual void SetCVFocusPos(const FLOATVECTOR4& vCVPos);
-    virtual void SetCVFocusPos(const RenderRegion& renderRegion,
-                               const INTVECTOR2& vPos);
-    virtual FLOATVECTOR4 GetCVFocusPos() const {return m_vCVPos;}
     virtual INTVECTOR2 GetCVMousePos() const {return m_vCVMousePos;}
 
     virtual FLOATVECTOR3 Pick(const UINTVECTOR2&) const = 0;
 
-    virtual void ScheduleCompleteRedraw();
-    virtual void Schedule3DWindowRedraws(); // Redraw all 3D windows.
     virtual void ScheduleWindowRedraw(RenderRegion *renderRegion);
 
-    bool GetUseOnlyPowerOfTwo() const {return m_bUseOnlyPowerOfTwo;}
-    bool GetDownSampleTo8Bits() const {return m_bDownSampleTo8Bits;}
-    bool GetDisableBorder() const {return m_bDisableBorder;}
-
-    void SetMIPRotationAngle(float fAngle) {
-      m_fMIPRotationAngle = fAngle;
-      ScheduleCompleteRedraw();
-    }
-
-    /// Prepends the given directory to the list of paths Tuvok will
-    /// try to find shaders in.
-    void AddShaderPath(const std::string &path) {
-      m_vShaderSearchDirs.insert(m_vShaderSearchDirs.begin(), path);
-    }
 
     void SetViewParameters(float angle, float znear, float zfar,
                            const FLOATVECTOR3& eye,
@@ -473,23 +266,6 @@ class AbstrRenderer: public Scriptable {
 
     virtual void NewFrameClear(const RenderRegion &) = 0;
 
-    const std::vector<RenderRegion*>& GetRenderRegions() const {
-      return renderRegions;
-    }
-    void SetRenderRegions(const std::vector<RenderRegion*>&);
-    void GetVolumeAABB(FLOATVECTOR3& vCenter, FLOATVECTOR3& vExtend);
-
-    virtual void ScanForNewMeshes() {}
-    virtual void RemoveMeshData(size_t index);
-    std::vector<RenderMesh*>& GetMeshes() {return m_Meshes;}
-    bool SupportsMeshes() const {return m_bSupportsMeshes;}
-    void ReloadMesh(size_t index, const Mesh* m);
-
-    void Timestep(size_t);
-    size_t Timestep() const;
-
-    virtual void FixedFunctionality() const = 0;
-    virtual void SyncStateManager() = 0;
 
     const FLOATMATRIX4& GetProjectionMatrix(size_t eyeIndex = 0) const {return m_mProjection[eyeIndex];}
     const FLOATMATRIX4& GetViewMatrix(size_t eyeIndex = 0) const {return m_mView[eyeIndex];}
@@ -498,11 +274,389 @@ class AbstrRenderer: public Scriptable {
                  const std::vector<std::string>& strParams,
                  std::string& strMessage);
 
+    std::shared_ptr<Context> GetContext() const {return m_pContext;}
+
+    /// Registers base class Lua functions.
+    static void RegisterLuaFunctions(LuaClassRegistration<AbstrRenderer>& reg,
+                                     AbstrRenderer* me,
+                                     LuaScripting* ss);
+
+    /// This function is called at the end of RegisterLuaFunctions.
+    /// Use it to register additional renderer specific functions.
+    /// (functions that don't make sense as an inherited abstract method).
+    virtual void RegisterDerivedClassLuaFunctions(
+        LuaClassRegistration<AbstrRenderer>&,
+        LuaScripting*) 
+    {}
+
+    /// "PH" == "paper hacks".  sorry.  delete these after pacvis.
+    ///@{
+    virtual void PH_ClearWorkingSet();
+    virtual void PH_SetPagedBricks(size_t bricks);
+    virtual size_t PH_FramePagedBricks() const;
+    virtual size_t PH_SubframePagedBricks() const;
+    virtual void PH_RecalculateVisibility();
+    virtual bool PH_Converged() const;
+
+    virtual double PH_BrickIOTime() const;
+    virtual void PH_SetBrickIOTime(double d);
+    virtual uint64_t PH_BrickIOBytes() const;
+    virtual void PH_SetBrickIOBytes(uint64_t b);
+    virtual double PH_RenderingTime() const;
+
+    virtual bool PH_OpenLogfile(const std::string&);
+    virtual bool PH_CloseLogfile();
+    virtual void PH_SetOptimalFrameAverageCount(size_t);
+    virtual size_t PH_GetOptimalFrameAverageCount() const;
+    virtual bool PH_IsDebugViewAvailable() const;
+    virtual bool PH_IsWorkingSetTrackerAvailable() const;
+    ///@}
+
+  public:
+    // The following functions are public ONLY because there are dependencies
+    // upon them within Tuvok. Their dependencies are listed.
+
+    // Dependency: RenderRegion.cpp
+    virtual void ShowClipPlane(bool, RenderRegion *renderRegion=NULL);
+
+    // Dependency: RenderRegion.cpp
+    const ExtendedPlane& GetClipPlane() const;
+
+  protected:
+    // Functions in this section were made protected due to the transition to 
+    // Lua. All functions in this section are exposed through the Lua interface.
+
+    virtual ERendererType GetRendererType() const {return RT_INVALID;}
+    virtual void SetRendermode(ERenderMode eRenderMode);
+    virtual void SetBlendPrecision(EBlendPrecision eBlendPrecision);
+    
+    /** Sets up a gradient background which fades vertically.
+     * @param vColors[0] is the color at the bottom;
+     * @param vColors[1] is the color at the top. */
+    virtual bool SetBackgroundColors(FLOATVECTOR3 vTopColor,
+                                     FLOATVECTOR3 vBotColor) {
+      if (vTopColor != m_vBackgroundColors[0] ||
+          vBotColor != m_vBackgroundColors[1]) {
+        m_vBackgroundColors[0]=vTopColor;
+        m_vBackgroundColors[1]=vBotColor;
+        ScheduleCompleteRedraw();
+        return true;
+      } return false;
+    }
+
+    virtual bool SetTextColor(FLOATVECTOR4 vColor) {
+      if (vColor != m_vTextColor) {
+        m_vTextColor=vColor;ScheduleCompleteRedraw();
+        return true;
+      } return false;
+    }
+
+    virtual void SetLogoParams(std::string strLogoFilename, int iLogoPos);
+    virtual void ClipPlaneRelativeLock(bool);
+
+    // Clear view
+    virtual bool SupportsClearView() {return false;}
+    virtual std::string ClearViewDisableReason() const {
+      return "this renderer does not support ClearView";
+    }
+    virtual void SetCV(bool bEnable);
+    virtual bool GetCV() const {return m_bDoClearView;}
+    virtual void SetCVIsoValue(float fIsovalue);
+    virtual float GetCVIsoValue() const { return m_fCVIsovalue; }
+    virtual void SetCVColor(const FLOATVECTOR3& vColor);
+    virtual FLOATVECTOR3 GetCVColor() const {return m_vCVColor;}
+    virtual void SetCVSize(float fSize);
+    virtual float GetCVSize() const {return m_fCVSize;}
+    virtual void SetCVContextScale(float fScale);
+    virtual float GetCVContextScale() const {return m_fCVContextScale;}
+    virtual void SetCVBorderScale(float fScale);
+    virtual float GetCVBorderScale() const {return m_fCVBorderScale;}
+    virtual void SetCVFocusPosFVec(const FLOATVECTOR4& vCVPos);
+    virtual void SetCVFocusPos(LuaClassInstance renderRegion,
+                               const INTVECTOR2& vPos);
+    virtual FLOATVECTOR4 GetCVFocusPos() const {return m_vCVPos;}
+
+    RenderRegion3D* GetFirst3DRegion();
+
+    virtual void SetRenderCoordArrows(bool bRenderCoordArrows);
+    virtual bool GetRenderCoordArrows() const {return m_bRenderCoordArrows;}
+
+    virtual void Transfer3DRotationToMIP();
+
+    void SetMIPLOD(bool bMIPLOD) {m_bMIPLOD = bMIPLOD;}
+
+    virtual void  SetStereo(bool bStereoRendering);
+    virtual void  SetStereoEyeDist(float fStereoEyeDist);
+    virtual void  SetStereoFocalLength(float fStereoFocalLength);
+    virtual void  SetStereoMode(EStereoMode mode);
+    virtual void  SetStereoEyeSwap(bool bSwap);
+    virtual bool  GetStereo() const {return m_bRequestStereoRendering;}
+    virtual float GetStereoEyeDist() const {return m_fStereoEyeDist;}
+    virtual float GetStereoFocalLength() const {return m_fStereoFocalLength;}
+    virtual EStereoMode GetStereoMode() const {return m_eStereoMode;}
+    virtual bool  GetStereoEyeSwap() const {return m_bStereoEyeSwap;}
+    virtual void InitStereoFrame();
+    virtual void ToggleStereoFrame();
+
+    virtual void ScheduleCompleteRedraw();
+    /** Query whether or not we should redraw the next frame, else we should
+     * reuse what is already rendered or continue with the current frame if it
+     * is not complete yet. */
+    virtual bool CheckForRedraw();
+
+    /** Deallocates GPU memory allocated during the rendering process. */
+    virtual void Cleanup() = 0;
+
+    virtual void SetOrthoView(bool bOrthoView);
+    virtual bool GetOrthoView() const {return m_bOrthoView;}
+
+    virtual void SetUseLighting(bool bUseLighting);
+
+    virtual void SetSampleRateModifier(float fSampleRateModifier);
+
+    virtual void SetIsoValue(float fIsovalue);
+    /// the given isovalue is in the range [0,1] and is interpreted as a
+    /// percentage of the available isovalues.
+    virtual void SetIsoValueRelative(float fIsovalue);
+
+    virtual void SetIsosufaceColor(const FLOATVECTOR3& vColor);
+    virtual FLOATVECTOR3 GetIsosufaceColor() const { return m_vIsoColor; }
+
+    UINTVECTOR2 GetSize() const {return m_vWinSize;}
+
+    /** Change the size of the render window.  Any previous image is
+     * destroyed, causing a full redraw on the next render.
+     * \param vWinSize  new width and height of the view window */
+    virtual void Resize(const UINTVECTOR2& vWinSize);
+
+    virtual bool Paint() {
+      if (m_bDatasetIsInvalid) return true;
+
+      if (renderRegions.empty()) {
+        renderRegions.push_back(&simpleRenderRegion3D);
+      }
+      if (renderRegions.size() == 1 && renderRegions[0] == &simpleRenderRegion3D) {
+        simpleRenderRegion3D.maxCoord = m_vWinSize; //minCoord is always (0,0)
+      }
+
+      // check if we are rendering a stereo frame
+      m_bDoStereoRendering = m_bRequestStereoRendering &&
+                             renderRegions.size() == 1 &&
+                             renderRegions[0]->is3D();
+
+      return true; // nothing can go wrong here
+    }
+
+    virtual void  SetConsiderPreviousDepthbuffer(bool bConsiderPreviousDepthbuffer);
+    virtual bool  GetConsiderPreviousDepthbuffer() const {
+      return m_bConsiderPreviousDepthbuffer;
+    }
+
+    /** Sends a message to the master to ask for a dataset to be loaded.
+     * @param strFilename path to a file */
+    virtual bool LoadDataset(const std::string& strFilename);
+
+    virtual bool Initialize(std::shared_ptr<Context> ctx);
+
+    virtual void SetInterpolant(Interpolant eInterpolant);
+
+    virtual void ScanForNewMeshes() {}
+    virtual void RemoveMeshData(size_t index);
+
+    virtual void SetRescaleFactors(const DOUBLEVECTOR3& vfRescale) {
+      m_pDataset->SetRescaleFactors(vfRescale);
+      ScheduleCompleteRedraw();
+    }
+
+    virtual void Schedule3DWindowRedraws(); // Redraw all 3D windows.
+
+    virtual bool CropDataset(const std::string& strTempDir, bool bKeepOldData) = 0;
+
+    ExtendedPlane LuaGetClipPlane() const;
+
+    void GetVolumeAABB(FLOATVECTOR3& vCenter, FLOATVECTOR3& vExtend) const;
+
+    float GetIsoValue() const { return m_fIsovalue; }
+
+    virtual void CycleDebugViews();
+    virtual void SetDebugView(uint32_t iDebugView);
+    virtual uint32_t GetDebugViewCount() const;
+    uint32_t GetDebugView() const;
+
+  private:
+    // Functions in this section were made private due to the transition to Lua.
+    // All functions in this section are exposed through the Lua interface.
+    // If a derived class needs access to these functions, it is appropriate
+    // to move the function to protected, but not public.
+
+    ERendererTarget GetRendererTarget() const { return m_eRendererTarget; }
+
+    void SetRendererTarget(ERendererTarget eRendererTarget) {
+      m_eRendererTarget = eRendererTarget;
+    }
+
+    void SetViewPos(const FLOATVECTOR3& vPos);
+    FLOATVECTOR3 GetViewPos() const;
+    void ResetViewPos();
+    void SetViewDir(const FLOATVECTOR3& vDir);
+    FLOATVECTOR3 GetViewDir() const;
+    void ResetViewDir();
+    void SetUpDir(const FLOATVECTOR3& vDir);
+    FLOATVECTOR3 GetUpDir() const;
+    void ResetUpDir();
+
+    ERenderMode GetRendermode() {return m_eRenderMode;}
+
+    Dataset&       GetDataset()       { return *m_pDataset; }
+    const Dataset& GetDataset() const { return *m_pDataset; }
+
+    TransferFunction1D* Get1DTrans() {return m_p1DTrans;}
+    TransferFunction2D* Get2DTrans() {return m_p2DTrans;}
+
+    FLOATVECTOR3 GetBackgroundColor(int i) const {
+      return m_vBackgroundColors[i];
+    }
+
+    EBlendPrecision GetBlendPrecision() {return m_eBlendPrecision;}
+
+    void SetRenderRegions(const std::vector<RenderRegion*>&);
+    void LuaSetRenderRegions(std::vector<LuaClassInstance>);
+
+    const std::vector<RenderRegion*>& GetRenderRegions() const {
+      return renderRegions;
+    }
+    std::vector<LuaClassInstance> LuaGetRenderRegions();
+
+    LuaClassInstance LuaGetFirst3DRegion();
+
+    bool ClipPlaneLocked() const  { return m_bClipPlaneLocked; }
+    bool ClipPlaneEnabled() const { return m_bClipPlaneOn; }
+    bool ClipPlaneShown() const   { return m_bClipPlaneDisplayed; }
+
+    bool SupportsMeshes() const {return m_bSupportsMeshes;}
+
+    void SetMIPRotationAngle(float fAngle) {
+      m_fMIPRotationAngle = fAngle;
+      ScheduleCompleteRedraw();
+    }
+
+    virtual void Set2DPlanesIn3DView(bool bRenderPlanesIn3D,
+                                     RenderRegion* renderRegion=NULL);
+    virtual bool Get2DPlanesIn3DView(RenderRegion* = NULL) const {
+      /// @todo: Make this bool a per 3d render region toggle.
+      return m_bRenderPlanesIn3D;}
+
+    void LuaSet2DPlanesIn3DView(bool bRenderPlanesIn3D,LuaClassInstance region);
+    bool LuaGet2DPlanesIn3DView() const {return Get2DPlanesIn3DView();}
+
+    void SetTimeSlice(uint32_t iMSecs) {m_iTimeSliceMSecs = iMSecs;}
+
+    void SetPerfMeasures(uint32_t iMinFramerate, 
+                         bool bRenderLowResIntermediateResults,
+                         float fScreenResDecFactor,
+                         float fSampleDecFactor, uint32_t iStartDelay);
+
+    void LuaCloneRenderMode(LuaClassInstance inst);
+
+    bool GetUseLighting() const { return m_bUseLighting; }
+
+    void SetFoV(float fFoV);
+    float GetFoV() const {return m_fFOV;}
+
+    void SetTimestep(size_t);
+    size_t Timestep() const;
+
+    void SetGlobalBBox(bool bRenderBBox);
+    bool GetGlobalBBox() const {return m_bRenderGlobalBBox;}
+    void SetLocalBBox(bool bRenderBBox);
+    bool GetLocalBBox() const {return m_bRenderLocalBBox;}
+
+    void LuaShowClipPlane(bool bShown, LuaClassInstance inst);
+
+
+    // scheduling routines
+    uint64_t GetCurrentSubFrameCount() const
+        { return 1+m_iMaxLODIndex-m_iMinLODForCurrentView; }
+    uint32_t GetWorkingSubFrame() const
+        { return 1+static_cast<uint32_t>(m_iMaxLODIndex-m_iCurrentLOD); }
+
+    uint32_t GetCurrentBrickCount() const
+        { return uint32_t(m_vCurrentBrickList.size()); }
+    uint32_t GetWorkingBrick() const
+        { return static_cast<uint32_t>(m_iBricksRenderedInThisSubFrame); }
+
+    virtual uint32_t GetFrameProgress() const {
+        return uint32_t(100.0f * float(GetWorkingSubFrame()) /
+                      float(GetCurrentSubFrameCount()));
+    }
+    virtual uint32_t GetSubFrameProgress() const {
+        return (m_vCurrentBrickList.empty()) ? 100 :
+                uint32_t(100.0f * m_iBricksRenderedInThisSubFrame /
+                float(m_vCurrentBrickList.size()));
+    }
+
+    uint64_t GetMaxLODIndex() const { return m_iMaxLODIndex; }
+    uint64_t GetMinLODIndex() const { return m_iMinLODForCurrentView; }
+
+    bool GetUseOnlyPowerOfTwo() const {return m_bUseOnlyPowerOfTwo;}
+    bool GetDownSampleTo8Bits() const {return m_bDownSampleTo8Bits;}
+    bool GetDisableBorder() const {return m_bDisableBorder;}
+
+    /// Prepends the given directory to the list of paths Tuvok will
+    /// try to find shaders in.
+    void AddShaderPath(const std::string &path) {
+      m_vShaderSearchDirs.insert(m_vShaderSearchDirs.begin(), path);
+    }
+
+    UINTVECTOR2 GetLODLimits() const { return m_iLODLimits; }
+    void SetLODLimits(const UINTVECTOR2 iLODLimits);
+
+    void SetColors(FLOATVECTOR4 ambient,
+                   FLOATVECTOR4 diffuse,
+                   FLOATVECTOR4 specular,
+                   FLOATVECTOR3 lightDir);
+
+    Interpolant GetInterpolant() const {return m_eInterpolant;}
+
+    FLOATVECTOR4 GetAmbient() const;
+    FLOATVECTOR4 GetDiffuse() const;
+    FLOATVECTOR4 GetSpecular()const;
+    FLOATVECTOR3 GetLightDir()const;
+
+    void SetDatasetIsInvalid(bool bDatasetIsInvalid) {
+      m_bDatasetIsInvalid = bDatasetIsInvalid;
+    }
+
+    // Return value references are not supported by lua.
+    std::vector<std::shared_ptr<RenderMesh>> GetMeshes() {return m_Meshes;}
+    void ReloadMesh(size_t index, const std::shared_ptr<Mesh> m);
+
+    float GetSampleRateModifier() const { return m_fSampleRateModifier; }
+
+    DOUBLEVECTOR3 GetRescaleFactors() const {
+      return m_pDataset->GetRescaleFactors();
+    }
+
+    FLOATVECTOR3 LuaGetVolumeAABBExtents() const;
+    FLOATVECTOR3 LuaGetVolumeAABBCenter() const;
+
+    virtual void FixedFunctionality() const = 0;
+    virtual void SyncStateManager() = 0;
+
   protected:
     /// Unsets the current transfer function, including deleting it from GPU
     /// memory.  It's expected you'll set another one directly afterwards.
     void Free1DTrans();
 
+    /// This is the function that should be called after creating a new 1D
+    /// transfer function and associating it with the variable m_p1DTrans.
+    /// @todo Find a more elegant way of doing this.
+    void LuaBindNew1DTrans();
+
+    /// This is the function that should be called after creating a new 2D
+    /// transfer function and associating it with the variable m_p2DTrans.
+    void LuaBindNew2DTrans();
+
     /// @return the current iso value, normalized to be in [0,1]
     double GetNormalizedIsovalue() const;
     /// @return the current clearview iso value, normalized to be in [0,1]
@@ -510,14 +664,15 @@ class AbstrRenderer: public Scriptable {
 
     virtual void        ClearColorBuffer() const = 0;
     virtual void        UpdateLightParamsInShaders() = 0;
-    virtual void        CVFocusHasChanged(const RenderRegion &);
+    virtual void        CVFocusHasChanged(LuaClassInstance region);
+
+    /// @returns if the data we're rendering is color or not.
+    virtual bool ColorData() const;
 
-    /// @returns if the data we're rendering is RGBA or not.
-    virtual bool RGBAData() const;
 
   protected:
     MasterController*   m_pMasterController;
-    std::tr1::shared_ptr<Context> m_pContext;
+    std::shared_ptr<Context> m_pContext;
     ERenderMode         m_eRenderMode;
     bool                m_bFirstDrawAfterModeChange;
     bool                m_bFirstDrawAfterResize;
@@ -535,47 +690,50 @@ class AbstrRenderer: public Scriptable {
     UINTVECTOR2         m_vWinSize;
     int                 m_iLogoPos;
     std::string         m_strLogoFilename;
+    uint32_t            m_iDebugView;
+
+    Interpolant         m_eInterpolant;
 
     bool                m_bSupportsMeshes;
-    std::vector<RenderMesh*> m_Meshes;
+    std::vector<std::shared_ptr<RenderMesh>> m_Meshes;
 
 
     /// parameters for dynamic resolution adjustments
     ///@{
     float  msecPassed[2]; // time taken for last two frames
     float  msecPassedCurrentFrame; // time taken for our current rendering
-    UINT32 m_iLODNotOKCounter; // n frames waited for render to become faster
+    uint32_t m_iLODNotOKCounter; // n frames waited for render to become faster
     bool decreaseScreenRes; ///< dec.'d display resolution (lower n_fragments)
     bool decreaseScreenResNow;
     bool decreaseSamplingRate; ///< dec.'d sampling rate (less shader work)
     bool decreaseSamplingRateNow;
-    bool doAnotherRedrawDueToAllMeans; ///< previous subframe had res or rate
+    bool doAnotherRedrawDueToLowResOutput; ///< previous subframe had res or rate
                                        ///  reduced; we need another render to
                                        ///  complete finish up.
     float  m_fMaxMSPerFrame;
     float  m_fScreenResDecFactor;
     float  m_fSampleDecFactor;
-    bool   m_bUseAllMeans;
+    bool   m_bRenderLowResIntermediateResults;
     bool   m_bOffscreenIsLowRes;
-    UINT32 m_iStartDelay;
-    UINT64 m_iMinLODForCurrentView;
-    UINT32 m_iTimeSliceMSecs;
+    uint32_t m_iStartDelay;
+    uint64_t m_iMinLODForCurrentView;
+    uint32_t m_iTimeSliceMSecs;
     ///@}
 
-    UINT64              m_iIntraFrameCounter;
-    UINT64              m_iFrameCounter;
-    UINT32              m_iCheckCounter;
-    UINT64              m_iMaxLODIndex;
+    uint64_t            m_iIntraFrameCounter;
+    uint64_t            m_iFrameCounter;
+    uint32_t            m_iCheckCounter;
+    uint64_t            m_iMaxLODIndex;
     UINTVECTOR2         m_iLODLimits;
-    UINT64              m_iPerformanceBasedLODSkip;
-    UINT64              m_iCurrentLODOffset;
-    UINT64              m_iStartLODOffset;
+    uint64_t            m_iPerformanceBasedLODSkip;
+    uint64_t            m_iCurrentLODOffset;
+    uint64_t            m_iStartLODOffset;
     size_t              m_iTimestep;
     CullingLOD          m_FrustumCullingLOD;
     bool                m_bClearFramebuffer;
     bool                m_bConsiderPreviousDepthbuffer;
-    UINT64              m_iCurrentLOD;
-    UINT64              m_iBricksRenderedInThisSubFrame;
+    uint64_t            m_iCurrentLOD;
+    uint64_t            m_iBricksRenderedInThisSubFrame;
     std::vector<Brick>  m_vCurrentBrickList;
     std::vector<Brick>  m_vLeftEyeBrickList;
     ERendererTarget     m_eRendererTarget;
@@ -596,6 +754,13 @@ class AbstrRenderer: public Scriptable {
     bool                m_bPerformReCompose;
     bool                m_bRequestStereoRendering;
     bool                m_bDoStereoRendering;
+    bool                m_bUserMatrices;
+    FLOATMATRIX4        m_UserView;
+    FLOATMATRIX4        m_UserViewLeft;
+    FLOATMATRIX4        m_UserViewRight;
+    FLOATMATRIX4        m_UserProjection;
+    FLOATMATRIX4        m_UserProjectionLeft;
+    FLOATMATRIX4        m_UserProjectionRight;
   	int				          m_iAlternatingFrameID;
     float               m_fStereoEyeDist;
     float               m_fStereoFocalLength;
@@ -624,6 +789,7 @@ class AbstrRenderer: public Scriptable {
     FLOATVECTOR3        m_vEye, m_vAt, m_vUp;
     float               m_fFOV;
     float               m_fZNear, m_fZFar;
+    bool                m_bFirstPersonMode;
     ///@}
 
     std::vector<RenderRegion*> renderRegions;
@@ -646,7 +812,7 @@ class AbstrRenderer: public Scriptable {
     virtual void        ScheduleRecompose(RenderRegion *renderRegion=NULL);
     void                ComputeMinLODForCurrentView();
     void                ComputeMaxLODForCurrentView();
-    void                PlanFrame(RenderRegion3D& region);
+    virtual void        PlanFrame(RenderRegion3D& region);
     void                PlanHQMIPFrame(RenderRegion& renderRegion);
     /// @return true if the brick is needed to render the given region
     bool RegionNeedsBrick(const RenderRegion& rr, const BrickKey& key,
@@ -667,15 +833,33 @@ class AbstrRenderer: public Scriptable {
     double              MaxValue() const;
     bool                OnlyRecomposite(RenderRegion* region) const;
 
-    RenderRegion3D* GetFirst3DRegion();
 
     virtual bool IsVolumeResident(const BrickKey& key) const = 0;
 
-    virtual void RegisterCalls(Scripting*);
+    /// Since m_Meshes no longer gets returned as a reference from GetMeshes,
+    /// we need to implement these mutation functions to provide support via
+    /// lua.
+    void ClearRendererMeshes();
+
+    LuaClassInstance m_pLua2DTrans;
+    LuaTransferFun2DProxy* m_pLua2DTransPtr; /// Used by GLRenderer.
+    LuaClassInstance LuaGet2DTrans();
 
   private:
     float               m_fIsovalue;
     float               m_fCVIsovalue;
+
+
+    // Private Lua-only functions/variables
+  private:
+
+    LuaClassInstance m_pLuaDataset;
+    LuaDatasetProxy* m_pLuaDatasetPtr;
+    LuaClassInstance LuaGetDataset();
+
+    LuaClassInstance m_pLua1DTrans;
+    LuaTransferFun1DProxy* m_pLua1DTransPtr;
+    LuaClassInstance LuaGet1DTrans();
 };
 
 }; //namespace tuvok
