@@ -42,7 +42,7 @@
 #include "Renderer/GL/GLTexture2D.h"
 #include "Renderer/GPUMemMan/GPUMemMan.h"
 #include "Renderer/TFScaling.h"
-
+#include "Basics/MathTools.h"
 using namespace std;
 using namespace tuvok;
 
@@ -75,30 +75,31 @@ void GLSBVR::CleanupShaders() {
 }
 
 bool GLSBVR::LoadShaders() {
-
   if (!GLRenderer::LoadShaders()) {
     T_ERROR("Error in parent call -> aborting");
     return false;
   }
 
-  const std::string tfqn = m_pDataset
-                           ? (m_pDataset->GetComponentCount() == 3 ||
-                              m_pDataset->GetComponentCount() == 4)
-                              ? "vr-col-tfqn.glsl"
-                              : "vr-scal-tfqn.glsl"
-                           : "vr-scal-tfqn.glsl";
+  std::string tfqn = m_pDataset
+                     ? this->ColorData()
+                        ? "VRender1D-Color"
+                        : "VRender1D"
+                     : "VRender1D";
   const std::string tfqnLit = m_pDataset
-                           ? (m_pDataset->GetComponentCount() == 3 ||
-                              m_pDataset->GetComponentCount() == 4)
-                              ? "vr-col-tfqn-lit.glsl"
-                              : "vr-scal-tfqn-lit.glsl"
-                           : "vr-scal-tfqn.glsl";
+                           ? this->ColorData()
+                              ? "VRender1DLit-Color.glsl"
+                              : "VRender1DLit.glsl"
+                           : "VRender1DLit.glsl";
+  const std::string bias = tfqn + "-BScale.glsl";
+  tfqn += ".glsl";
 
   if(!LoadAndVerifyShader(&m_pProgram1DTrans[0], m_vShaderSearchDirs,
                           "GLSBVR-VS.glsl",
                           NULL,
                           "Volume3D.glsl",      // SampleVolume
                           tfqn.c_str(),         // VRender1D
+                          bias.c_str(),
+                          "VRender1DProxy.glsl",
                           "FTB.glsl",           // TraversalOrderDepColor
                           "GLSBVR-1D-FS.glsl", NULL) ||
      !LoadAndVerifyShader(&m_pProgram1DTrans[1], m_vShaderSearchDirs,
@@ -142,8 +143,10 @@ bool GLSBVR::LoadShaders() {
                           NULL,
                           "Volume3D.glsl",      // SampleVolume
                           tfqn.c_str(),         // VRender1D
+                          bias.c_str(),
                           "FTB.glsl",           // TraversalOrderDepColor
                           "lighting.glsl",      // Lighting (for Mesh)
+                          "VRender1DProxy.glsl",
                           "GLSBVR-Mesh-1D-FS.glsl", NULL) ||
      !LoadAndVerifyShader(&m_pProgram1DTransMesh[1], m_vShaderSearchDirs,
                           "GLSBVR-Mesh-VS.glsl",
@@ -218,13 +221,13 @@ void GLSBVR::SetDataDepShaderVars() {
     switch (m_eRenderMode) {
       case RM_1DTRANS: {
         m_pProgram1DTransMesh[m_bUseLighting ? 1 : 0]->Enable();
-        m_pProgram1DTransMesh[m_bUseLighting ? 1 : 0]->SetUniformVector("fTransScale",fScale);
+        m_pProgram1DTransMesh[m_bUseLighting ? 1 : 0]->Set("fTransScale",fScale);
         break;
       }
       case RM_2DTRANS: {
         m_pProgram2DTransMesh[m_bUseLighting ? 1 : 0]->Enable();
-        m_pProgram2DTransMesh[m_bUseLighting ? 1 : 0]->SetUniformVector("fTransScale",fScale);
-        m_pProgram2DTransMesh[m_bUseLighting ? 1 : 0]->SetUniformVector("fGradientScale",fGradientScale);
+        m_pProgram2DTransMesh[m_bUseLighting ? 1 : 0]->Set("fTransScale",fScale);
+        m_pProgram2DTransMesh[m_bUseLighting ? 1 : 0]->Set("fGradientScale",fGradientScale);
         break;
       }
 
@@ -237,13 +240,22 @@ void GLSBVR::SetDataDepShaderVars() {
     MESSAGE("setting TF bias (%5.3f) and scale (%5.3f)", bias_scale.first,
             bias_scale.second);
     m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->Enable();
-    m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->SetUniformVector("TFuncBias", bias_scale.first);
-    m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->SetUniformVector("fTransScale", bias_scale.second);
+    m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->Set("TFuncBias", bias_scale.first);
+    m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->Set("fTransScale", bias_scale.second);
   }
 }
 
 void GLSBVR::SetBrickDepShaderVars(const Brick& currentBrick) {
-  FLOATVECTOR3 vStep(1.0f/currentBrick.vVoxelCount.x, 1.0f/currentBrick.vVoxelCount.y, 1.0f/currentBrick.vVoxelCount.z);
+  FLOATVECTOR3 vVoxelSizeTexSpace;
+  if (m_bUseOnlyPowerOfTwo)  {
+    UINTVECTOR3 vP2VoxelCount(MathTools::NextPow2(currentBrick.vVoxelCount.x),
+                              MathTools::NextPow2(currentBrick.vVoxelCount.y),
+                              MathTools::NextPow2(currentBrick.vVoxelCount.z));
+
+    vVoxelSizeTexSpace = 1.0f/FLOATVECTOR3(vP2VoxelCount);
+  } else {
+    vVoxelSizeTexSpace = 1.0f/FLOATVECTOR3(currentBrick.vVoxelCount);
+  }
 
   float fStepScale = m_SBVRGeogen.GetOpacityCorrection();
 
@@ -254,9 +266,9 @@ void GLSBVR::SetBrickDepShaderVars(const Brick& currentBrick) {
                    ? (m_pProgram1DTransMesh[m_bUseLighting ? 1 : 0])
                    : (m_pProgram1DTrans[m_bUseLighting ? 1 : 0]));
       shader->Enable();
-      shader->SetUniformVector("fStepScale", fStepScale);
+      shader->Set("fStepScale", fStepScale);
       if (m_bUseLighting) {
-        shader->SetUniformVector("vVoxelStepsize", vStep.x, vStep.y, vStep.z);
+        shader->Set("vVoxelStepsize", vVoxelSizeTexSpace.x, vVoxelSizeTexSpace.y, vVoxelSizeTexSpace.z);
       }
       break;
     }
@@ -265,15 +277,14 @@ void GLSBVR::SetBrickDepShaderVars(const Brick& currentBrick) {
                     ? (m_pProgram2DTransMesh[m_bUseLighting ? 1 : 0])
                     : (m_pProgram2DTrans[m_bUseLighting ? 1 : 0]));
       shader->Enable();
-      shader->SetUniformVector("fStepScale", fStepScale);
-      shader->SetUniformVector("vVoxelStepsize", vStep.x, vStep.y, vStep.z);
+      shader->Set("fStepScale", fStepScale);
+      shader->Set("vVoxelStepsize", vVoxelSizeTexSpace.x, vVoxelSizeTexSpace.y, vVoxelSizeTexSpace.z);
       break;
     }
     case RM_ISOSURFACE: {
-      shader = (m_pDataset->GetComponentCount() == 1) ?
-               m_pProgramIso : m_pProgramColor;
+      shader = this->ColorData() ? m_pProgramColor : m_pProgramIso;
       shader->Enable();
-      shader->SetUniformVector("vVoxelStepsize", vStep.x, vStep.y, vStep.z);
+      shader->Set("vVoxelStepsize", vVoxelSizeTexSpace.x, vVoxelSizeTexSpace.y, vVoxelSizeTexSpace.z);
       break;
     }
     default: T_ERROR("Invalid rendermode set"); break;
@@ -353,17 +364,17 @@ void GLSBVR::RenderProxyGeometry() const {
 }
 
 void GLSBVR::Render3DInLoop(const RenderRegion3D& renderRegion,
-                            size_t iCurrentBrick, int iStereoID) {
+                            size_t iCurrentBrick, EStereoID eStereoID) {
 
   m_pContext->GetStateManager()->Apply(m_BaseState);
 
-  const Brick& b = (iStereoID == 0) ? m_vCurrentBrickList[iCurrentBrick] : m_vLeftEyeBrickList[iCurrentBrick];
+  const Brick& b = (eStereoID == SI_LEFT_OR_MONO) ? m_vCurrentBrickList[iCurrentBrick] : m_vLeftEyeBrickList[iCurrentBrick];
   
   if (m_iBricksRenderedInThisSubFrame == 0 && m_eRenderMode == RM_ISOSURFACE){
-    m_TargetBinder.Bind(m_pFBOIsoHit[iStereoID], 0, m_pFBOIsoHit[iStereoID], 1);
+    m_TargetBinder.Bind(m_pFBOIsoHit[size_t(eStereoID)], 0, m_pFBOIsoHit[size_t(eStereoID)], 1);
     GL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
     if (m_bDoClearView) {
-      m_TargetBinder.Bind(m_pFBOCVHit[iStereoID], 0, m_pFBOCVHit[iStereoID], 1);
+      m_TargetBinder.Bind(m_pFBOCVHit[size_t(eStereoID)], 0, m_pFBOCVHit[size_t(eStereoID)], 1);
       GL(glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT));
     }
   }
@@ -375,17 +386,17 @@ void GLSBVR::Render3DInLoop(const RenderRegion3D& renderRegion,
                             b.vTexcoordsMin, b.vTexcoordsMax);
   FLOATMATRIX4 maBricktTrans;
   maBricktTrans.Translation(b.vCenter.x, b.vCenter.y, b.vCenter.z);
-  m_mProjection[iStereoID].setProjection();
-  renderRegion.modelView[iStereoID].setModelview();
+  m_mProjection[size_t(eStereoID)].setProjection();
+  renderRegion.modelView[size_t(eStereoID)].setModelview();
 
   m_SBVRGeogen.SetBrickTrans(b.vCenter);
   m_SBVRGeogen.SetWorld(renderRegion.rotation*renderRegion.translation);
-  m_SBVRGeogen.SetView(m_mView[iStereoID]);
+  m_SBVRGeogen.SetView(m_mView[size_t(eStereoID)]);
 
   if (m_bSupportsMeshes) {
     m_SBVRGeogen.ResetMesh();
     if (m_eRenderMode != RM_ISOSURFACE) {
-      for (vector<RenderMesh*>::iterator mesh = m_Meshes.begin();
+      for (vector<shared_ptr<RenderMesh>>::iterator mesh = m_Meshes.begin();
            mesh != m_Meshes.end(); mesh++) {
         if ((*mesh)->GetActive()) {
           m_SBVRGeogen.AddMesh((*mesh)->GetInPointList(false));
@@ -399,26 +410,26 @@ void GLSBVR::Render3DInLoop(const RenderRegion3D& renderRegion,
   if (m_eRenderMode == RM_ISOSURFACE) {
     m_pContext->GetStateManager()->SetEnableBlend(false);
 
-    m_TargetBinder.Bind(m_pFBOIsoHit[iStereoID], 0, m_pFBOIsoHit[iStereoID], 1);
+    m_TargetBinder.Bind(m_pFBOIsoHit[size_t(eStereoID)], 0, m_pFBOIsoHit[size_t(eStereoID)], 1);
     SetBrickDepShaderVars(b);
     
-    GLSLProgram* shader = (m_pDataset->GetComponentCount() == 1) ? m_pProgramIso : m_pProgramColor;
-    shader->SetUniformVector("fIsoval", static_cast<float>
+    GLSLProgram* shader = this->ColorData() ? m_pProgramColor : m_pProgramIso;
+    shader->Set("fIsoval", static_cast<float>
                                         (this->GetNormalizedIsovalue()));    
     RenderProxyGeometry();
 
     if (m_bDoClearView) {
-      m_TargetBinder.Bind(m_pFBOCVHit[iStereoID], 0, m_pFBOCVHit[iStereoID], 1);
+      m_TargetBinder.Bind(m_pFBOCVHit[size_t(eStereoID)], 0, m_pFBOCVHit[size_t(eStereoID)], 1);
 
       m_pProgramIso->Enable();
-      m_pProgramIso->SetUniformVector("fIsoval", static_cast<float>
+      m_pProgramIso->Set("fIsoval", static_cast<float>
                                                  (GetNormalizedCVIsovalue()));
       RenderProxyGeometry();
     }
   } else {
     m_pContext->GetStateManager()->SetDepthMask(false);
 
-    m_TargetBinder.Bind(m_pFBO3DImageCurrent[iStereoID]);
+    m_TargetBinder.Bind(m_pFBO3DImageNext[size_t(eStereoID)]);
     SetBrickDepShaderVars(b);
     RenderProxyGeometry();
   }
@@ -476,34 +487,34 @@ void GLSBVR::UpdateLightParamsInShaders() {
   FLOATVECTOR3 scale = 1.0f/FLOATVECTOR3(m_pDataset->GetScale());
 
   m_pProgram1DTransMesh[0]->Enable();
-  m_pProgram1DTransMesh[0]->SetUniformVector("vLightAmbientM",aM.x,aM.y,aM.z);
-  m_pProgram1DTransMesh[0]->SetUniformVector("vLightDiffuseM",dM.x,dM.y,dM.z);
-  m_pProgram1DTransMesh[0]->SetUniformVector("vLightSpecularM",sM.x,sM.y,sM.z);
-  m_pProgram1DTransMesh[0]->SetUniformVector("vLightDir",m_vLightDir.x,m_vLightDir.y,m_vLightDir.z);
+  m_pProgram1DTransMesh[0]->Set("vLightAmbientM",aM.x,aM.y,aM.z);
+  m_pProgram1DTransMesh[0]->Set("vLightDiffuseM",dM.x,dM.y,dM.z);
+  m_pProgram1DTransMesh[0]->Set("vLightSpecularM",sM.x,sM.y,sM.z);
+  m_pProgram1DTransMesh[0]->Set("vLightDir",m_vLightDir.x,m_vLightDir.y,m_vLightDir.z);
 
   m_pProgram2DTransMesh[0]->Enable();
-  m_pProgram2DTransMesh[0]->SetUniformVector("vLightAmbientM",aM.x,aM.y,aM.z);
-  m_pProgram2DTransMesh[0]->SetUniformVector("vLightDiffuseM",dM.x,dM.y,dM.z);
-  m_pProgram2DTransMesh[0]->SetUniformVector("vLightSpecularM",sM.x,sM.y,sM.z);
-  m_pProgram2DTransMesh[0]->SetUniformVector("vLightDir",m_vLightDir.x,m_vLightDir.y,m_vLightDir.z);
+  m_pProgram2DTransMesh[0]->Set("vLightAmbientM",aM.x,aM.y,aM.z);
+  m_pProgram2DTransMesh[0]->Set("vLightDiffuseM",dM.x,dM.y,dM.z);
+  m_pProgram2DTransMesh[0]->Set("vLightSpecularM",sM.x,sM.y,sM.z);
+  m_pProgram2DTransMesh[0]->Set("vLightDir",m_vLightDir.x,m_vLightDir.y,m_vLightDir.z);
 
   m_pProgram1DTransMesh[1]->Enable();
-  m_pProgram1DTransMesh[1]->SetUniformVector("vLightAmbient",a.x,a.y,a.z);
-  m_pProgram1DTransMesh[1]->SetUniformVector("vLightDiffuse",d.x,d.y,d.z);
-  m_pProgram1DTransMesh[1]->SetUniformVector("vLightSpecular",s.x,s.y,s.z);
-  m_pProgram1DTransMesh[1]->SetUniformVector("vLightAmbientM",aM.x,aM.y,aM.z);
-  m_pProgram1DTransMesh[1]->SetUniformVector("vLightDiffuseM",dM.x,dM.y,dM.z);
-  m_pProgram1DTransMesh[1]->SetUniformVector("vLightSpecularM",sM.x,sM.y,sM.z);
-  m_pProgram1DTransMesh[1]->SetUniformVector("vLightDir",m_vLightDir.x,m_vLightDir.y,m_vLightDir.z);
-  m_pProgram1DTransMesh[1]->SetUniformVector("vDomainScale",scale.x,scale.y,scale.z);
+  m_pProgram1DTransMesh[1]->Set("vLightAmbient",a.x,a.y,a.z);
+  m_pProgram1DTransMesh[1]->Set("vLightDiffuse",d.x,d.y,d.z);
+  m_pProgram1DTransMesh[1]->Set("vLightSpecular",s.x,s.y,s.z);
+  m_pProgram1DTransMesh[1]->Set("vLightAmbientM",aM.x,aM.y,aM.z);
+  m_pProgram1DTransMesh[1]->Set("vLightDiffuseM",dM.x,dM.y,dM.z);
+  m_pProgram1DTransMesh[1]->Set("vLightSpecularM",sM.x,sM.y,sM.z);
+  m_pProgram1DTransMesh[1]->Set("vLightDir",m_vLightDir.x,m_vLightDir.y,m_vLightDir.z);
+  m_pProgram1DTransMesh[1]->Set("vDomainScale",scale.x,scale.y,scale.z);
 
   m_pProgram2DTransMesh[1]->Enable();
-  m_pProgram2DTransMesh[1]->SetUniformVector("vLightAmbient",a.x,a.y,a.z);
-  m_pProgram2DTransMesh[1]->SetUniformVector("vLightDiffuse",d.x,d.y,d.z);
-  m_pProgram2DTransMesh[1]->SetUniformVector("vLightSpecular",s.x,s.y,s.z);
-  m_pProgram2DTransMesh[1]->SetUniformVector("vLightAmbientM",aM.x,aM.y,aM.z);
-  m_pProgram2DTransMesh[1]->SetUniformVector("vLightDiffuseM",dM.x,dM.y,dM.z);
-  m_pProgram2DTransMesh[1]->SetUniformVector("vLightSpecularM",sM.x,sM.y,sM.z);
-  m_pProgram2DTransMesh[1]->SetUniformVector("vLightDir",m_vLightDir.x,m_vLightDir.y,m_vLightDir.z);
-  m_pProgram2DTransMesh[1]->SetUniformVector("vDomainScale",scale.x,scale.y,scale.z);
+  m_pProgram2DTransMesh[1]->Set("vLightAmbient",a.x,a.y,a.z);
+  m_pProgram2DTransMesh[1]->Set("vLightDiffuse",d.x,d.y,d.z);
+  m_pProgram2DTransMesh[1]->Set("vLightSpecular",s.x,s.y,s.z);
+  m_pProgram2DTransMesh[1]->Set("vLightAmbientM",aM.x,aM.y,aM.z);
+  m_pProgram2DTransMesh[1]->Set("vLightDiffuseM",dM.x,dM.y,dM.z);
+  m_pProgram2DTransMesh[1]->Set("vLightSpecularM",sM.x,sM.y,sM.z);
+  m_pProgram2DTransMesh[1]->Set("vLightDir",m_vLightDir.x,m_vLightDir.y,m_vLightDir.z);
+  m_pProgram2DTransMesh[1]->Set("vDomainScale",scale.x,scale.y,scale.z);
 }
