@@ -68,12 +68,21 @@ void GLSBVR2D::CleanupShaders() {
   GLRenderer::CleanupShaders();
 }
 
-void GLSBVR2D::SetUse3DTexture(bool bUse3DTexture) {
+void GLSBVR2D::SetInterpolant(Interpolant eInterpolant) {
+  if (m_eInterpolant != eInterpolant) {
+    m_eInterpolant = eInterpolant;
+    CleanupShaders();
+    LoadShaders();
+    ScheduleCompleteRedraw();
+  }
+}
 
+void GLSBVR2D::SetUse3DTexture(bool bUse3DTexture) {
   if (bUse3DTexture != m_bUse3DTexture) {
     m_bUse3DTexture = bUse3DTexture;
     CleanupShaders();
     LoadShaders();
+    ScheduleCompleteRedraw();
   }
 }
 
@@ -91,38 +100,49 @@ bool GLSBVR2D::LoadShaders() {
   // do not call GLRenderer::LoadShaders as we want to control
   // what volume access function is linked (Volume3D or Volume2D)
 
-  string volumeAccessFunction = m_bUse3DTexture ? "Volume3D.glsl"
-                                                : "Volume2D.glsl";
+  string volumeAccessFunction = m_bUse3DTexture ? "Volume3D"
+                                                : "Volume2D";
+  // add the appropriate suffix in 2D.  We need separate shaders because we do
+  // manual sampling in the 2D shaders.
+  if (!m_bUse3DTexture) {
+    switch (m_eInterpolant) {
+      case Linear :          volumeAccessFunction += "-linear"; break;
+      case NearestNeighbor : volumeAccessFunction += "-nearest"; break;
+    }
+  }
+  volumeAccessFunction += ".glsl";
 
-  if (!GLRenderer::LoadShaders(volumeAccessFunction)) {
+  if (!GLRenderer::LoadShaders(volumeAccessFunction, m_bUse3DTexture)) {
     T_ERROR("Error in parent call -> aborting");
     return false;
   }
 
-  const std::string tfqn = m_pDataset
-                           ? (m_pDataset->GetComponentCount() == 3 ||
-                              m_pDataset->GetComponentCount() == 4)
-                              ? "vr-col-tfqn.glsl"
-                              : "vr-scal-tfqn.glsl"
-                           : "vr-scal-tfqn.glsl";
+  std::string tfqn = m_pDataset
+                     ? this->ColorData()
+                        ? "VRender1D-Color"
+                        : "VRender1D"
+                     : "VRender1D";
   const std::string tfqnLit = m_pDataset
-                           ? (m_pDataset->GetComponentCount() == 3 ||
-                              m_pDataset->GetComponentCount() == 4)
-                              ? "vr-col-tfqn-lit.glsl"
-                              : "vr-scal-tfqn-lit.glsl"
-                           : "vr-scal-tfqn.glsl";
+                           ? this->ColorData()
+                              ? "VRender1DLit-Color.glsl"
+                              : "VRender1DLit.glsl"
+                           : "VRender1DLit.glsl";
+  const std::string bias = tfqn + "-BScale.glsl";
+  tfqn += ".glsl";
   
   if(!LoadAndVerifyShader(&m_pProgram1DTrans[0], m_vShaderSearchDirs,
                           "GLSBVR-VS.glsl",
                           NULL,
-                          volumeAccessFunction.c_str(), // SampleVolume
+                          volumeAccessFunction.c_str(), // sampleVolume
                           tfqn.c_str(),         // VRender1D
+                          bias.c_str(),
+                          "VRender1DProxy.glsl",
                           "FTB.glsl",           // TraversalOrderDepColor
                           "GLSBVR-1D-FS.glsl", NULL) ||
      !LoadAndVerifyShader(&m_pProgram1DTrans[1], m_vShaderSearchDirs,
                           "GLSBVR-VS.glsl",
                           NULL,                          
-                          volumeAccessFunction.c_str(), // SampleVolume
+                          volumeAccessFunction.c_str(),
                           tfqnLit.c_str(),         // VRender1DLit
                           "lighting.glsl",      // Lighting
                           "FTB.glsl",           // TraversalOrderDepColor
@@ -130,30 +150,30 @@ bool GLSBVR2D::LoadShaders() {
      !LoadAndVerifyShader(&m_pProgram2DTrans[0], m_vShaderSearchDirs,
                           "GLSBVR-VS.glsl",
                           NULL,
-                          volumeAccessFunction.c_str(), // SampleVolume, ComputeGradient
+                          volumeAccessFunction.c_str(),
                           "FTB.glsl",           // TraversalOrderDepColor
                           "GLSBVR-2D-FS.glsl", NULL) ||
      !LoadAndVerifyShader(&m_pProgram2DTrans[1], m_vShaderSearchDirs,
                           "GLSBVR-VS.glsl",
                           NULL,
-                          volumeAccessFunction.c_str(), // SampleVolume, ComputeGradient
-                          "lighting.glsl",      // Lighting
+                          volumeAccessFunction.c_str(),
+                          "lighting.glsl",
                           "FTB.glsl",           // TraversalOrderDepColor
                           "GLSBVR-2D-light-FS.glsl", NULL) ||
      !LoadAndVerifyShader(&m_pProgramHQMIPRot, m_vShaderSearchDirs,
                           "GLSBVR-VS.glsl",
                           NULL,
-                          volumeAccessFunction.c_str(), // SampleVolume
+                          volumeAccessFunction.c_str(),
                           "GLSBVR-MIP-Rot-FS.glsl", NULL) ||
      !LoadAndVerifyShader(&m_pProgramIso, m_vShaderSearchDirs,
                           "GLSBVR-VS.glsl",
                           NULL,
-                          volumeAccessFunction.c_str(), // SampleVolume, ComputeNormal
+                          volumeAccessFunction.c_str(),
                           "GLSBVR-ISO-FS.glsl", NULL) ||
      !LoadAndVerifyShader(&m_pProgramColor, m_vShaderSearchDirs,
                           "GLSBVR-VS.glsl",
                           NULL,
-                          volumeAccessFunction.c_str(), // SampleVolume, ComputeNormal
+                          volumeAccessFunction.c_str(),
                           "GLSBVR-Color-FS.glsl", NULL))
   {
       Cleanup();
@@ -192,8 +212,8 @@ void GLSBVR2D::SetDataDepShaderVars() {
     MESSAGE("setting TF bias (%5.3f) and scale (%5.3f)", bias_scale.first,
             bias_scale.second);
     m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->Enable();
-    m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->SetUniformVector("TFuncBias", bias_scale.first);
-    m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->SetUniformVector("fTransScale", bias_scale.second);
+    m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->Set("TFuncBias", bias_scale.first);
+    m_pProgram1DTrans[m_bUseLighting ? 1 : 0]->Set("fTransScale", bias_scale.second);
   }
 }
 
@@ -215,24 +235,23 @@ void GLSBVR2D::SetBrickDepShaderVars(const RenderRegion3D&,
   switch (m_eRenderMode) {
     case RM_1DTRANS: {
       GLSLProgram *shader = m_pProgram1DTrans[m_bUseLighting ? 1 : 0];
-      shader->SetUniformVector("fStepScale", fStepScale);
+      shader->Set("fStepScale", fStepScale);
       if (m_bUseLighting) {
-        m_pProgram1DTrans[1]->SetUniformVector("vVoxelStepsize",
+        m_pProgram1DTrans[1]->Set("vVoxelStepsize",
                                                vStep.x, vStep.y, vStep.z);
       }
       break;
     }
     case RM_2DTRANS: {
       GLSLProgram *shader = m_pProgram2DTrans[m_bUseLighting ? 1 : 0];
-      shader->SetUniformVector("fStepScale", fStepScale);
-      shader->SetUniformVector("vVoxelStepsize", vStep.x, vStep.y, vStep.z);
+      shader->Set("fStepScale", fStepScale);
+      shader->Set("vVoxelStepsize", vStep.x, vStep.y, vStep.z);
       break;
     }
     case RM_ISOSURFACE: {
       GLSLProgram *shader;
-      shader = (m_pDataset->GetComponentCount() == 1) ?
-               m_pProgramIso : m_pProgramColor;
-      shader->SetUniformVector("vVoxelStepsize", vStep.x, vStep.y, vStep.z);
+      shader = this->ColorData() ? m_pProgramColor : m_pProgramIso;
+      shader->Set("vVoxelStepsize", vStep.x, vStep.y, vStep.z);
       break;
     }
     case RM_INVALID: T_ERROR("Invalid rendermode set"); break;
@@ -294,7 +313,7 @@ void GLSBVR2D::RenderProxyGeometry() const {
 // data structure; for each slice_geom, we store the texture to use, a
 // set of (3D) texture coords, and a set of (3D) vertices.
 struct slice_geom {
-  UINT32 texid;
+  uint32_t texid;
   std::vector<float> texcoords;
   std::vector<float> tris;
 };
@@ -311,7 +330,7 @@ static void submit_vert_arrays(const GLVolume2DTex* vol,
         slice != slices.end(); ++slice) {
       // skip empty arrays.
       if(slice->texcoords.empty() || slice->tris.empty()) { continue; }
-
+    
       vol->Bind(0, static_cast<int>(slice->texid)+0, static_cast<int>(dimension));
       vol->Bind(2, static_cast<int>(slice->texid)+1, static_cast<int>(dimension));
       vol->Bind(3, static_cast<int>(slice->texid)+2, static_cast<int>(dimension));
@@ -527,18 +546,16 @@ void GLSBVR2D::RenderProxyGeometry3D() const {
 }
 
 void GLSBVR2D::Render3DInLoop(const RenderRegion3D& renderRegion,
-                              size_t iCurrentBrick, int iStereoID) {
-
-
+                              size_t iCurrentBrick, EStereoID eStereoID) {
   m_pContext->GetStateManager()->Apply(m_BaseState);
 
-  const Brick& b = (iStereoID == 0) ? m_vCurrentBrickList[iCurrentBrick] : m_vLeftEyeBrickList[iCurrentBrick];
+  const Brick& b = (eStereoID == SI_LEFT_OR_MONO) ? m_vCurrentBrickList[iCurrentBrick] : m_vLeftEyeBrickList[iCurrentBrick];
 
   if (m_iBricksRenderedInThisSubFrame == 0 && m_eRenderMode == RM_ISOSURFACE){
-    m_TargetBinder.Bind(m_pFBOIsoHit[iStereoID], 0, m_pFBOIsoHit[iStereoID], 1);
+    m_TargetBinder.Bind(m_pFBOIsoHit[size_t(eStereoID)], 0, m_pFBOIsoHit[size_t(eStereoID)], 1);
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
     if (m_bDoClearView) {
-      m_TargetBinder.Bind(m_pFBOCVHit[iStereoID], 0, m_pFBOCVHit[iStereoID], 1);
+      m_TargetBinder.Bind(m_pFBOCVHit[size_t(eStereoID)], 0, m_pFBOCVHit[size_t(eStereoID)], 1);
       glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
     }
   }
@@ -553,37 +570,37 @@ void GLSBVR2D::Render3DInLoop(const RenderRegion3D& renderRegion,
                             b.vTexcoordsMin, b.vTexcoordsMax);
   FLOATMATRIX4 maBricktTrans;
   maBricktTrans.Translation(b.vCenter.x, b.vCenter.y, b.vCenter.z);
-  m_mProjection[iStereoID].setProjection();
-  renderRegion.modelView[iStereoID].setModelview();
+  m_mProjection[size_t(eStereoID)].setProjection();
+  renderRegion.modelView[size_t(eStereoID)].setModelview();
 
   m_SBVRGeogen.SetBrickTrans(b.vCenter);
   m_SBVRGeogen.SetWorld(renderRegion.rotation*renderRegion.translation);
-  m_SBVRGeogen.SetView(m_mView[iStereoID]);
+  m_SBVRGeogen.SetView(m_mView[size_t(eStereoID)]);
 
   m_SBVRGeogen.ComputeGeometry(b.bIsEmpty);
 
   if (m_eRenderMode == RM_ISOSURFACE) {
     m_pContext->GetStateManager()->SetEnableBlend(false);
 
-    GLSLProgram* shader = (m_pDataset->GetComponentCount() == 1) ? m_pProgramIso : m_pProgramColor;
-    m_TargetBinder.Bind(m_pFBOIsoHit[iStereoID], 0, m_pFBOIsoHit[iStereoID], 1);
+    GLSLProgram* shader = this->ColorData() ? m_pProgramColor : m_pProgramIso;
+    m_TargetBinder.Bind(m_pFBOIsoHit[size_t(eStereoID)], 0, m_pFBOIsoHit[size_t(eStereoID)], 1);
 
     shader->Enable();
     SetBrickDepShaderVars(renderRegion, b);
-    shader->SetUniformVector("fIsoval", static_cast<float>
+    shader->Set("fIsoval", static_cast<float>
                                         (this->GetNormalizedIsovalue()));
     RenderProxyGeometry();
 
     if (m_bDoClearView) {
-      m_TargetBinder.Bind(m_pFBOCVHit[iStereoID], 0, m_pFBOCVHit[iStereoID], 1);
+      m_TargetBinder.Bind(m_pFBOCVHit[size_t(eStereoID)], 0, m_pFBOCVHit[size_t(eStereoID)], 1);
 
       m_pProgramIso->Enable();
-      m_pProgramIso->SetUniformVector("fIsoval", static_cast<float>
+      m_pProgramIso->Set("fIsoval", static_cast<float>
                                                  (GetNormalizedCVIsovalue()));
       RenderProxyGeometry();
     }
   } else {
-    m_TargetBinder.Bind(m_pFBO3DImageCurrent[iStereoID]);
+    m_TargetBinder.Bind(m_pFBO3DImageNext[size_t(eStereoID)]);
 
     m_pContext->GetStateManager()->SetDepthMask(false);
     SetBrickDepShaderVars(renderRegion, b);
@@ -635,8 +652,8 @@ bool GLSBVR2D::LoadDataset(const string& strFilename) {
   } else return false;
 }
 
-void GLSBVR2D::ComposeSurfaceImage(RenderRegion& renderRegion, int iStereoID) {
-  GLRenderer::ComposeSurfaceImage(renderRegion, iStereoID);
+void GLSBVR2D::ComposeSurfaceImage(const RenderRegion& renderRegion, EStereoID eStereoID) {
+  GLRenderer::ComposeSurfaceImage(renderRegion, eStereoID);
 }
 
 void GLSBVR2D::UpdateLightParamsInShaders() {
@@ -644,7 +661,7 @@ void GLSBVR2D::UpdateLightParamsInShaders() {
 }
 
 bool GLSBVR2D::BindVolumeTex(const BrickKey& bkey,
-                             const UINT64 iIntraFrameCounter) {
+                             const uint64_t iIntraFrameCounter) {
 
   if (m_bUse3DTexture) return GLRenderer::BindVolumeTex(bkey,iIntraFrameCounter);
 
@@ -654,8 +671,10 @@ bool GLSBVR2D::BindVolumeTex(const BrickKey& bkey,
                                             m_bDisableBorder,
                                             true,
                                             iIntraFrameCounter,
-                                            m_iFrameCounter);
+                                            m_iFrameCounter,
+                                            m_pContext->GetShareGroupID());
   if(m_pGLVolume) {
+    m_pGLVolume->SetFilter(ComputeGLFilter(), ComputeGLFilter());
     return true;
   } else {
     return false;
@@ -667,17 +686,18 @@ bool GLSBVR2D::IsVolumeResident(const BrickKey& key) const{
     return GLRenderer::IsVolumeResident(key);
   else
     return m_pMasterController->MemMan()->IsResident(m_pDataset, key,
-                                                  m_bUseOnlyPowerOfTwo,
-                                                  m_bDownSampleTo8Bits,
-                                                  m_bDisableBorder,
-                                                  true);
+      m_bUseOnlyPowerOfTwo, m_bDownSampleTo8Bits, m_bDisableBorder, true,
+      m_pContext->GetShareGroupID()
+    );
 }
 
 void GLSBVR2D::RenderSlice(const RenderRegion2D& region, double fSliceIndex,
                              FLOATVECTOR3 vMinCoords, FLOATVECTOR3 vMaxCoords,
                              DOUBLEVECTOR3 vAspectRatio,
                              DOUBLEVECTOR2 vWinAspectRatio) {
+  
   GLVolume2DTex* pGLVolume =  static_cast<GLVolume2DTex*>(m_pGLVolume);
+
   switch (region.windowMode) {
     case RenderRegion::WM_AXIAL :
     {
