@@ -35,12 +35,9 @@
 */
 
 #include <cassert>
+#include <cstddef>
 #include <algorithm>
-#ifndef _MSC_VER
-# include <tr1/functional>
-#else
-# include <functional>
-#endif
+#include <functional>
 #include <numeric>
 #include <typeinfo>
 // normally we'd include Qt headers first, but we want to make sure to get GLEW
@@ -59,14 +56,25 @@
 #include "IO/IOManager.h"
 #include "IO/TransferFunction1D.h"
 #include "IO/TransferFunction2D.h"
+#include "IO/TuvokSizes.h"
 #include "Renderer/AbstrRenderer.h"
+#include "Renderer/ShaderDescriptor.h"
 #include "Renderer/GL/GLError.h"
 #include "Renderer/GL/GLTexture1D.h"
 #include "Renderer/GL/GLTexture2D.h"
 #include "Renderer/GL/GLVolume.h"
 
+#include "IO/uvfDataset.h"
+#include "Renderer/GL/GLVolumePool.h"
+
+#include "../LuaScripting/LuaScripting.h"
+#include "../LuaScripting/LuaMemberReg.h"
+#include "../LuaScripting/TuvokSpecific/LuaTuvokTypes.h"
+#include "../LuaScripting/TuvokSpecific/LuaTransferFun1DProxy.h"
+#include "../LuaScripting/TuvokSpecific/LuaTransferFun2DProxy.h"
+
 using namespace std;
-using namespace std::tr1::placeholders;
+using namespace std::placeholders;
 using namespace tuvok;
 
 GPUMemMan::GPUMemMan(MasterController* masterController) :
@@ -75,7 +83,8 @@ GPUMemMan::GPUMemMan(MasterController* masterController) :
   m_iAllocatedGPUMemory(0),
   m_iAllocatedCPUMemory(0),
   m_iFrameCounter(0),
-  m_iInCoreSize(DEFAULT_INCORESIZE)
+  m_iInCoreSize(DEFAULT_INCORESIZE),
+  m_pMemReg(new LuaMemberReg(masterController->LuaScript()))
 {
   if (masterController && masterController->IOMan()) {
     m_iInCoreSize = masterController->IOMan()->GetIncoresize();
@@ -83,6 +92,8 @@ GPUMemMan::GPUMemMan(MasterController* masterController) :
 
   m_vUploadHub.resize(size_t(m_iInCoreSize*4));
   m_iAllocatedCPUMemory = size_t(m_iInCoreSize*4);
+
+  RegisterLuaCommands();
 }
 
 GPUMemMan::~GPUMemMan() {
@@ -282,7 +293,8 @@ void GPUMemMan::FreeDataset(Dataset* pVolumeDataset,
   if(vol_ds->qpUser.empty()) {
     dbg.Message(_func_, "Cleaning up all 3D textures associated w/ dataset %s",
                 ds_name.c_str());
-    FreeAssociatedTextures(pVolumeDataset);
+    if (requester->GetContext()) // if we never created a context then we never created any textures
+      FreeAssociatedTextures(pVolumeDataset, requester->GetContext()->GetShareGroupID());
     dbg.Message(_func_, "Released Dataset %s", ds_name.c_str());
     delete pVolumeDataset;
     m_vpVolumeDatasets.erase(vol_ds);
@@ -294,10 +306,10 @@ void GPUMemMan::FreeDataset(Dataset* pVolumeDataset,
 
 // ******************** Simple Textures
 
-GLTexture2D* GPUMemMan::Load2DTextureFromFile(const string& strFilename) {
+GLTexture2D* GPUMemMan::Load2DTextureFromFile(const string& strFilename, int iShareGroupID) {
   for (SimpleTextureListIter i = m_vpSimpleTextures.begin();
        i < m_vpSimpleTextures.end(); ++i) {
-    if (i->strFilename == strFilename) {
+    if (i->strFilename == strFilename && i->m_iShareGroupID == iShareGroupID) {
       MESSAGE("Reusing %s", strFilename.c_str());
       i->iAccessCounter++;
       return i->pTexture;
@@ -315,13 +327,13 @@ GLTexture2D* GPUMemMan::Load2DTextureFromFile(const string& strFilename) {
   QImage glimage = QGLWidget::convertToGLFormat(image);
 
   GLTexture2D* tex = new GLTexture2D(glimage.width(),glimage.height(),
-                                     GL_RGBA8, GL_RGBA, GL_UNSIGNED_BYTE, 4,
-                                     glimage.bits(), GL_LINEAR, GL_LINEAR);
+                                     GL_RGBA8, GL_RGBA, GL_UNSIGNED_BYTE,
+                                     glimage.bits());
 
   m_iAllocatedGPUMemory += tex->GetGPUSize();
   m_iAllocatedCPUMemory += tex->GetCPUSize();
 
-  m_vpSimpleTextures.push_back(SimpleTextureListElem(1,tex,strFilename));
+  m_vpSimpleTextures.push_back(SimpleTextureListElem(1,tex,strFilename,iShareGroupID));
   return tex;
 #else
   T_ERROR("No Qt support!");
@@ -356,12 +368,25 @@ void GPUMemMan::FreeTexture(GLTexture2D* pTexture) {
 
 // ******************** 1D Trans
 
-void GPUMemMan::Changed1DTrans(const AbstrRenderer* requester,
-                               TransferFunction1D* pTransferFunction1D) {
+void GPUMemMan::Changed1DTrans(LuaClassInstance luaAbstrRen,
+                               LuaClassInstance tf1d) {
   MESSAGE("Sending change notification for 1D transfer function");
 
+  shared_ptr<LuaScripting> ss = m_MasterController->LuaScript();
+
+  // Convert LuaClassInstance -> LuaTransferFun1DProxy -> TransferFunction1D
+  LuaTransferFun1DProxy* tfProxy = 
+      tf1d.getRawPointer<LuaTransferFun1DProxy>(ss);
+  TransferFunction1D* pTransferFunction1D = tfProxy->get1DTransferFunction();
+
   pTransferFunction1D->ComputeNonZeroLimits();
 
+  // Retrieve raw pointer for the Abstract renderer.
+  AbstrRenderer* requester = NULL;
+  if (luaAbstrRen.isValid(ss)) {
+    requester = luaAbstrRen.getRawPointer<AbstrRenderer>(ss);
+  }
+
   for (Trans1DListIter i = m_vpTrans1DList.begin();i<m_vpTrans1DList.end();i++) {
     if (i->pTransferFunction1D == pTransferFunction1D) {
       for (AbstrRendererListIter j = i->qpUser.begin();j<i->qpUser.end();j++) {
@@ -381,9 +406,8 @@ void GPUMemMan::GetEmpty1DTrans(size_t iSize, AbstrRenderer* requester,
 
   std::vector<unsigned char> vTFData;
   (*ppTransferFunction1D)->GetByteArray(vTFData);
-  *tex = new GLTexture1D(UINT32((*ppTransferFunction1D)->GetSize()), GL_RGBA8,
-                         GL_RGBA, GL_UNSIGNED_BYTE, 4, &vTFData.at(0),
-                         GL_LINEAR, GL_LINEAR);
+  *tex = new GLTexture1D(uint32_t((*ppTransferFunction1D)->GetSize()), GL_RGBA8,
+                         GL_RGBA, GL_UNSIGNED_BYTE, &vTFData.at(0));
 
   m_iAllocatedGPUMemory += (*tex)->GetGPUSize();
   m_iAllocatedCPUMemory += (*tex)->GetCPUSize();
@@ -405,9 +429,8 @@ void GPUMemMan::Get1DTransFromFile(const string& strFilename,
 
   std::vector<unsigned char> vTFData;
   (*ppTransferFunction1D)->GetByteArray(vTFData);
-  *tex = new GLTexture1D(UINT32((*ppTransferFunction1D)->GetSize()), GL_RGBA8,
-                         GL_RGBA, GL_UNSIGNED_BYTE, 4, &vTFData.at(0),
-                         GL_LINEAR, GL_LINEAR);
+  *tex = new GLTexture1D(uint32_t((*ppTransferFunction1D)->GetSize()), GL_RGBA8,
+                         GL_RGBA, GL_UNSIGNED_BYTE, &vTFData.at(0));
 
   m_iAllocatedGPUMemory += (*tex)->GetGPUSize();
   m_iAllocatedCPUMemory += (*tex)->GetCPUSize();
@@ -422,15 +445,15 @@ GPUMemMan::SetExternal1DTrans(const std::vector<unsigned char>& rgba,
 {
   const size_t sz = rgba.size() / 4; // RGBA, i.e. 4 components.
   MESSAGE("Setting %u element 1D TF from external source.",
-          static_cast<UINT32>(sz));
+          static_cast<uint32_t>(sz));
   assert(!rgba.empty());
 
   TransferFunction1D *tf1d = new TransferFunction1D(sz);
   tf1d->Set(rgba);
 
-  GLTexture1D *tex = new GLTexture1D(static_cast<UINT32>(tf1d->GetSize()),
-                                     GL_RGBA8, GL_RGBA, GL_UNSIGNED_BYTE, 4,
-                                     &rgba.at(0), GL_LINEAR, GL_LINEAR);
+  GLTexture1D *tex = new GLTexture1D(static_cast<uint32_t>(tf1d->GetSize()),
+                                     GL_RGBA8, GL_RGBA, GL_UNSIGNED_BYTE,
+                                     &rgba.at(0));
   m_iAllocatedGPUMemory += tex->GetGPUSize();
   m_iAllocatedCPUMemory += tex->GetCPUSize();
 
@@ -486,13 +509,26 @@ void GPUMemMan::Free1DTrans(TransferFunction1D* pTransferFunction1D,
 
 // ******************** 2D Trans
 
-void GPUMemMan::Changed2DTrans(const AbstrRenderer* requester,
-                               TransferFunction2D* pTransferFunction2D) {
+void GPUMemMan::Changed2DTrans(LuaClassInstance luaAbstrRen,
+                               LuaClassInstance tf2d) {
   MESSAGE("Sending change notification for 2D transfer function");
 
+  shared_ptr<LuaScripting> ss = m_MasterController->LuaScript();
+
+  // Convert LuaClassInstance -> LuaTransferFun2DProxy -> TransferFunction2D
+  LuaTransferFun2DProxy* tfProxy = 
+      tf2d.getRawPointer<LuaTransferFun2DProxy>(ss);
+  TransferFunction2D* pTransferFunction2D = tfProxy->get2DTransferFunction();
+
   pTransferFunction2D->InvalidateCache();
   pTransferFunction2D->ComputeNonZeroLimits();
 
+  // Retrieve raw pointer for the Abstract renderer.
+  AbstrRenderer* requester = NULL;
+  if (luaAbstrRen.isValid(ss)) {
+    requester = luaAbstrRen.getRawPointer<AbstrRenderer>(ss);
+  }
+
   for (Trans2DListIter i = m_vpTrans2DList.begin();
        i < m_vpTrans2DList.end(); ++i) {
     if (i->pTransferFunction2D == pTransferFunction2D) {
@@ -513,14 +549,15 @@ void GPUMemMan::GetEmpty2DTrans(const VECTOR2<size_t>& iSize,
 
   unsigned char* pcData = NULL;
   (*ppTransferFunction2D)->GetByteArray(&pcData);
-  *tex = new GLTexture2D(UINT32(iSize.x), UINT32(iSize.y), GL_RGBA8, GL_RGBA,
-                         GL_UNSIGNED_BYTE, 4, pcData);
+  *tex = new GLTexture2D(uint32_t(iSize.x), uint32_t(iSize.y), GL_RGBA8, GL_RGBA,
+                         GL_UNSIGNED_BYTE, pcData);
   delete [] pcData;
 
   m_iAllocatedGPUMemory += (*tex)->GetGPUSize();
   m_iAllocatedCPUMemory += (*tex)->GetCPUSize();
 
-  m_vpTrans2DList.push_back(Trans2DListElem(*ppTransferFunction2D, *tex, requester));
+  m_vpTrans2DList.push_back(Trans2DListElem(*ppTransferFunction2D, *tex,
+                                            requester));
 }
 
 void GPUMemMan::Get2DTransFromFile(const string& strFilename,
@@ -546,9 +583,9 @@ void GPUMemMan::Get2DTransFromFile(const string& strFilename,
 
   unsigned char* pcData = NULL;
   (*ppTransferFunction2D)->GetByteArray(&pcData);
-  *tex = new GLTexture2D(UINT32((*ppTransferFunction2D)->GetSize().x),
-                         UINT32((*ppTransferFunction2D)->GetSize().y),
-                         GL_RGBA8, GL_RGBA, GL_UNSIGNED_BYTE,4,pcData);
+  *tex = new GLTexture2D(uint32_t((*ppTransferFunction2D)->GetSize().x),
+                         uint32_t((*ppTransferFunction2D)->GetSize().y),
+                         GL_RGBA8, GL_RGBA, GL_UNSIGNED_BYTE,pcData);
   delete [] pcData;
 
   m_iAllocatedGPUMemory += (*tex)->GetGPUSize();
@@ -610,12 +647,13 @@ bool GPUMemMan::IsResident(const Dataset* pDataset,
                            const BrickKey& key,
                            bool bUseOnlyPowerOfTwo, bool bDownSampleTo8Bits,
                            bool bDisableBorder,
-                           bool bEmulate3DWith2DStacks) const {
+                           bool bEmulate3DWith2DStacks,
+                           int iShareGroupID) const {
   for(GLVolumeListConstIter i = m_vpTex3DList.begin();
       i < m_vpTex3DList.end(); ++i) {
     if((*i)->Equals(pDataset, key, bUseOnlyPowerOfTwo,
                     bDownSampleTo8Bits, bDisableBorder,
-                    bEmulate3DWith2DStacks)) {
+                    bEmulate3DWith2DStacks, iShareGroupID)) {
       return true;
     }
   }
@@ -624,10 +662,10 @@ bool GPUMemMan::IsResident(const Dataset* pDataset,
 
 /// Calculates the amount of memory the given brick will take up.
 /// Slightly complicated because we might have an N-dimensional brick.
-static UINT64
+static uint64_t
 required_cpu_memory(const Dataset& ds, const BrickKey& key)
 {
-  UINT64 mem = 1;
+  uint64_t mem = 1;
   const UINTVECTOR3 size = ds.GetBrickVoxelCounts(key);
   mem = size[0] * size[1] * size[2];
   mem *= ds.GetBitWidth()/8;
@@ -640,16 +678,18 @@ required_cpu_memory(const Dataset& ds, const BrickKey& key)
 static GLVolumeListIter
 find_closest_texture(GLVolumeList &lst, const UINTVECTOR3& vSize,
                      bool use_pot, bool downsample, bool disable_border,
-                     bool bEmulate3DWith2DStacks)
+                     bool bEmulate3DWith2DStacks,
+                     int iShareGroupID)
 {
-  UINT64 iTargetFrameCounter = UINT64_INVALID;
-  UINT64 iTargetIntraFrameCounter = UINT64_INVALID;
+  uint64_t iTargetFrameCounter = UINT64_INVALID;
+  uint64_t iTargetIntraFrameCounter = UINT64_INVALID;
 
   GLVolumeListIter iBestMatch = lst.end();
   for (GLVolumeListIter i=lst.begin(); i < lst.end(); ++i) {
     if ((*i)->BestMatch(vSize, use_pot, downsample, disable_border,
                         bEmulate3DWith2DStacks,
-                        iTargetIntraFrameCounter, iTargetFrameCounter)) {
+                        iTargetIntraFrameCounter, iTargetFrameCounter,
+                        iShareGroupID)) {
       iBestMatch = i;
     }
   }
@@ -667,16 +707,16 @@ find_closest_texture(GLVolumeList &lst, const UINTVECTOR3& vSize,
 // texes would be expensive.
 template <typename ForwIter> static ForwIter
 find_brick_with_usercount(ForwIter first, const ForwIter last,
-                          UINT32 user_count)
+                          int iShareGroupID, uint32_t user_count)
 {
-  while(first != last && (*first)->iUserCount != user_count) {
+  while(first != last && !((*first)->iUserCount == user_count && iShareGroupID == (*first)->GetShareGroupID())) {
     ++first;
   }
   return first;
 }
 
 // Gets rid of *all* unused bricks.  Returns the number of bricks it deleted.
-size_t GPUMemMan::DeleteUnusedBricks() {
+size_t GPUMemMan::DeleteUnusedBricks(int iShareGroupID) {
   size_t removed = 0;
   // This is a bit harsh.  erase() in the middle of a deque invalidates *all*
   // iterators.  So we repeatedly search for unused bricks, until our search
@@ -690,7 +730,7 @@ size_t GPUMemMan::DeleteUnusedBricks() {
     found = false;
     const GLVolumeListIter& iter = find_brick_with_usercount(
                                       m_vpTex3DList.begin(),
-                                      m_vpTex3DList.end(), 0
+                                      m_vpTex3DList.end(), 0, iShareGroupID
                                     );
     if(iter != m_vpTex3DList.end()) {
       ++removed;
@@ -704,17 +744,17 @@ size_t GPUMemMan::DeleteUnusedBricks() {
 }
 
 // We don't have enough CPU memory to load something.  Get rid of a brick.
-void GPUMemMan::DeleteArbitraryBrick() {
+void GPUMemMan::DeleteArbitraryBrick(int iShareGroupID) {
   assert(!m_vpTex3DList.empty());
 
   // Identify the least used brick.  The 128 is an arbitrary choice.  We want
   // it to be high enough to hit every conceivable number of users for a brick.
   // We don't want to use 2^32 though, because then the application would feel
   // like it hung if we had some other bug.
-  for(UINT32 in_use_by=0; in_use_by < 128; ++in_use_by) {
+  for(uint32_t in_use_by=0; in_use_by < 128; ++in_use_by) {
     const GLVolumeListIter& iter = find_brick_with_usercount(
                                       m_vpTex3DList.begin(),
-                                      m_vpTex3DList.end(), in_use_by
+                                      m_vpTex3DList.end(), iShareGroupID, in_use_by
                                     );
     if(iter != m_vpTex3DList.end()) {
       MESSAGE("  Deleting texture %d",
@@ -727,13 +767,67 @@ void GPUMemMan::DeleteArbitraryBrick() {
           "cannot make space for a new brick.");
 }
 
+void GPUMemMan::DeleteVolumePool(GLVolumePool** pool) {
+  delete  *pool;
+  *pool = NULL;
+}
+
+GLVolumePool* GPUMemMan::GetVolumePool(UVFDataset* dataSet, GLenum filter, int /* iShareGroupID */) {
+
+  const uint64_t iBitWidth  = dataSet->GetBitWidth();
+  const uint64_t iCompCount = dataSet->GetComponentCount();
+  const UINTVECTOR3 vMaxBS = UINTVECTOR3(dataSet->GetMaxUsedBrickSizes());
+
+  // Compute the pool size as a (almost) cubed texture that fits 
+  // into the user specified GPU mem, is a multiple of the bricksize
+  // and is no bigger than what OpenGL tells us is possible
+  GLint iMaxVolumeDims;
+  GL(glGetIntegerv(GL_MAX_3D_TEXTURE_SIZE_EXT, &iMaxVolumeDims));
+  const uint64_t iMaxGPUMem = Controller::Instance().SysInfo()->GetMaxUsableGPUMem()-m_iAllocatedGPUMemory;
+  const uint64_t iElemSize = iMaxGPUMem/(iCompCount*iBitWidth/8);
+  const uint32_t r3ElemSize = std::min(uint32_t(iMaxVolumeDims), uint32_t(pow(iElemSize, 1.0f/3.0f)));
+
+
+
+  // the max brick layout the GPU would let us create
+  // based on the max texture size an the available memory
+  const UINTVECTOR3 maxBricksForGPU( (r3ElemSize/vMaxBS.x)*vMaxBS.x,
+                                     (r3ElemSize/vMaxBS.y)*vMaxBS.y,
+                                     (r3ElemSize/vMaxBS.z)*vMaxBS.z);
+
+  const uint64_t iTotalBrickCount = dataSet->GetTotalBrickCount();
+
+  // the max brick layout required by the dataset
+  const uint64_t r3Bricks = uint64_t(pow(iTotalBrickCount, 1.0f/3.0f));
+  UINT64VECTOR3 maxBricksForDataset;
+  maxBricksForDataset.x = std::min(uint64_t(iMaxVolumeDims), vMaxBS.x*r3Bricks);
+  maxBricksForDataset.y = std::min(uint64_t(iMaxVolumeDims), vMaxBS.y*uint64_t(ceil(float(iTotalBrickCount) / ((maxBricksForDataset.x/vMaxBS.x) * (maxBricksForDataset.x/vMaxBS.x)))));
+  maxBricksForDataset.z = std::min(uint64_t(iMaxVolumeDims), vMaxBS.z*uint64_t(ceil(float(iTotalBrickCount) / ((maxBricksForDataset.x/vMaxBS.x) * (maxBricksForDataset.y/vMaxBS.y)))));
+
+  // now use the smaller of the two layouts, normally that
+  // would be the maxBricksForGPU but for small datasets that
+  // can be rendered entirely in-core we may need less space
+  const UINTVECTOR3 poolSize = (maxBricksForDataset.volume() < UINT64VECTOR3(maxBricksForGPU).volume())
+                                    ? UINTVECTOR3(maxBricksForDataset)
+                                    : maxBricksForGPU;
+  GLVolumePool* pPool = NULL;
+  try {
+    pPool = new GLVolumePool(poolSize, dataSet, filter);
+  } catch (tuvok::Exception const& e) {
+    pPool = NULL;
+    T_ERROR(e.what());
+  }
+  return pPool;
+}
+
 GLVolume* GPUMemMan::GetVolume(Dataset* pDataset, const BrickKey& key,
                                bool bUseOnlyPowerOfTwo,
                                bool bDownSampleTo8Bits,
                                bool bDisableBorder,
                                bool bEmulate3DWith2DStacks,
-                               UINT64 iIntraFrameCounter,
-                               UINT64 iFrameCounter) {
+                               uint64_t iIntraFrameCounter,
+                               uint64_t iFrameCounter,
+                               int iShareGroupID) {
   // It can occur that we can create the brick in CPU memory but OpenGL must
   // perform a texture copy to obtain the texture.  If that happens, we'll
   // delete any brick and then try again.
@@ -742,7 +836,8 @@ GLVolume* GPUMemMan::GetVolume(Dataset* pDataset, const BrickKey& key,
       return this->AllocOrGetVolume(pDataset, key,
                                     bUseOnlyPowerOfTwo, bDownSampleTo8Bits,
                                     bDisableBorder, bEmulate3DWith2DStacks,
-                                    iIntraFrameCounter, iFrameCounter);
+                                    iIntraFrameCounter, iFrameCounter,
+                                    iShareGroupID);
     } catch(OutOfMemory&) { // Texture allocation failed.
       // If texture allocation failed and we had no bricks loaded, then the
       // system must be extremely memory limited.  Make a note and then bail.
@@ -752,13 +847,13 @@ GLVolume* GPUMemMan::GetVolume(Dataset* pDataset, const BrickKey& key,
       }
       // Delete all bricks that aren't used.  If that ends up being nothing,
       // then we're pretty screwed.  Stupidly choose a brick in that case.
-      if(0 == DeleteUnusedBricks()) {
+      if(0 == DeleteUnusedBricks(iShareGroupID)) {
         WARNING("No bricks unused.  Falling back to "
                 "deleting bricks that ARE in use!");
         // Delete up to 4 bricks.  We want to delete multiple bricks here
         // because we'll temporarily need copies of the bricks in memory.
         for(size_t i=0; i < 4 && !m_vpTex3DList.empty(); ++i) {
-          DeleteArbitraryBrick();
+          DeleteArbitraryBrick(iShareGroupID);
         }
       }
     }
@@ -773,27 +868,28 @@ GLVolume* GPUMemMan::AllocOrGetVolume(Dataset* pDataset,
                                       bool bDownSampleTo8Bits,
                                       bool bDisableBorder,
                                       bool bEmulate3DWith2DStacks,
-                                      UINT64 iIntraFrameCounter,
-                                      UINT64 iFrameCounter) {
+                                      uint64_t iIntraFrameCounter,
+                                      uint64_t iFrameCounter,
+                                      int iShareGroupID) {
 
   for (GLVolumeListIter i = m_vpTex3DList.begin();
        i < m_vpTex3DList.end(); i++) {
     if ((*i)->Equals(pDataset, key, bUseOnlyPowerOfTwo,
                      bDownSampleTo8Bits, bDisableBorder,
-                     bEmulate3DWith2DStacks)) {
+                     bEmulate3DWith2DStacks, iShareGroupID)) {
       GL_CHECK();
       MESSAGE("Reusing 3D texture");
       return (*i)->Access(iIntraFrameCounter, iFrameCounter);
     }
   }
 
-  UINT64 iNeededCPUMemory = required_cpu_memory(*pDataset, key);
+  uint64_t iNeededCPUMemory = required_cpu_memory(*pDataset, key);
 
   /// @todo FIXME these keys are all wrong; we shouldn't be using N-dimensional
   /// data structures for the keys here.
   const UINTVECTOR3 sz = pDataset->GetBrickVoxelCounts(key);
-  const UINT64 iBitWidth = pDataset->GetBitWidth();
-  const UINT64 iCompCount = pDataset->GetComponentCount();
+  const uint64_t iBitWidth = pDataset->GetBitWidth();
+  const uint64_t iCompCount = pDataset->GetComponentCount();
 
   // for OpenGL we ignore the GPU memory load and let GL do the paging
   if (m_iAllocatedCPUMemory + iNeededCPUMemory >
@@ -805,19 +901,21 @@ GLVolume* GPUMemMan::AllocOrGetVolume(Dataset* pDataset,
     // search for best brick to replace with this brick
     UINTVECTOR3 vSize = pDataset->GetBrickVoxelCounts(key);
     GLVolumeListIter iBestMatch = find_closest_texture(m_vpTex3DList, vSize,
-                                                        bUseOnlyPowerOfTwo,
-                                                        bDownSampleTo8Bits,
-                                                        bDisableBorder,
-                                                        bEmulate3DWith2DStacks);
+                                                       bUseOnlyPowerOfTwo,
+                                                       bDownSampleTo8Bits,
+                                                       bDisableBorder,
+                                                       bEmulate3DWith2DStacks,
+                                                       iShareGroupID);
     if (iBestMatch != m_vpTex3DList.end()) {
       // found a suitable brick that can be replaced
       (*iBestMatch)->Replace(pDataset, key, bUseOnlyPowerOfTwo,
                              bDownSampleTo8Bits, bDisableBorder,
                              bEmulate3DWith2DStacks,
                              iIntraFrameCounter, iFrameCounter,
-                             m_vUploadHub);
+                             m_vUploadHub,
+                             iShareGroupID);
       (*iBestMatch)->iUserCount++;
-      return (*iBestMatch)->volumes[0];
+      return (*iBestMatch)->volume;
     } else {
       // We know the brick doesn't fit in memory, and we know there's no
       // existing texture which matches enough that we could overwrite it with
@@ -836,15 +934,18 @@ GLVolume* GPUMemMan::AllocOrGetVolume(Dataset* pDataset,
           return NULL;
         }
 
-        DeleteArbitraryBrick();
+        DeleteArbitraryBrick(iShareGroupID);
       }
     }
   }
 
-
-  MESSAGE("Creating new gl volume %u x %u x %u, "
-          "bitsize=%llu, componentcount=%llu",
-          sz[0], sz[1], sz[2], iBitWidth, iCompCount);
+  {
+    std::ostringstream newvol;
+    newvol << "Creating new GL volume " << sz[0] << " x " << sz[1] << " x "
+           << sz[2] << ", bitsize=" << iBitWidth << ", componentcount="
+           << iCompCount << ", interpolation=";
+    MESSAGE("%s", newvol.str().c_str());
+  }
 
   GLVolumeListElem* pNew3DTex = new GLVolumeListElem(pDataset, key,
                                                      bUseOnlyPowerOfTwo,
@@ -854,9 +955,10 @@ GLVolume* GPUMemMan::AllocOrGetVolume(Dataset* pDataset,
                                                      iIntraFrameCounter,
                                                      iFrameCounter,
                                                      m_MasterController,
-                                                     m_vUploadHub);
+                                                     m_vUploadHub,
+                                                     iShareGroupID);
 
-  if (pNew3DTex->volumes[0] == NULL) {
+  if (pNew3DTex->volume == NULL) {
     T_ERROR("Failed to create OpenGL resource for volume.");
     delete pNew3DTex;
     return NULL;
@@ -868,14 +970,16 @@ GLVolume* GPUMemMan::AllocOrGetVolume(Dataset* pDataset,
   m_iAllocatedCPUMemory += pNew3DTex->GetCPUSize();
 
   m_vpTex3DList.push_back(pNew3DTex);
-  return (*(m_vpTex3DList.end()-1))->volumes[0];
+  return (*(m_vpTex3DList.end()-1))->volume;
 }
 
 void GPUMemMan::Release3DTexture(GLVolume* pGLVolume) {
   for (size_t i = 0;i<m_vpTex3DList.size();i++) {
-    if (m_vpTex3DList[i]->volumes[0] == pGLVolume) {
+    if (m_vpTex3DList[i]->volume == pGLVolume) {
       if (m_vpTex3DList[i]->iUserCount > 0) {
-          m_vpTex3DList[i]->iUserCount--;
+        m_vpTex3DList[i]->iUserCount--;
+        MESSAGE("Decreased 3D texture use count to %u",
+                m_vpTex3DList[i]->iUserCount);
       } else {
         WARNING("Attempting to release a 3D volume that is not in use.");
       }
@@ -884,13 +988,15 @@ void GPUMemMan::Release3DTexture(GLVolume* pGLVolume) {
 }
 
 
-void GPUMemMan::Delete3DTexture(const GLVolumeListIter &tex) {
+void GPUMemMan::Delete3DTexture(const GLVolumeListIter& tex) {
   m_iAllocatedGPUMemory -= (*tex)->GetGPUSize();
   m_iAllocatedCPUMemory -= (*tex)->GetCPUSize();
 
   if((*tex)->iUserCount != 0) {
     WARNING("Freeing used GL volume!");
   }
+  MESSAGE("Deleting GL texture with use count %u",
+          static_cast<unsigned>((*tex)->iUserCount));
   delete *tex;
   m_vpTex3DList.erase(tex);
 }
@@ -901,28 +1007,32 @@ void GPUMemMan::Delete3DTexture(size_t iIndex) {
 
 // Functor to identify a texture that belongs to a particular dataset.
 struct DatasetTexture: std::binary_function<Dataset, GLVolumeListElem, bool> {
-  bool operator()(const GLVolumeListElem* tex, const Dataset* ds) const {
-    return tex->pDataset == ds;
+  bool operator()(const GLVolumeListElem* tex, std::pair<const Dataset*, int> dsinfo) const {
+    return tex->pDataset == dsinfo.first && tex->GetShareGroupID() == dsinfo.second;
   }
 };
 
-void GPUMemMan::FreeAssociatedTextures(Dataset* pDataset) {
+void GPUMemMan::FreeAssociatedTextures(Dataset* pDataset, int iShareGroupID) {
   // Don't use singleton; see destructor comments.
-  AbstrDebugOut &dbg = *(m_MasterController->DebugOut());
+  AbstrDebugOut& dbg = *(m_MasterController->DebugOut());
 
+  try {
+    const FileBackedDataset& fbd = dynamic_cast<const
+      FileBackedDataset&>(*pDataset);
+    dbg.Message(_func_, "Deleting textures associated with '%s' dataset.",
+                fbd.Filename().c_str());
+  } catch(const std::bad_cast&) {}
   while(1) { // exit condition is checked for and `break'd in the loop.
     const GLVolumeListIter& iter =
       std::find_if(m_vpTex3DList.begin(), m_vpTex3DList.end(),
-                   std::tr1::bind(DatasetTexture(), _1, pDataset));
+                   std::bind(DatasetTexture(), _1, make_pair(pDataset,iShareGroupID)) );
     if(iter == m_vpTex3DList.end()) {
       break;
     }
-    dbg.Message(_func_, "Deleting a gl volume");
     Delete3DTexture(iter);
   }
 }
 
-
 void GPUMemMan::MemSizesChanged() {
   if (m_iAllocatedCPUMemory > m_SystemInfo->GetMaxUsableCPUMem()) {
       /// \todo CPU free resources to match max mem requirements
@@ -936,24 +1046,23 @@ void GPUMemMan::MemSizesChanged() {
 
 GLFBOTex* GPUMemMan::GetFBO(GLenum minfilter, GLenum magfilter,
                             GLenum wrapmode, GLsizei width, GLsizei height,
-                            GLenum intformat, UINT32 iSizePerElement,
+                            GLenum intformat, GLenum format, GLenum type,
+                            int iShareGroupID,
                             bool bHaveDepth, int iNumBuffers) {
   MESSAGE("Creating new FBO of size %i x %i", int(width), int(height));
 
-  UINT64 m_iCPUMemEstimate = GLFBOTex::EstimateCPUSize(width, height,
-                                                       iSizePerElement,
-                                                       bHaveDepth, iNumBuffers);
+  uint64_t m_iCPUMemEstimate = GLFBOTex::EstimateCPUSize(width, height,
+                                                         bHaveDepth, iNumBuffers);
 
   // if we are running out of mem, kick out bricks to create room for the FBO
   while (m_iAllocatedCPUMemory + m_iCPUMemEstimate >
          m_SystemInfo->GetMaxUsableCPUMem() && m_vpTex3DList.size() > 0) {
-    MESSAGE("Not enough memory for FBO %i x %i (%ubit * %i), "
-            "paging out bricks ...", int(width), int(height),
-            iSizePerElement, iNumBuffers);
+    MESSAGE("Not enough memory for FBO %i x %i x %i, "
+            "paging out bricks ...", int(width), int(height), iNumBuffers);
 
     // search for best brick to replace with this brick
-    UINT64 iMinTargetFrameCounter;
-    UINT64 iMaxTargetIntraFrameCounter;
+    uint64_t iMinTargetFrameCounter;
+    uint64_t iMaxTargetIntraFrameCounter;
     (*m_vpTex3DList.begin())->GetCounters(iMaxTargetIntraFrameCounter,
                                           iMinTargetFrameCounter);
     size_t iIndex = 0;
@@ -961,8 +1070,8 @@ GLFBOTex* GPUMemMan::GetFBO(GLenum minfilter, GLenum magfilter,
 
     for (GLVolumeListIter i = m_vpTex3DList.begin()+1;
          i < m_vpTex3DList.end(); ++i) {
-      UINT64 iTargetFrameCounter = UINT64_INVALID;
-      UINT64 iTargetIntraFrameCounter = UINT64_INVALID;
+      uint64_t iTargetFrameCounter = UINT64_INVALID;
+      uint64_t iTargetIntraFrameCounter = UINT64_INVALID;
       (*i)->GetCounters(iTargetIntraFrameCounter, iTargetFrameCounter);
       iIndex++;
 
@@ -985,7 +1094,8 @@ GLFBOTex* GPUMemMan::GetFBO(GLenum minfilter, GLenum magfilter,
 
   FBOListElem* e = new FBOListElem(m_MasterController, minfilter, magfilter,
                                    wrapmode, width, height, intformat,
-                                   iSizePerElement, bHaveDepth, iNumBuffers);
+                                   format, type,
+                                   bHaveDepth, iNumBuffers, iShareGroupID);
 
   if(!e->pFBOTex->Valid()) {
     T_ERROR("FBO creation failed!");
@@ -1023,26 +1133,32 @@ void GPUMemMan::FreeFBO(GLFBOTex* pFBO) {
   WARNING("FBO to free not found.");
 }
 
-GLSLProgram* GPUMemMan::GetGLSLProgram(const std::vector<std::string>& vert,
-                                       const std::vector<std::string>& frag)
+struct deref_glsl : public std::binary_function<GLSLListElem*, GLSLListElem*,
+                                                bool> {
+  bool operator ()(const GLSLListElem* a, const GLSLListElem* b) const {
+    return *a == *b;
+  }
+};
+
+GLSLProgram* GPUMemMan::GetGLSLProgram(const ShaderDescriptor& sdesc,
+                                       int iShareGroupID)
 {
-  for (GLSLListIter i = m_vpGLSLList.begin();i<m_vpGLSLList.end();i++) {
-    if(vert.size() == (*i)->vertex.size() &&
-       frag.size() == (*i)->fragment.size() &&
-       std::equal(vert.begin(), vert.end(), (*i)->vertex.begin()) &&
-       std::equal(frag.begin(), frag.end(), (*i)->fragment.begin()))
-    {
-      MESSAGE("Reusing GLSL program.");
-      (*i)->iAccessCounter++;
-      return (*i)->pGLSLProgram;
-    }
+  GLSLListElem elem(m_MasterController, sdesc, iShareGroupID, false);
+  using namespace std::placeholders;
+  GLSLListIter i = std::find_if(m_vpGLSLList.begin(), m_vpGLSLList.end(),
+                                std::bind(deref_glsl(), _1, &elem));
+  if(i != m_vpGLSLList.end()) {
+    MESSAGE("Reusing GLSL program.");
+    (*i)->iAccessCounter++;
+    return (*i)->pGLSLProgram;
   }
 
   MESSAGE("Creating new GLSL program from %u-element VS and %u-element FS",
-          static_cast<unsigned>(vert.size()),
-          static_cast<unsigned>(frag.size()));
+          std::distance(sdesc.begin_vertex(), sdesc.end_vertex()),
+          std::distance(sdesc.begin_fragment(), sdesc.end_fragment()));
 
-  GLSLListElem* e = new GLSLListElem(m_MasterController, vert, frag);
+  GLSLListElem* e = new GLSLListElem(m_MasterController, sdesc,
+                                     iShareGroupID);
 
   if(e->pGLSLProgram == NULL) {
     T_ERROR("Failed to create program!");
@@ -1063,7 +1179,8 @@ void GPUMemMan::FreeGLSLProgram(GLSLProgram* pGLSLProgram) {
     if (m_vpGLSLList[i]->pGLSLProgram == pGLSLProgram) {
       m_vpGLSLList[i]->iAccessCounter--;
       if (m_vpGLSLList[i]->iAccessCounter == 0) {
-        MESSAGE("Freeing GLSL program");
+        MESSAGE("Freeing GLSL program %u",
+                static_cast<unsigned>(GLuint(*pGLSLProgram)));
         m_iAllocatedGPUMemory -= m_vpGLSLList[i]->pGLSLProgram->GetGPUSize();
         m_iAllocatedCPUMemory -= m_vpGLSLList[i]->pGLSLProgram->GetCPUSize();
 
@@ -1079,9 +1196,21 @@ void GPUMemMan::FreeGLSLProgram(GLSLProgram* pGLSLProgram) {
   WARNING("GLSL program to free not found.");
 }
 
-UINT64 GPUMemMan::GetCPUMem() const {return m_SystemInfo->GetCPUMemSize();}
-UINT64 GPUMemMan::GetGPUMem() const {return m_SystemInfo->GetGPUMemSize();}
-UINT32 GPUMemMan::GetBitWidthMem() const {
+uint64_t GPUMemMan::GetCPUMem() const {return m_SystemInfo->GetCPUMemSize();}
+uint64_t GPUMemMan::GetGPUMem() const {return m_SystemInfo->GetGPUMemSize();}
+uint32_t GPUMemMan::GetBitWidthMem() const {
   return m_SystemInfo->GetProgramBitWidth();
 }
-UINT32 GPUMemMan::GetNumCPUs() const {return m_SystemInfo->GetNumberOfCPUs();}
+uint32_t GPUMemMan::GetNumCPUs() const {return m_SystemInfo->GetNumberOfCPUs();}
+
+
+void GPUMemMan::RegisterLuaCommands() {
+  std::string id;
+  const std::string nm = "tuvok.gpu."; // namespace
+
+  id = m_pMemReg->registerFunction(this,&GPUMemMan::Changed1DTrans,
+                                   nm + "changed1DTrans", "", false);
+  id = m_pMemReg->registerFunction(this,&GPUMemMan::Changed2DTrans,
+                                   nm + "changed2DTrans", "", false);
+}
+
