@@ -34,16 +34,12 @@
   \date    August 2008
 */
 
-#ifdef _MSC_VER
-# include <functional>
-#else
-# include <tr1/functional>
-#endif
 #include <cstring>
+#include <functional>
 #include <new>
 #include <numeric>
 #include <typeinfo>
-#include <IO/IOManager.h>
+#include "IO/IOManager.h"
 #include "GPUMemManDataStructs.h"
 #include "Basics/MathTools.h"
 #include "Controller/Controller.h"
@@ -54,17 +50,17 @@
 #include "Renderer/GL/GLVolume2DTex.h"
 
 using namespace tuvok;
-using namespace std::tr1;
 
 GLVolumeListElem::GLVolumeListElem(Dataset* _pDataset, const BrickKey& key,
-                                    bool bIsPaddedToPowerOfTwo,
-                                    bool bIsDownsampledTo8Bits,
-                                    bool bDisableBorder,
-                                    bool bEmulate3DWith2DStacks,
-                                    UINT64 iIntraFrameCounter,
-                                    UINT64 iFrameCounter,
-                                    MasterController* pMasterController,
-                                    std::vector<unsigned char>& vUploadHub) :
+                                   bool bIsPaddedToPowerOfTwo,
+                                   bool bIsDownsampledTo8Bits,
+                                   bool bDisableBorder,
+                                   bool bEmulate3DWith2DStacks,
+                                   uint64_t iIntraFrameCounter,
+                                   uint64_t iFrameCounter,
+                                   MasterController* pMasterController,
+                                   std::vector<unsigned char>& vUploadHub,
+                                   int iShareGroupID) :
   pDataset(_pDataset),
   iUserCount(1),
   m_iIntraFrameCounter(iIntraFrameCounter),
@@ -75,11 +71,12 @@ GLVolumeListElem::GLVolumeListElem(Dataset* _pDataset, const BrickKey& key,
   m_bIsDownsampledTo8Bits(bIsDownsampledTo8Bits),
   m_bDisableBorder(bDisableBorder),
   m_bEmulate3DWith2DStacks(bEmulate3DWith2DStacks),
-  m_bUsingHub(false)
+  m_bUsingHub(false),
+  m_iShareGroupID(iShareGroupID)
 {
   // initialize the volumes to be null pointers.
-  std::fill(volumes.begin(), volumes.end(), static_cast<GLVolume*>(NULL));
-  if (!CreateTexture(vUploadHub) && volumes[0]) {
+  volume = NULL;
+  if (!CreateTexture(vUploadHub) && volume) {
     FreeTexture();
   }
 }
@@ -92,26 +89,28 @@ GLVolumeListElem::~GLVolumeListElem() {
 bool GLVolumeListElem::Equals(const Dataset* _pDataset, const BrickKey& key,
                               bool bIsPaddedToPowerOfTwo,
                               bool bIsDownsampledTo8Bits, bool bDisableBorder,
-                              bool bEmulate3DWith2DStacks)
+                              bool bEmulate3DWith2DStacks,
+                              int iShareGroupID)
 {
   if (_pDataset != pDataset ||
       m_Key != key ||
       m_bIsPaddedToPowerOfTwo != bIsPaddedToPowerOfTwo ||
       m_bIsDownsampledTo8Bits != bIsDownsampledTo8Bits ||
       m_bDisableBorder != bDisableBorder ||
-      m_bEmulate3DWith2DStacks != bEmulate3DWith2DStacks) {
+      m_bEmulate3DWith2DStacks != bEmulate3DWith2DStacks ||
+      m_iShareGroupID != iShareGroupID) {
     return false;
   }
 
   return true;
 }
 
-GLVolume* GLVolumeListElem::Access(UINT64& iIntraFrameCounter, UINT64& iFrameCounter) {
+GLVolume* GLVolumeListElem::Access(uint64_t& iIntraFrameCounter, uint64_t& iFrameCounter) {
   m_iIntraFrameCounter = iIntraFrameCounter;
   m_iFrameCounter = iFrameCounter;
   iUserCount++;
 
-  return volumes[0];
+  return volume;
 }
 
 bool GLVolumeListElem::BestMatch(const UINTVECTOR3& vDimension,
@@ -119,14 +118,16 @@ bool GLVolumeListElem::BestMatch(const UINTVECTOR3& vDimension,
                                  bool bIsDownsampledTo8Bits,
                                  bool bDisableBorder,
                                  bool bEmulate3DWith2DStacks,
-                                 UINT64& iIntraFrameCounter,
-                                 UINT64& iFrameCounter)
+                                 uint64_t& iIntraFrameCounter,
+                                 uint64_t& iFrameCounter,
+                                 int iShareGroupID)
 {
   if (!Match(vDimension) || iUserCount > 0
       || m_bIsPaddedToPowerOfTwo != bIsPaddedToPowerOfTwo
       || m_bIsDownsampledTo8Bits != bIsDownsampledTo8Bits
       || m_bDisableBorder != bDisableBorder
-      || m_bEmulate3DWith2DStacks != bEmulate3DWith2DStacks) {
+      || m_bEmulate3DWith2DStacks != bEmulate3DWith2DStacks
+      || m_iShareGroupID != iShareGroupID) {
     return false;
   }
 
@@ -170,7 +171,9 @@ namespace nonstd {
 
 size_t GLVolumeListElem::GetGPUSize() const
 {
-  // Apprently MSVC fixed this in 2010, but 2008 is broken.
+  return size_t(volume->GetGPUSize());
+/*
+  // Apparently MSVC fixed this in 2010, but 2008 is broken.
 #if defined(_MSC_VER) && _MSC_VER <= 1500
   size_t sz=0;
   for(size_t i=0; i < volumes.size(); ++i) {
@@ -180,30 +183,35 @@ size_t GLVolumeListElem::GetGPUSize() const
 #else
   return static_cast<size_t>(nonstd::accumulate(
     this->volumes.begin(), this->volumes.end(), 0,
-    std::tr1::mem_fn(&GLVolume::GetGPUSize))
+    std::mem_fn(&GLVolume::GetGPUSize))
   );
 #endif
+  */
 }
 size_t GLVolumeListElem::GetCPUSize() const
 {
-#if defined(_MSC_VER) && _MSC_VER <= 1500
+  return size_t(volume->GetCPUSize());
+  /*
+
+  #if defined(_MSC_VER) && _MSC_VER <= 1500
   size_t sz=0;
   for(size_t i=0; i < volumes.size(); ++i) {
     sz += size_t(volumes[i]->GetGPUSize());
   }
   return sz;
 #else
-  return static_cast<size_t>(nonstd::accumulate(
+  return static_cast<size_t>(nonstd::accumulate( 
     this->volumes.begin(), this->volumes.end(), 0,
-    std::tr1::mem_fn(&GLVolume::GetCPUSize))
+    std::mem_fn(&GLVolume::GetCPUSize))
   );
 #endif
+  */
 }
 
 
 
 bool GLVolumeListElem::Match(const UINTVECTOR3& vDimension) {
-  if(volumes.empty()) { return false; }
+  if(!volume) { return false; }
 
   const UINTVECTOR3 vSize = pDataset->GetBrickVoxelCounts(m_Key);
 
@@ -222,17 +230,29 @@ bool GLVolumeListElem::Replace(Dataset* _pDataset,
                                bool bIsDownsampledTo8Bits,
                                bool bDisableBorder,
                                bool bEmulate3DWith2DStacks,
-                               UINT64 iIntraFrameCounter,
-                               UINT64 iFrameCounter,
-                               std::vector<unsigned char>& vUploadHub) {
-  if(volumes.empty()) { return false; }
+                               uint64_t iIntraFrameCounter,
+                               uint64_t iFrameCounter,
+                               std::vector<unsigned char>& vUploadHub,
+                               int iShareGroupID) {
+  
+  if(!volume) { return false; }
 
   pDataset = _pDataset;
+#ifdef __clang__
+  BrickKey tmpKey = key;
+  m_Key = tmpKey;
+#else
   m_Key    = key;
+#endif
   m_bIsPaddedToPowerOfTwo  = bIsPaddedToPowerOfTwo;
   m_bIsDownsampledTo8Bits  = bIsDownsampledTo8Bits;
   m_bDisableBorder         = bDisableBorder;
   m_bEmulate3DWith2DStacks = bEmulate3DWith2DStacks;
+  assert(m_iShareGroupID == iShareGroupID);
+#ifdef NDEBUG
+  // avoid 'unused variable' warning.
+  (void)iShareGroupID;
+#endif
 
   m_iIntraFrameCounter = iIntraFrameCounter;
   m_iFrameCounter = iFrameCounter;
@@ -243,14 +263,23 @@ bool GLVolumeListElem::Replace(Dataset* _pDataset,
   }
   while (glGetError() != GL_NO_ERROR) {};  // clear gl error flags
 
-  std::pair<shared_ptr<unsigned char>, UINTVECTOR3> padded = PadData(
-    m_bUsingHub ? &vUploadHub.at(0) : &vData.at(0),
-    pDataset->GetBrickVoxelCounts(m_Key),
-    pDataset->GetBitWidth(),
-    pDataset->GetComponentCount()
-  );
+  const UINTVECTOR3 vSize = pDataset->GetBrickVoxelCounts(m_Key);
 
-  volumes[0]->SetData(padded.first.get());
+  if (!m_bIsPaddedToPowerOfTwo ||
+      (MathTools::IsPow2(uint32_t(vSize[0])) &&
+       MathTools::IsPow2(uint32_t(vSize[1])) &&
+       MathTools::IsPow2(uint32_t(vSize[2])))) {
+    volume->SetData(m_bUsingHub ? &vUploadHub.at(0) : &vData.at(0));
+  } else {
+    std::pair<std::shared_ptr<unsigned char>, UINTVECTOR3> padded = PadData(
+      m_bUsingHub ? &vUploadHub.at(0) : &vData.at(0),
+      pDataset->GetBrickVoxelCounts(m_Key),
+      pDataset->GetBitWidth(),
+      pDataset->GetComponentCount()
+    );
+
+    volume->SetData(padded.first.get());
+  }
 
   return GL_NO_ERROR==glGetError();
 }
@@ -258,13 +287,13 @@ bool GLVolumeListElem::Replace(Dataset* _pDataset,
 
 bool GLVolumeListElem::LoadData(std::vector<unsigned char>& vUploadHub) {
   const UINTVECTOR3 vSize = pDataset->GetBrickVoxelCounts(m_Key);
-  UINT64 iByteWidth  = pDataset->GetBitWidth()/8;
-  UINT64 iCompCount = pDataset->GetComponentCount();
+  uint64_t iByteWidth  = pDataset->GetBitWidth()/8;
+  uint64_t iCompCount = pDataset->GetComponentCount();
 
-  UINT64 iBrickSize = vSize[0]*vSize[1]*vSize[2]*iByteWidth * iCompCount;
+  uint64_t iBrickSize = vSize[0]*vSize[1]*vSize[2]*iByteWidth * iCompCount;
 
   if (!vUploadHub.empty() && iBrickSize <=
-      UINT64(m_pMasterController->IOMan()->GetIncoresize()*4)) {
+      uint64_t(m_pMasterController->IOMan()->GetIncoresize()*4)) {
     m_bUsingHub = true;
     return pDataset->GetBrick(m_Key, vUploadHub);
   } else {
@@ -278,28 +307,28 @@ void  GLVolumeListElem::FreeData() {
 
 static void DeleteArray(unsigned char* p) { delete[] p; }
 
-std::pair<shared_ptr<unsigned char>, UINTVECTOR3>
-GLVolumeListElem::PadData(unsigned char* pRawData, UINTVECTOR3 vSize, UINT64 iBitWidth,
-                          UINT64 iCompCount)
+std::pair<std::shared_ptr<unsigned char>, UINTVECTOR3>
+GLVolumeListElem::PadData(unsigned char* pRawData, UINTVECTOR3 vSize, uint64_t iBitWidth,
+                          uint64_t iCompCount)
 {
   // pad the data to a power of two
-  UINTVECTOR3 vPaddedSize(MathTools::NextPow2(UINT32(vSize[0])),
-                          MathTools::NextPow2(UINT32(vSize[1])),
-                          MathTools::NextPow2(UINT32(vSize[2])));
+  UINTVECTOR3 vPaddedSize(MathTools::NextPow2(uint32_t(vSize[0])),
+                          MathTools::NextPow2(uint32_t(vSize[1])),
+                          MathTools::NextPow2(uint32_t(vSize[2])));
   size_t iTarget = 0;
   size_t iSource = 0;
   size_t iElementSize = static_cast<size_t>(iBitWidth/8*iCompCount);
   size_t iRowSizeSource = vSize[0]*iElementSize;
   size_t iRowSizeTarget = vPaddedSize[0]*iElementSize;
 
-  shared_ptr<unsigned char> pPaddedData;
+  std::shared_ptr<unsigned char> pPaddedData;
   try {
-    pPaddedData = shared_ptr<unsigned char>(
+    pPaddedData = std::shared_ptr<unsigned char>(
       new unsigned char[iRowSizeTarget * vPaddedSize[1] * vPaddedSize[2]],
       DeleteArray
     );
   } catch(std::bad_alloc&) {
-    return std::make_pair(shared_ptr<unsigned char>(), vPaddedSize);
+    return std::make_pair(std::shared_ptr<unsigned char>(), vPaddedSize);
   }
   memset(pPaddedData.get(), 0, iRowSizeTarget*vPaddedSize[1]*vPaddedSize[2]);
 
@@ -347,7 +376,10 @@ bool GLVolumeListElem::CreateTexture(std::vector<unsigned char>& vUploadHub,
   if (bDeleteOldTexture) FreeTexture();
 
   if (vData.empty()) {
+    MESSAGE("Completely reloading brick");
     if (!LoadData(vUploadHub)) { return false; }
+  } else {
+    MESSAGE("Reusing CPU copy of brick data");
   }
 
   unsigned char* pRawData = (m_bUsingHub) ? &vUploadHub.at(0) : &vData.at(0);
@@ -356,8 +388,8 @@ bool GLVolumeListElem::CreateTexture(std::vector<unsigned char>& vUploadHub,
   const UINTVECTOR3 vSize = pDataset->GetBrickVoxelCounts(m_Key);
 
   bool bToggleEndian = !pDataset->IsSameEndianness();
-  UINT64 iBitWidth  = pDataset->GetBitWidth();
-  UINT64 iCompCount = pDataset->GetComponentCount();
+  uint64_t iBitWidth  = pDataset->GetBitWidth();
+  uint64_t iCompCount = pDataset->GetComponentCount();
 
   MESSAGE("%llu components of width %llu", iCompCount, iBitWidth);
 
@@ -368,6 +400,7 @@ bool GLVolumeListElem::CreateTexture(std::vector<unsigned char>& vUploadHub,
   if (m_bIsDownsampledTo8Bits && iBitWidth != 8) {
     // here we assume that data which is not 8 bit is 16 bit
     if (iBitWidth != 16) {
+      T_ERROR("Don't know how to handle %llu-bit data.", iBitWidth);
       FreeData();
       return false;
     }
@@ -405,11 +438,10 @@ bool GLVolumeListElem::CreateTexture(std::vector<unsigned char>& vUploadHub,
       if (bToggleEndian) {
         /// @todo BROKEN for N-dimensional data; we're assuming we only get 3D
         /// data here.
-        UINT64 iElemCount = vSize[0] * vSize[1] * vSize[2];
+        uint64_t iElemCount = vSize[0] * vSize[1] * vSize[2];
         short* pShorData = (short*)pRawData;
-        for (UINT64 i = 0;i<iCompCount*iElemCount;i++) {
-          EndianConvert::Swap<short>(pShorData+i);
-        }
+        std::transform(pShorData, pShorData+(iCompCount*iElemCount), pShorData,
+                       EndianConvert::Swap<short>);
       }
 
       switch (iCompCount) {
@@ -431,47 +463,46 @@ bool GLVolumeListElem::CreateTexture(std::vector<unsigned char>& vUploadHub,
     }
   }
 
+
   glGetError();
   if (!m_bIsPaddedToPowerOfTwo ||
-      (MathTools::IsPow2(UINT32(vSize[0])) &&
-       MathTools::IsPow2(UINT32(vSize[1])) &&
-       MathTools::IsPow2(UINT32(vSize[2])))) {
+      (MathTools::IsPow2(uint32_t(vSize[0])) &&
+       MathTools::IsPow2(uint32_t(vSize[1])) &&
+       MathTools::IsPow2(uint32_t(vSize[2])))) {
     GLenum clamp = m_bDisableBorder ? GL_CLAMP_TO_EDGE : GL_CLAMP;
 
     if (m_bEmulate3DWith2DStacks) {
-      volumes[0] = new GLVolume2DTex(UINT32(vSize[0]), UINT32(vSize[1]),
-                                     UINT32(vSize[2]),
+      volume = new GLVolume2DTex(uint32_t(vSize[0]), uint32_t(vSize[1]),
+                                     uint32_t(vSize[2]),
                                      glInternalformat, glFormat, glType,
-                                     UINT32(iBitWidth/8*iCompCount), pRawData,
+                                     pRawData,
                                      GL_LINEAR, GL_LINEAR,
                                      clamp, clamp, clamp);
     } else {
-      volumes[0] = new GLVolume3DTex(UINT32(vSize[0]), UINT32(vSize[1]),
-                                     UINT32(vSize[2]),
+      volume = new GLVolume3DTex(uint32_t(vSize[0]), uint32_t(vSize[1]),
+                                     uint32_t(vSize[2]),
                                      glInternalformat, glFormat, glType,
-                                     UINT32(iBitWidth/8*iCompCount), pRawData,
+                                     pRawData,
                                      GL_LINEAR, GL_LINEAR,
                                      clamp, clamp, clamp);
     }
   } else {
-    std::pair<shared_ptr<unsigned char>, UINTVECTOR3> padded =
+    std::pair<std::shared_ptr<unsigned char>, UINTVECTOR3> padded =
       PadData(pRawData, vSize, iBitWidth, iCompCount);
-    const shared_ptr<unsigned char>& pPaddedData = padded.first;
+    const std::shared_ptr<unsigned char>& pPaddedData = padded.first;
     const UINTVECTOR3& vPaddedSize = padded.second;
 
     if (m_bEmulate3DWith2DStacks) {
-      volumes[0] = new GLVolume2DTex(vPaddedSize[0], vPaddedSize[1], vPaddedSize[2],
+      volume = new GLVolume2DTex(vPaddedSize[0], vPaddedSize[1], vPaddedSize[2],
                                      glInternalformat, glFormat, glType,
-                                     UINT32(iBitWidth/8*iCompCount),
                                      pPaddedData.get(),
                                      GL_LINEAR, GL_LINEAR,
                                      m_bDisableBorder ? GL_CLAMP_TO_EDGE : GL_CLAMP,
                                      m_bDisableBorder ? GL_CLAMP_TO_EDGE : GL_CLAMP,
                                      m_bDisableBorder ? GL_CLAMP_TO_EDGE : GL_CLAMP);
     } else {
-      volumes[0] = new GLVolume3DTex(vPaddedSize[0], vPaddedSize[1], vPaddedSize[2],
+      volume = new GLVolume3DTex(vPaddedSize[0], vPaddedSize[1], vPaddedSize[2],
                                      glInternalformat, glFormat, glType,
-                                     UINT32(iBitWidth/8*iCompCount),
                                      pPaddedData.get(),
                                      GL_LINEAR, GL_LINEAR,
                                      m_bDisableBorder ? GL_CLAMP_TO_EDGE : GL_CLAMP,
@@ -480,14 +511,16 @@ bool GLVolumeListElem::CreateTexture(std::vector<unsigned char>& vUploadHub,
     }
   }
 
+  // In the OpenGL case we can release the data at this point as we
+  // let the OpenGL subsystem handle the CPU/GPU paging, i.e. we ignore
+  // the GPU usage
   FreeData();
   return GL_NO_ERROR==glGetError();
 }
 
 void GLVolumeListElem::FreeTexture() {
-  if (volumes[0]) {
-    volumes[0]->FreeGLResources();
-    delete volumes[0];
-    volumes[0] = NULL;
+  if (volume) {
+    delete volume;
+    volume = NULL;
   }
 }
