@@ -37,16 +37,7 @@
 #include "StdTuvokDefines.h"
 #include <algorithm>
 #include <cassert>
-#include <StdTuvokDefines.h>
-#if defined(_MSC_VER) ||                                               \
-    (defined(__GNUC__) && (__GNUC__ == 4 && __GNUC_MINOR__ <= 1)) ||   \
-    defined(DETECTED_OS_APPLE)
-// MS puts tr1 in standard headers.  Old gcc's and all gcc's on Apple are
-// broken, not including this part of tr1.
-# include <cmath>
-#else
-# include <tr1/cmath>
-#endif
+#include <cmath>
 #include <functional>
 #include <limits>
 #include "SBVRGeogen2D.h"
@@ -153,7 +144,7 @@ void SBVRGeogen2D::ComputeGeometryRezk() {
 
   VERTEX_FORMAT pfSliceVertex[4];
   float fDelta = GetDelta(iStack);
-  UINT32 iLayerCount = UINT32(floor(1.0f/fDelta));
+  uint32_t iLayerCount = uint32_t(floor(1.0f/fDelta));
   fDelta = 1.0f / float(iLayerCount-1);
   float fDepth = 0;
   if (bFlipStack) {
@@ -163,7 +154,7 @@ void SBVRGeogen2D::ComputeGeometryRezk() {
 
   switch (iStack) {
     case 0 :{
-              for (UINT32 x = 0;x<iLayerCount;x++) {
+              for (uint32_t x = 0;x<iLayerCount;x++) {
                 InterpolateVertices(m_pfBBOXVertex[1], m_pfBBOXVertex[0], 
                                     fDepth, pfSliceVertex[0]);
                 InterpolateVertices(m_pfBBOXVertex[2], m_pfBBOXVertex[3], 
@@ -186,7 +177,7 @@ void SBVRGeogen2D::ComputeGeometryRezk() {
             } break;
     case 1 :{
 
-              for (UINT32 y = 0;y<iLayerCount;y++) {
+              for (uint32_t y = 0;y<iLayerCount;y++) {
 
                 InterpolateVertices(m_pfBBOXVertex[0], m_pfBBOXVertex[4], 
                                     fDepth, pfSliceVertex[0]);
@@ -208,7 +199,7 @@ void SBVRGeogen2D::ComputeGeometryRezk() {
               }
             }  break;
     case 2 :{
-              for (UINT32 z = 0;z<iLayerCount;z++) {
+              for (uint32_t z = 0;z<iLayerCount;z++) {
                 InterpolateVertices(m_pfBBOXVertex[0], m_pfBBOXVertex[3],
                                     fDepth, pfSliceVertex[0]);
                 InterpolateVertices(m_pfBBOXVertex[1], m_pfBBOXVertex[2],
@@ -384,7 +375,7 @@ void SBVRGeogen2D::ComputeGeometryKrueger() {
 
   // if something of the x stack is visible
   if (fCosAngleX > fMinCos) {
-    UINT32 iLayerCount = UINT32(floor(1.0f/fDelta.x));
+    uint32_t iLayerCount = uint32_t(floor(1.0f/fDelta.x));
     fDelta.x = 1.0f / float(iLayerCount-1);
 
     // detemine if we a moving back to front or front to back
@@ -396,7 +387,7 @@ void SBVRGeogen2D::ComputeGeometryKrueger() {
     }
 
     // generate ALL stack quads
-    for (UINT32 x = 0;x<iLayerCount;x++) {
+    for (uint32_t x = 0;x<iLayerCount;x++) {
 
       InterpolateVertices(m_pfBBOXVertex[2], m_pfBBOXVertex[3],
                           a, pfSliceVertex[0]);
@@ -434,7 +425,7 @@ void SBVRGeogen2D::ComputeGeometryKrueger() {
   // if something of the y stack is visible
   if (fCosAngleY > fMinCos) {
 
-    UINT32 iLayerCount = UINT32(floor(1.0f/fDelta.y));
+    uint32_t iLayerCount = uint32_t(floor(1.0f/fDelta.y));
     fDelta.y = 1.0f / float(iLayerCount-1);
     float a = 0;
 
@@ -446,7 +437,7 @@ void SBVRGeogen2D::ComputeGeometryKrueger() {
     }
 
     // generate ALL stack quads
-    for (UINT32 y = 0;y<iLayerCount;y++) {
+    for (uint32_t y = 0;y<iLayerCount;y++) {
 
       InterpolateVertices(m_pfBBOXVertex[0], m_pfBBOXVertex[4],
                           a, pfSliceVertex[0]);
@@ -482,7 +473,7 @@ void SBVRGeogen2D::ComputeGeometryKrueger() {
 
   // if something of the z stack is visible
   if (fCosAngleZ > fMinCos) {
-    UINT32 iLayerCount = UINT32(floor(1.0f/fDelta.z));
+    uint32_t iLayerCount = uint32_t(floor(1.0f/fDelta.z));
     fDelta.z = 1.0f / float(iLayerCount-1);
     float a = 0;
 
@@ -494,7 +485,7 @@ void SBVRGeogen2D::ComputeGeometryKrueger() {
     }
 
    // generate ALL stack quads
-    for (UINT32 z = 0;z<iLayerCount;z++) {
+    for (uint32_t z = 0;z<iLayerCount;z++) {
       InterpolateVertices(m_pfBBOXVertex[3], m_pfBBOXVertex[0],
                           a, pfSliceVertex[0]);
       InterpolateVertices(m_pfBBOXVertex[2], m_pfBBOXVertex[1],
@@ -849,7 +840,7 @@ void SBVRGeogen2D::BuildStackQuads(
 
 
   // compute number of layers to cover depth
-  UINT32 iLayerCount = UINT32(floor(fDistScale/fDelta));
+  uint32_t iLayerCount = uint32_t(floor(fDistScale/fDelta));
   fDelta = 1.0f / float(iLayerCount-1);
 
   vSliceTriangles.resize(iLayerCount*6);
